/*
Java Hummer Project
File: MessageI.java
Date: 12-10-98
Author: Mike Russell
Revised: Jesse C. McConnell

Description: This file contains the tool interface. The tool interface is
             the program that the log tools, alert tools, and the hummer
       itself uses to pass messages into the hummingbird system.

 Copyright (C) 1997-1998 The HummingBird Project - hummer@cs.uidaho.edu

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
package edu.uidaho.hummer.client;

import java.io.*;
import java.util.Properties;
import java.net.Socket;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.security.GeneralSecurityException;
import edu.uidaho.hummer.util.HummerKeyStore;
import edu.uidaho.hummer.util.StaticMethods;
import edu.uidaho.hummer.util.UnloadedKeyStoreException;
import edu.uidaho.hummer.util.UnknownAliasException;
import edu.uidaho.hummer.util.socket.HummerServerSocket;
import edu.uidaho.hummer.util.socket.HummerClientSocket;
import edu.uidaho.hummer.util.socket.HummerSocketException;
import edu.uidaho.hummer.dataobject.*;

/**
This class contains the tool interface. The tool interface is the program
that the log tools, alert tools, and hummer itself uses to pass
messages into and around the hummer system.

@author Mike Russell, Jesse C. McConnell
*/
public class MessageI {

  /**
  System Properties object that when created contains the System.getProperties
  and gets the properties defined in Hummer.properties.
  */
  private Properties _constants = new Properties(System.getProperties());

  /**Needed for authentication*/
  private HummerKeyStore _key_store;

  /**Needed for authentication*/
  private PersonalConfig _personal;

  /**
  Port that the MessagingServer can be connected from.
  */
  private int _message_port;

  /**
  HummerSocket object used to send messages to the MessagingServer.  To
  open the connection use openMessageConnection, and to close the connection
  use closeMessageConnection.
  */
  HummerClientSocket _hummer_client_socket;

  /**
  This constructor initializes the _constants Hummer.properties information.
  */
  public MessageI(HummerKeyStore key_store) {
    String address = new String();

  //++This try and catch is needed for authentication
    try {
      address = InetAddress.getLocalHost().getHostAddress();
    }
    catch (UnknownHostException e) {
      System.err.println("MessageI: " + e);
    }
    //++Needed for authentication
    this._key_store = key_store;
    _personal = new PersonalConfig(address, null);

    try {
      _constants = StaticMethods.getHummerProperties();
      _message_port = (new Integer(_constants.getProperty("port.MESSAGE_PORT"))).intValue();
    }
    catch (NumberFormatException ne) {
      System.err.println("MessageI:sendToHummer:Bad port.MESSAGE_PORT property.");
    }
    catch (FileNotFoundException fnfe) {
      System.err.println("MessageI: Hummer.properties file not found.");
    }
    catch (IOException ioe) {
      System.err.println("MessageI: File Hummer.properties not valid.");
    }
  }


  /**
  This constructor initializes the _constants with the passed in parameter.

  @param constants A Properties Object that has been created from the 
  Hummer.properties file.
  */
  public MessageI(Properties constants) {
    _constants = constants;
  }



  /**
  Print the instructions for using MessageI from the command-line.
  */
  public void printInstruct() {

    System.out.println("usage: MessageI m \"<message text>\" " +
                       "[h <hostname>] [f <file>] [s <tool>]\n\n"+
		       "NOTE: hostname is for testing purposes only.");
    System.exit(0);
  }


  /**
  Parse the command line parameters and place the the data in the message
  object. This function also report errors in the command-line so the
  instructions can be printed.

  @param args[] The argument array read from the command-line.
  @param tmp_msg The empty Message object to fill.
  @return A true if the parameters parsed correctly, false otherwise.
  */
  public boolean parseParameters(String args[], Message tmp_msg) {

    int i = 0;

    while (i < args.length) {

      switch (args[i].charAt(0)) {

      // Copy the message over
      case 'm':
        tmp_msg.description = args[++i];
        i++;
        break;

      // Copy the host address over
      case 'h':
        tmp_msg.host_IP = args[++i];
	try {
	  tmp_msg.host_IP = InetAddress.getByName(tmp_msg.host_IP).
	    getHostAddress();
	}
	catch (UnknownHostException uhe) {
	  System.err.println("MessageI:parseParameters:" + uhe);
	  return false;
	}
        i++;
        break;

      // Copy the filename over
      case 'f':
        tmp_msg.filename = args[++i];
        i++;
        break;

      // Copy the tool name over
      case 's':
        tmp_msg.tool = args[++i];
        i++;
        break;

      // If it was something else print out an error
      default:
        System.err.println("MessageI:parseParameters:Unknown Option:" + args[i]);
        return false;
      }
    }

    return true;
  }


  /**
  Make sure the command line arguments are within some requirements.  If the 
  hostname was not specified on the command line then this method sets it to
  the localhost by calling InetAddress.getLocalHost().getHostAddress().  This 
  should be the standard practice.

  @param tmp_msg The Message object to validate some of the parameters of.
  @return A true if the parameters are good, false otherwise.
  */
  public boolean validateParameters(Message tmp_msg) {

    // Check to make sure the required fields were specified.

    if (tmp_msg.description.equals("")) {

      System.err.println("MessageI:validateParameters: Required Fields not filled.");
      return false;
    }
    //If a hostname was not specified, then use the local host. This is for 
    //testing,
    try {
      if (tmp_msg.host_IP.equals("")) {
	tmp_msg.host_IP = InetAddress.getLocalHost().getHostAddress();
      }
    }
    catch (UnknownHostException uhe) {
      System.err.println("MessageI:validateParameters:" + uhe);
      return false;
    }
    return true;
  }

  /**
  This function checks to make sure the date and time are valid. At this
  point this just checks to make sure they are not zero.

  @deprecated We no longer deal with Date and Time as values, but as 
  java.util.Date objects instantiated when the Message object is constructed.
  @param date String representing the Date.
  @return True if the date checks out, false otherwise.
  */
  public boolean checkDateTime(String date) {

    // The date/time must form an int
    Integer temp_date = new Integer(date);

    if (temp_date.intValue() < 1) {
      return true; //There is a valid date or time...
    }
    else {
      return false; //There is no valid date or time...
    }
  }

  /**
  Opens a HummerClientSocket to the MessagingServer.
  */
  public void openMessageConnection(String host_name) {
    Socket client_socket;

  //++This try and catch is needed for authentication
    try {

      client_socket = new Socket(InetAddress.getByName(host_name), _message_port);
      _hummer_client_socket = new HummerClientSocket(_personal, client_socket, _key_store);
      //    dcs.validation(InetAdress.getLocalHost().getHostAddress());
      //		this should be the same as below
      _hummer_client_socket.validation(null);
    }
    catch (IOException e) {
      System.err.println("ConfigI:closeServer: " + e);
    }
    catch (HummerSocketException e) {
      System.err.println("ConfigI:closeServer: " + e);
    }
    catch (GeneralSecurityException e) {
      System.err.println("ConfigI:closeServer: " + e);
    }
    catch (UnloadedKeyStoreException e) {
      System.err.println("ConfigI:closeServer: " + e);
    }
    catch (UnknownAliasException e) {
      System.err.println("ConfigI:closeServer: " + e);
    }
  }


  /**
  Closes the open HummerClientSocket connection previously made.
  */
  public void closeMessageConnection() {
    try {
      _hummer_client_socket.close();
    }
    catch(IOException e) {
      System.err.println("MessageI:closeMessageConnection: Could not close connectin " +
                         "to MessagingServer: " + e);
    }
  }

  /**
  Sends the given message to the MessagingServer.

  @param tmp_msg Message object that is to be serialized and sent out on the
  wire.
  @return Boolean value representing successful completion of this method call.
  */

  public boolean sendToHummer(Message tmp_msg) {

    //++Needed for authentication
    Response response = new Response();

    //++Needed for authentication
    response._message = tmp_msg;
    // End of protocol mesage, and close out connection
    //++This try and catch is needed for authentication
    try  {
      //Initialize Message Age
      tmp_msg.message_age = new Integer(0);
      _hummer_client_socket.writeResponseObject(response);
    }
    catch (UnknownHostException e) {
      System.err.println("MessageI:sendToHummer: " + e);
      return false;
    }
    catch (IOException e) {
      System.err.println("MessageI:sendToHummer:MessagingServer not accepting"+
			           " connections.");
      return false;
    }
    catch(edu.uidaho.hummer.util.socket.HummerSocketException e) {
      System.err.println("MessageI:sendToHummer: Error while sending information: " + e);
      return false;
    }
    return true;
  }

  /**
  This is the main function of the tool interface. It handles the input
  passed into it via command line parameters.

  @param args[] The command-line arguments.
  */

  public static void main(String args[]) {
    HummerKeyStore key_store = null;
    try {
      key_store = HummerKeyStore.getInstance();
      key_store.load(true, null);
    }
    catch(IOException e) {
      System.err.println("MessageI:main: Could not create/load HummerKeyStore: " + e);
      return;
    }
    catch(GeneralSecurityException e) {
      System.err.println("MessageI:main: Could not load HummerKeyStore: " + e);
      return;
    }
    catch(ClassNotFoundException e) {
      System.err.println("MessageI:main: Could not load HummerKeyStore: " + e);
      return;
    }
    Message tmp_msg = new Message();

    MessageI message_interface = new MessageI(key_store);

    if(args.length != 1) {

      if(!message_interface.parseParameters(args, tmp_msg)) {
        message_interface.printInstruct();
      }

      else if(!message_interface.validateParameters(tmp_msg)){
        message_interface.printInstruct();
      }
      message_interface.openMessageConnection(tmp_msg.host_IP);
      message_interface.sendToHummer(tmp_msg);
      message_interface.closeMessageConnection();
    }
    else {
      message_interface.printInstruct();        // Print the instructions
    }
  }
}