/*
Java Hummer Project
File: AliasInfo.java
Date: 4-21-99
Author: Chad Ward
Description:  Dataobject holding all the information about a given
              alias in a HummerKeystore.

 Copyright (C) 1997-1998 The Hummer Project - hummer@cs.uidaho.edu

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package edu.uidaho.hummer.dataobject;

import java.util.Hashtable;
import java.io.Serializable;
import java.security.Key;

public abstract class AliasInfo implements Serializable {

	/**
	Timestamp value representing the time that a given key set was
	created.  The exact meaning of this value may be dependant
	upon the concrete implementation, but for all implementations,
	-1 implies the current key set.  For default this value
	represents the time that the current signature key pair for the
	key store was created that the account knows.
	@serial
	*/
  protected long _backup_timetamp = -1;

	/**
	The type of account the alias is for.
	@serial
	*/
  protected int _type;

	/**
	The table of access values for this alias.
	@serial
	*/
  protected Hashtable _access_table;


	/**
	HummerCertificate granted by this alias for the owner of the
	HummerKeyStore.  HummerCertificates should only be accepted from
	alias types _SERVER_ALIAS_TYPE and _PEER_ALIAS_TYPE.
	@serial
	*/
  protected HummerCertificate _hummer_certificate = null;

	/**
	Array of keys for this account.  The types of keys and the number
	is protocol dependant.
	@serial
	*/
  protected Key[] _keys = null;

	/*
	Object for validating new accounts with a password,
	random timestamp value and a PublicKey
	@serial
	*/
  protected NewUserValidator _new_user_validator = null;

	/**
	Value for _SERVER_ALIAS_TYPE accounts where true
	means that HummerCertificates granted by this server will
	give holder access to the system.
	@serial
	*/
  protected boolean _certificate_honored = false;


	/**
	Type used for a user type alias
	@see AliasInfo#_SERVER_ALIAS_TYPE
	@see AliasInfo#_PEER_ALIAS_TYPE
	@serial
	 */
  public static final int _USER_ALIAS_TYPE = 0;

	/**
	Type used for a server type alias
	@see AliasInfo#_USER_ALIAS_TYPE
	@see AliasInfo#_PEER_ALIAS_TYPE
	@serial
	 */
  public static final int _SERVER_ALIAS_TYPE = 1;

	/**
	Type used for a peer type alias
	@see AliasInfo#_USER_ALIAS_TYPE
	@see AliasInfo#_SERVER_ALIAS_TYPE
	@serial
	 */
  public static final int _PEER_ALIAS_TYPE = 2;

	/**
	Index into _keys array for a PublicKey.	
	@serial
	*/
  public static final int _PUBLIC_KEY_TYPE = 0;

	/**
	Default constructor, initializing the alias type and
	access table. (_timestamp starts as -1)
	*/
  public AliasInfo(int type, Hashtable access_table) {
    this._type = type;
    if(access_table != null) {
      this._access_table = (Hashtable)access_table.clone();
    }
  }

	/**
	Returns the Key in the _key array at the given index.
	*/
  public final Key getKey(int key_type_index) { return _keys[key_type_index]; }

	/**
	Returns the alias type for this account.
	*/
  public final int getType() { return _type; }

	/**
	Returns the _backup_timetamp for this account.
	*/
  public final long getBackupTimestamp() { return _backup_timetamp; }

	/**
	Returns the access table for this account.
	*/
  public final Hashtable getAccessTable() { return _access_table; }

	/**
	Returns a value in the access table.
	*/
  public final boolean getAccessValue(String access_name) {
    return ((Boolean)_access_table.get(access_name)).booleanValue();
  }

	/**
	Returns the new user validator for this account.
	*/
  public final NewUserValidator getNewUserValidator() { return _new_user_validator; }

	/**
	A protocol depenendant method for reseting the alias.  The concrete
	AliasType class defining this method must set _new_user_validator reference
	to a null value, because following the call of this method,
	isNewAccount should return false.
	@param initializing_object Object used for initializing this account.  See
		specific protocol for details on the meaning of this object.
	@return True if the initialization was successful, false otherwise.
	*/
  public abstract boolean initializeAccount(Object initializing_object);

	/**
	Returns true if this account has not been initialized, false otherwise.
	An account is initialized if the _new_user_validator is null.
	*/
  public final boolean isNewAccount() { return _new_user_validator != null;  }

	/**
	Sets the _hummer_certificate object and all keys in the _keys array
	to null, and then creates a NewUserValidator object with the new pasword.
	*/
  public final void reset(byte[] new_password) {
    this._hummer_certificate = null;
    for(int i = 0; i < _keys.length; i++) {
      this._keys[i] = null;
    }
    this._new_user_validator = new NewUserValidator(new_password);
  }

	/**
	Returns the value of _certificate_honored.  If the account type is not
	_SERVER_ALIAS_TYPE then false is always returned.
	*/
  public final boolean isCertificateHonored() {
    if(_type != _SERVER_ALIAS_TYPE) {
      return false;
    }
    return _certificate_honored;
  }

	/**
	Returns the HummerCertifiate object for this account.
	*/
  public final HummerCertificate getHummerCertificate() { return _hummer_certificate; }

	/**
	Returns false if the alias type is _USER_ALIAS_TYPE or the HummerCertifiate
	for the alias is null.
	*/
  public final boolean holdingHummerCertificate() {
    if(this._type == AliasInfo._USER_ALIAS_TYPE) {
      return false;
    }
    return _hummer_certificate != null;
  }

	/**
	Sets the alias type for this account.
	*/
  public final void setType(int type) {  this._type = type; }

	/**
	Sets a new access table for this account.
	*/
  public final void setAccessTable(Hashtable access_table) { this._access_table = access_table; }

	/**
	Sets a given value in the access table for this account.
	*/
  public final void setAccessValue(String access_name, boolean value) {
    this._access_table.put(access_name, new Boolean(value));
  }

	/**
	Sets the timestamp for when a backup key for the HummerKeyStore
	was created (-1 means current key).
	*/
  public final void setBackupTimestamp(long backup_timetamp) {
    this._backup_timetamp = backup_timetamp;
  }

	/**
	Sets the HummerCertificate object for this account.
	*/
  public final void setHummerCertificate(HummerCertificate hummer_certificate) {
    if(_type != _USER_ALIAS_TYPE) {
      this._hummer_certificate = hummer_certificate;
    }
  }


	/**
	Sets the HummerCertificate for this account.
	*/
  public final void setCertificateHonored(boolean certificate_honored) {
    this._certificate_honored = certificate_honored;
  }

	/**
	Sets the _public_key object for this class.
	@param key_type_inex The type of key to update, see protocol for
		key types possible.
	@param key The new key to set for this alias
	@see AliasInfo#_PUBLIC_KEY_TYPE
	*/
  public final void setKey(int key_type_index, Key key) {
    this._keys[key_type_index] = key;
  }
}