/*
Java Hummer Project
File: HummerCertificate.java
Date: 1-30-99
Author: Chad Ward
Description:  General interface for Hummer communication on an internet socket.

 Copyright (C) 1997-1998 The Hummer Project - hummer@cs.uidaho.edu
   
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
     
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package edu.uidaho.hummer.dataobject;

import java.io.Serializable;
import java.io.IOException;
import java.util.Arrays;
import java.util.GregorianCalendar;
import java.util.Hashtable;
import java.lang.ClassNotFoundException;
import java.security.cert.CertificateException;
import java.security.PublicKey;
import java.security.PrivateKey;
import java.security.Signature;
import java.security.InvalidKeyException;
import java.security.SignatureException;
import java.security.NoSuchAlgorithmException;

import edu.uidaho.hummer.dataobject.HummerSignedObject;
import edu.uidaho.hummer.util.StaticMethods;

/**
A version of Certificates (not extending Sun's Certificate class) that holds
information for validation between systems.  HummerCertificates will also
hold information specifying the rights that have been granted to the holder
by the signer.
<h4>Class Usage and Flow:</h4>
~<ul>
  <li>Constructor creates a signed object holding a PublicKey, Hashtable of access rights,
      and an expiration date.
  <li>equals - Replaces the Object.equals method, because once a HummerCertificate has been
      serialized and deserialized the Object.equals method will always return false
  <li>"get" methods are used to retrieve the access table, expiration date, and PublicKey
  <li>isExpired - Returns true if the expiration date is before the current system time
  <li>isValid - Compares the expiration date on the certificate with the current time, and
      checks the signature on the certificate using a given PublicKey
</ul>
<h4>Usage Example</h4>
Create keys.
	<blockquote><b>
	KeyPair signer_key_pair = StaticMethods.generateKeyPair();
	KeyPair reciever_key_pair = StaticMethods.generateKeyPair();
 	</b></blockquote>

Create a Hashtable of access values for a reciever.
	<blockquote><b>
	Hashtable access_table = StaticMethods.getDefaultUserAccessTable();
 	</b></blockquote>

Create the expiration date for the HummerCertificate.  For this example, the
HummerCertificate will expire in 5 days.
	<blockquote><b>
	GregorianCalendar expiration_date = new GregorianCalendar(); <br>
	expiration_date.add(Calendar.DAY_OF_YEAR, 5);
 	</b></blockquote>


Create an instance of a HummerCertificate, using the values generated above.
	<blockquote><b>
	HummerCertificate hummer_certificate =
		<blockquote>
		new HummerCertificate(signer_key_pair.getPrivate(), SticMethods.getSignatureObject(),
                        reciever_key_pair.getPublic(), access_table, expiration_date);
		</blockquote>
	</b></blockquote>

The following code will return true, because the PublicKey of the signing KeyPair will be valid.
If this were called more than 5 days after creation of the HummerCertificate, then false would be returned.
	<blockquote><b>
	hummer_certificate.isValid(signer_key_pair.getPublic(), StaticMethods.getSignatureObject());
	</b></blockquote>

The following code will return false, because the wrong PublicKey is used to verify the HummerCertificate.
	<blockquote><b><pre>
	hummer_certificate.isValid(reciever_key_pair.getPublic(), StaticMethods.getSignatureObject());
	</pre></b></blockquote>

@author Chad Ward
*/
public class HummerCertificate implements Serializable {

	/**
	Holder of data, which is placed into a Vector, and then signed.
	<ul> The Following is a listing of the contents of this Vector.
		<li>	data[0] = PublicKey - PublicKey of the reciever of this certificate
		(not part of KeyPair in signing HummerCertificate)
		<li> data[1] = Hashtable - Table of Boolean values representing access rights. (Those rights
		recognized by server that is signing the HummerCertificate
		<li> data[2] = Calendar - Date that HummerCertificate will expire (equals date that singature key expires)
	</ul>
	@serial
	*/
  protected HummerSignedObject _signed_object = null;

	/**
	Default Constructor.  Creates a HummerSignedObject, signed
	with the algorithm given by <i>signature_type</i> using the PrivateKey
	<i>siganature_key</i>, containing the other parameters
        @param signature_object Object that will be used for signing the
                HummerCertificate's internal HummerSignedObject.  <b>
                This object must already be initialized for signing with correct
                PrivateKey.
	@param public_key PublicKey to be held within the HummerCertificate,
		usually PublicKey of reciever (signer is verifying this PublicKey)
	@param access_table Hashtable of access priveledges given by the signer
		to the reciever of HummerCertificate
	@param expiration_date Date that this HummerCertificate will
		expire (Should be expiration date of private_key)
	@throws CertificateException There was a problem while initializing the HummerSignedObject.
	*/
  public HummerCertificate(Signature signature_object, PublicKey public_key,
                         Hashtable access_table, GregorianCalendar expiration_date) 
         throws CertificateException {
    if(signature_object == null) {
      throw new NullPointerException("Signature signature_object cannot be null.");
    }
    if(public_key == null) {
      throw new NullPointerException("PublicKey public_key cannot be null.");
    }
    if(expiration_date == null) {
      throw new NullPointerException("GregorianCalendar expiration_date cannot be null.");
    }      
    Object[] data = new Object[3];
    data[0] = public_key;
    data[1] = access_table;
    data[2] = expiration_date;
    try {
      this._signed_object = new HummerSignedObject(data, signature_object);
    }
    catch(SignatureException e) {
      throw new CertificateException("HummerCertificate:constructor: " + e);
    }
    catch(IOException e) {
      throw new CertificateException("HummerCertificate:constructor: " + e);
    }
  }

	/**
	Compares this HummerCertificate against the specified HummerCertificate.
	The equals method given by Object does not work correctly because keys aren't the
	same after being serialized.
	@throws CertificateException There was a problem while retrieving the information
		within the HummerSignedObject, or there was a problem while verifying the HummerSignedObject.
	*/
  public boolean equals(HummerCertificate certificate, Signature verify_signature_object)
         throws CertificateException {
    boolean equal = true;
    equal = this.isValid(verify_signature_object) && certificate.isValid(verify_signature_object);

    if(this.getPublicKey() == null) {
      equal = equal && (certificate.getPublicKey() == null);
    }
    else {
      equal = equal && Arrays.equals(this.getPublicKey().getEncoded(),
                                     certificate.getPublicKey().getEncoded());
    }
    if(this.getAccessTable() == null) {
      equal = equal && (certificate.getAccessTable() == null);
    }
    else {
      equal = equal && this.getAccessTable().equals(certificate.getAccessTable());
    }
    if(this.getExpirationDate() == null) {
      equal = equal && (certificate.getExpirationDate() == null);
    }
    else {
      equal = equal && this.getExpirationDate().equals(certificate.getExpirationDate());
    }
    return equal;
  }
	/**
	Returns the PublicKey held by this HummerCertificate (HummerCertificate's owner's PublicKey)
	*/
  public PublicKey getPublicKey() {
    Object return_value = null;
    return_value = ((Object[])(_signed_object.getObject()))[0];
    return (PublicKey)(return_value);
  }

	/**
	Returns the access table granted by a server for the holder of this HummerCertificate.
	*/
  public Hashtable getAccessTable() {
    Object return_value = null;
    return_value = ((Object[])(_signed_object.getObject()))[1];
    return (Hashtable)(return_value);
  }

	/**
	Returns the date that this HummerCertificate is no longer valid.
	*/
  public GregorianCalendar getExpirationDate() {
    Object return_value = null;
    return_value = ((Object[])(_signed_object.getObject()))[2];
    return (GregorianCalendar)(return_value);
  }

	/**
	Returns true if this certificate is expired.
	*/
  public boolean isExpired() {
    return (new GregorianCalendar()).after(this.getExpirationDate());
  }

	/**
	Returns true if the signature on this certificate can be verified with
	the given PublicKey and the expiration date is not before the current system time.
	(key cannot be null)
	@param key PublicKey used to verify the HummerSignedObject in this HummerCertificate.
	*/
  public boolean isValid(Signature signature_object) throws CertificateException {
    if(this.isExpired()) {
      return false;
    }
    if(signature_object == null) {
      throw new NullPointerException("HummerCertificate:isValid: Cannot verify " +
                                     "Signature with null Signature object.");
    }
    boolean is_valid = false;
    try {
      is_valid = _signed_object.verify(signature_object);
    }
    catch(SignatureException e) {
      throw new CertificateException("HummerCertificate:isValid: " + e);
    }
    catch(IOException e) {
      throw new CertificateException("HummerCertificate:isValid: " + e);
    }
    return is_valid;
  }
}