/*
Java Hummer Project
File: HummerSignedObject.java
Date: 3-31-99
Author: Chad Ward
Description:  Replacement of Java SignedObject class, which assumes
              an initialized Signature object rather than doing
              the initialization internally.

 Copyright (C) 1997-1998 The Hummer Project - hummer@cs.uidaho.edu
   
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
     
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package edu.uidaho.hummer.dataobject;

import java.io.ObjectInputStream;
import java.io.ByteArrayInputStream;
import java.io.Serializable;
import java.security.Signature;
import java.security.SignatureException;
import edu.uidaho.hummer.util.StaticMethods;

/**
Replacement of Java SignedObject class, which assumes
an initialized Signature object rather than doing
the initialization internally.
@author Chad Ward
*/
public final class HummerSignedObject implements Serializable {

	/**
	Object contained within the SignedObject
	@serial
	*/
  private byte[] _object_bytes;

	/**
	Bytes returned after the signing has been done
	@serial
	*/
  private byte[] _signed_bytes;

	/**
	Algorithm used to sign the object
	@serial
	*/
  private String _algorithm;

	/**
	Signs the given Serializable object using the given Signature object.  If there
	is any data updated on the Signature object, it is cleared by a Signature.sign call.
	@param signature_object Signature object which has <b>already</b> been initilialized for signing
	@throws java.io.IOException There was a problem when serializing object to bytes.
	*/
  public HummerSignedObject(Object object, Signature signing_engine) throws SignatureException, java.io.IOException {
    signing_engine.sign();
    this._object_bytes = StaticMethods.getBytes(object);
    this._algorithm = signing_engine.getAlgorithm();
    signing_engine.update(StaticMethods.getBytes(object));
    this._signed_bytes = signing_engine.sign();
  }

	/**
	Returns the algorithm used for the signature
	*/
  public String getAlgorithm() { return this._algorithm; }

	/**
	Returns the object that was signed
	*/
  public Object getObject() {
    Object return_value = null;
try {
    ByteArrayInputStream byte_array_input = new ByteArrayInputStream(_object_bytes);
    ObjectInputStream data_input = new ObjectInputStream(byte_array_input);
    return_value = data_input.readObject();
}catch(Exception e) {}
    return return_value;
  }

	/**
	Returns that signature bytes
	*/
  public byte[] getSignature() { return this._signed_bytes; }

	/**
	Uses a Signature object initialized for verification to verify the bytes
	that were previously signed.
	@throws java.io.IOException There was a problem when serializing object to bytes.
	*/
  public boolean verify(Signature verification_engine) throws SignatureException, java.io.IOException {
    verification_engine.update(this._object_bytes);
    return verification_engine.verify(this._signed_bytes);
  }
}
