/*
Java Hummer Project
File: PersonalConfig.java
Date: 1-30-99
Author: Chad Ward
Description:  Holder for keys, dates created, username, and password.

 Copyright (C) 1997-1998 The Hummer Project - hummer@cs.uidaho.edu
   
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
     
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package edu.uidaho.hummer.dataobject;

import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.Vector;
import java.util.Arrays;
import java.io.Serializable;
import java.security.KeyPairGenerator;
import java.security.Key;
import java.security.KeyPair;
import java.security.NoSuchAlgorithmException;

/**
Holder for keys, dates created, username, and password.
<h4>Class Usage and Flow:</h4>
<ul>
  <li>Constructor creates a new username and password.  The
      password is copied into a local variable using System.arraycopy.
  <li>"get" methods are used to retrieve the username and password
  <li>isNewAccount returns true if password is null, false otherwise (null password means new account)
  <li>initializeAccount resets the account by filling the password with empty values
      and then sets it to null
</ul>
<h4>Usage Example</h4>
Create an instance of a PersonalConfig with username equal to "jwilson" and password equal to "password".
	<blockquote><b>
	PersonalConfig personal_config = new PersonalConfig("jwilson", "password".getBytes());
	</b></blockquote>

The following code will return true.
	<blockquote><b>
	personal_config.isNewAccount();
	</b></blockquote>

Once the password has become no longer needed, the account should be initialized.
	<blockquote><b>
	personal_config.initializeAccount();
	</b></blockquote>

Now that initializeAccount has been called, the following code will return true.
	<blockquote><b>
	personal_config.isNewAccount();
	</b></blockquote>

@author Chad Ward
*/
public class PersonalConfig implements Serializable {
	/**
	Configuration information for holder of this object
	@serial
	*/
  protected String _alias;

	/**
	Will be null if the account is not new, otherwise this will be used
	for validating to a server. (give null password if used for an account
	that has already been initialized on the server)
	@serial
	*/
  protected byte[] _password;

	/**
	Initializes PersonalConfig object.  If password is not null, then
	System.arraycopy is used to copy the contents of the given password
	into the class _password array.
	*/
  public PersonalConfig(String alias, byte[] password) {
    if(alias == null) {
      throw new NullPointerException("Alias cannot be null.");
    }
    this._alias = alias;
    if(password != null) {
      if(password.length > 0) {
        this._password = new byte[password.length];
        System.arraycopy(password, 0, this._password, 0, password.length);
      } else {
        this._password = null;
      }
    } else {
      this._password = null;
    }
  }

	/**
	Gets alias for this object.
	*/
  public String getAlias() { return _alias; }

	/**
	Gets password for this object.
	*/
  public byte[] getPassword() { return _password; }

	/**
	If _password is null, then returns true, otherwise false.
	*/
  public boolean isNewAccount() { return (_password != null); }

	/**
	Will initialize the PersonalConfig object once it is no longer for
	a new account (the account has been intialized). <Br>
	Fills _password array with blanks and then sets it to null.
	*/
  public void initializeAccount() {
    if(_password != null) {
      Arrays.fill(_password, (byte)' ');
      _password = null;
    }
  }
}
