/*
Java Hummer Project
File: ToolServer.java
Date: 4-14-99
Author: Chad Ward
Description:  Server to start and stop tools for the system.

 Copyright (C) 1997-1998 The Hummer Project - hummer@cs.uidaho.edu
   
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
     
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package edu.uidaho.hummer.server;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.BufferedInputStream;
import java.io.IOException;
import java.io.OptionalDataException;
import java.io.StreamCorruptedException;
import java.net.Socket;
import java.net.ServerSocket;
import java.net.UnknownHostException;
import java.security.KeyStoreException;

import java.util.Properties;
import java.util.Vector;
import java.util.Hashtable;
import java.util.Enumeration;

import edu.uidaho.hummer.util.HummerKeyStore;
import edu.uidaho.hummer.util.StaticMethods;
import edu.uidaho.hummer.tool.ToolGroup;

/**
Server to start and stop tools for the system.
@author Chad Ward
*/
public final class ToolServer implements Runnable {

	/**
	Thread group which all tool thread belong to.
	*/
  private ThreadGroup toolthreadgroup = new ThreadGroup("Tools Thread Group");    

	/**
	Port that the ToolServer should be waiting for connections on.
	*/
  private int _port;

	/**
	Information about the current state of messaging server.  This will
	usually hold error information.  If MessagingServer is in a normal state,
	this will be null.
	*/
  private String _status = null;

	/**
	True if the server is currently accepting connections from
	clients, and false otherwise. (this value will be true for
	one connection after setting to false because it is still
	pausing for a connection.)
	*/
  private boolean _running = false;

	/**
	HummerKeyStore used for all local Hummer servers on this machine.
	*/
  private HummerKeyStore _key_store;

	/**
	Thread used to wrap around this Runnable object.  This is done
	rather than having Messaging server extends Thread because once
	a thread completes the run() method it is considered dead...forever.
	So a new thread must be created each time that the server is started up
	again.
	*/
  private Thread _wrapper_thread;


	/**
	ToolGroup object representing the tools that are available on this system and
	the ability to start and stop those threads.
	*/
  private ToolGroup _tool_group;

	/**
	List of ToolHandler Threaded objects that are currently handling requests of users
	on the system.
	*/
  private Vector _handler_list = new Vector();

	/**
	Main method for starting the server from the command line.
	*/
  public static void main(String args[]){
    System.out.println("\n\n======================================================================\n" +
                       "Welcome to the Hummer system - Tool Server\n" +
                       " Copyright (C) 1997-1998 The Hummer " +
                       "Project - hummer@cs.uidaho.edu\n" +
                       "----------------------------------------------------------------------");
    ToolServer server = new ToolServer();
    server.setRunning(true);
  }

	/**
	Returns the current value of _status.
	*/
  public String getStatus() { return _status; }


  public ToolServer(HummerKeyStore key_store) {
    this._key_store = key_store;
    Properties constants = null;
    try {
      constants = StaticMethods.getHummerProperties();
    }
    catch(IOException e) {
      _status = "Error: " + e;
      System.err.println("Error while loading Hummer.properties: " + e);
      return;
    }
    _port = (new Integer(constants.getProperty("port.TOOL_PORT"))).intValue();
  }

  public ToolServer() {
    Properties constants = null;
    try {
      constants = StaticMethods.getHummerProperties();
    }
    catch(IOException e) {
      _status = "Error: " + e;
      System.err.println("Error while loading Hummer.properties: " + e);
      return;
    }
    _port = (new Integer(constants.getProperty("port.TOOL_PORT"))).intValue();
    if(_key_store == null) {
      System.out.print("ToolServer: Loading key store...");
      try {
        _key_store = HummerKeyStore.getInstance();
        _key_store.load(true, null);
      }
      catch(KeyStoreException e) {
        _status = "Error: " + e;
        System.err.println("ToolServer:constructor: Error while creating/loading " +
                           "HummerKeyStore: " + e);
        return;
      }
      catch(ClassNotFoundException e) {
        _status = "Error: " + e;
        System.err.println("ToolServer:constructor: Error while loading key store: " + e);
        return;
      }
      catch(StreamCorruptedException e) {
        _status = "Error: " + e;
        System.err.println("ToolServer:constructor: Error while loading key store: " + e);
        return;
      }
      catch(UnknownHostException e) {
        _status = "Error: " + e;
        System.err.println("ToolServer:constructor: Error while loading key store: " + e);
        return;
      }
      catch(FileNotFoundException e) {
        _status = "Error: " + e;
        System.err.println("ToolServer:constructor: Error while loading key store: " + e);
        return;
      }
      catch(OptionalDataException e) {
        _status = "Error: " + e;
        System.err.println("ToolServer:constructor: Error while loading key store: " + e);
        return;
      }
      catch(IOException e) {
        _status = "Error: " + e;
        System.err.println("ToolServer:constructor: Error while loading key store: " + e);
        return;
      }
      System.out.println("loaded.");
    }
  }

	/**
	Constructor taking the Hummer.properties file Properties object.
	*/
  public ToolServer(Properties constants) {
    _port = (new Integer(constants.getProperty("port.TOOL_PORT"))).intValue();
    if(_key_store == null) {
      System.out.print("ToolServer: Loading key store...");
      try {
        _key_store = HummerKeyStore.getInstance();
        _key_store.load(true, null);
      }
      catch(KeyStoreException e) {
        _status = "Error: " + e;
        System.err.println("ToolServer:constructor: Error while creating/loading " +
                           "HummerKeyStore: " + e);
        return;
      }
      catch(ClassNotFoundException e) {
        _status = "Error: " + e;
        System.err.println("ToolServer:constructor: Error while loading key store: " + e);
        return;
      }
      catch(StreamCorruptedException e) {
        _status = "Error: " + e;
        System.err.println("ToolServer:constructor: Error while loading key store: " + e);
        return;
      }
      catch(UnknownHostException e) {
        _status = "Error: " + e;
        System.err.println("ToolServer:constructor: Error while loading key store: " + e);
        return;
      }
      catch(FileNotFoundException e) {
        _status = "Error: " + e;
        System.err.println("ToolServer:constructor: Error while loading key store: " + e);
        return;
      }
      catch(OptionalDataException e) {
        _status = "Error: " + e;
        System.err.println("ToolServer:constructor: Error while loading key store: " + e);
        return;
      }
      catch(IOException e) {
        _status = "Error: " + e;
        System.err.println("ToolServer:constructor: Error while loading key store: " + e);
        return;
      }
      System.out.println("loaded.");
    }
  }

	/**
	Returns the value of _running.
	@see ToolServer#_running
	*/
  public boolean isRunning() {  return _running; }

	/**
	Sets the value of _running.  If the server is not currently
	running, then it is started (Thread.start is called start the server Thread)
	*/
  public void setRunning(boolean running) {
    if(!this._running && running) {
      this._running = running;
      _wrapper_thread = new Thread(this);
      _wrapper_thread.start();
    }
    this._running = running;
  }

	/**
	Method called for running the server
	thread.
	*/
  public void run() {
    if(_tool_group == null) {
      _tool_group = new ToolGroup(_key_store);
    }
    ServerSocket server_socket = null;
    try {
      server_socket = new ServerSocket(_port);
    }
    catch(IOException e) {
      _status = "Error: " + e;
    	 System.err.println("ToolServer:main: Error while attempting to create ServerSocket: " + e);
    	 return;
    }
    ToolHandler current_handler;
    Thread client_thread;
    while(_running) {
      try {
        for(int i = 0; i < _handler_list.size(); i++) {
   			/*
   			Do some clean up (remove client threads that have already died)
   			*/
          if(!((ToolHandler)_handler_list.get(i)).isRunning()) {
            _handler_list.remove(i);
          }
        }
        current_handler = new ToolHandler(_tool_group, _key_store, server_socket.accept());
        System.out.println("ToolServer: Connection recieved, will now start thread to handle client.");
        client_thread = new Thread(current_handler);
        _handler_list.add(current_handler);
      }
      catch(IOException e) {
        _status = "Error: " + e;
    	   System.err.println("ToolServer:main: Error while opening socket for client: " + e);
        return;
      }
      client_thread.start();
    }

    for(Enumeration enum = _handler_list.elements(); enum.hasMoreElements();) {
   			/*
   			Now that the server is shutting down, tell all of the client
   			handler threads to stop.
   			*/
      ((ToolHandler)enum.nextElement()).stop();
    }
    _handler_list.removeAllElements();

		/*
		Now that the ToolServer is shutting down, make sure that the
		tools are also stopped.
		*/
    _tool_group.setToolsRunning(false);
  }
}