/*
Java Hummer Project
File: Test.java
Date: 2-17-99
Author: Chad Ward
Description:  Parent class for all tests run.  Classes extending
              This class only have to define the testDefinition method,
              and then use the runTest method to perform the test.

 Copyright (C) 1997-1998 The Hummer Project - hummer@cs.uidaho.edu
   
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
     
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package edu.uidaho.hummer.test;

import java.io.FileOutputStream;
import java.io.PrintStream;
import java.util.Vector;
import java.lang.reflect.*;

/**
Parent class for all tests run.  Classes extending this class only have to 
define the testDefinition method.  To run the test use the runTest method.
<br>
<h4>Class Usage and Flow:</h4>
<ul>
  <li> <B>newMethod</B> - Used in runSimpleTest when a new method is being tested.
  <li> <B>getTestCount/getErrorCount</B> - Gets the statistics for the current test set
  <li> <B>getNextTestID</B> - Returns the id that should be used for the next test
  <li> <B>runClassTest</B> - Outputs headers, runs the tests defined in testDefinition, and then
                      gives a summary of results (displayTestResults)
  <li> <U><B>testDefinition</B></U> - Specific code for the testing of a class
  <li> <B>runSimpleTest</B> - The basic unit of testing, takes only a description of input, expected output,
                              and a boolean representing whether the test passed.  Keeps track of the test
                              count, error count and formats output.
  <li> <B>runConstructorTest</B> - Allows a test for the throwing of an exception within a constructor.
                            Takes the name of the test, the name of class being tested, and the parameters
                            to the constructor. Then it creates an instance of the class and catches any
                            exceptions that may be thrown.There are several instances of the same format 
                            allowing for multiple numbers of parameters to be passed.  These methods 
                            use the <i>java.lang.reflect</i> package.
  <li> <B>runExceptionTest</B> - Allows a test for the throwing of an exception.  Takes the name of the test,
                          the object being tested, and then uses a string representing the method name,
                          the exception that is being expected and the parameters being passed to the
                          method.  There are several instances of the same format allowing for multiple
                          numbers of parameters to be passed.  These methods use the <i>java.lang.reflect</i>
                          package.
  <li> <U><B>setClassTestName</B></U> - Must be defined.  Used to name the class that will be tested.
</ul>
<h4>Underlined classes should be defined in any classes that derive Test</h4>
@author Chad Ward
*/
public abstract class Test {

	/**
	True if a thread is currently running the runSimpleTest method, false otherwise.
	Is used because there may be two threads running tests for the same class.
	*/
  private static boolean _test_running = false;

	/**
	The name of the class that is being tested. Is set by naming the concrete test class with
	the same name as the class being tested, with "Test" concatenated to the end.  eg. MyClass->MyClassTest
	*/
  protected String _test_class_name;

	/**
	Number of tests performed for this test set (incremented in the runSimpleTest method).
	*/
  protected static int _test_count = 0;

	/**
	Number of encountered during this test set (incremented in the runSimpleTest method).
	*/
  protected static int _error_count = 0;

	/**
	Boolean value used when output should be directed to both standard out and error out
	*/
  protected static boolean _dual_output = false;

	/**
	Output stream for normal output messages
	*/
  protected static PrintStream _standard_out;

	/**
	Output stream for error output messages
	*/
  protected static PrintStream _error_out;

	/**
	Each test is given an id number, and they start with this value.
	*/
  protected static int _starting_id;

	/**
	Maximum number of characters that are allowed on one line
	(individual test descriptions or method descriptions).
	*/
  private final int _MAX_STRING_LENGTH = 65;

	/**
	Spacing of characters used for formatting of output. _indent1 < _indent2 < _indent3.
	*/
  protected final String _indent1 = "   ", _indent2 = _indent1 + _indent1, _indent3 = _indent1 + _indent2;

	/**
	Default constructor, which sets standard out (System.out) to _standard_out and _error_out to System.err.
	*/
  public Test() {
    _standard_out = System.out;
    _error_out = System.err;
    _dual_output = false;
  }

	/**
	Constructor for setting special output streams for standard messages and error messages.
	*/
  public Test(PrintStream standard_out, PrintStream error_out) {
    this._standard_out = standard_out;
    this._error_out = error_out;
    if(_standard_out != _error_out && !(_standard_out == System.out && _error_out == System.err)) {
        _dual_output = true;
    }
  }

	/**
	Displays the current test and error counts. Is automatically called after the test has been run.
	*/
  private void displayTestResults() {
    float ratio = ((float)_test_count - (float)_error_count) / (float)_test_count;
    int pre_success_rate = (int)(ratio * 1000);
    float sucess_rate = (float)pre_success_rate / (float)10.0;
    println(_indent1 + "Test count  -\t" + _test_count, true);
    println(_indent1 + "Error count -\t" + _error_count, true);
    println(_indent1 + "Success rate -\t" + sucess_rate + "%\n", true);
  }

	/**
	Attempts to replace any null positions with a correct representation needed to match parameters
	for a <b>Method</b>.
	*/
  private void fixNullPositions(Class test_class, String method_name, Class[] parameter_classes) 
          throws ClassNotFoundException {
    Method[] all_methods = test_class.getMethods();
    Class[] current_parameters;
    for(int i = 0; i < all_methods.length; i++) {
      if(all_methods[i].getName().equals(method_name)) {
        current_parameters = all_methods[i].getParameterTypes();
        if(current_parameters.length == parameter_classes.length) {
          for(int j = 0; j < parameter_classes.length; j++) {
            if(parameter_classes[j] == null) {
              parameter_classes[j] = current_parameters[j];
            }
          }
          break;
        }
      }
    }
    for(int i = 0; i < parameter_classes.length; i++) {
      if(parameter_classes[i] == null) {
        parameter_classes[i] = Class.forName("java.lang.Object");
      }
    }
  }

	/**
	Attempts to replace any null positions with a correct representation needed to match parameters
	for a <b>Constructor</b>.
	*/
  private void fixNullPositions(Class test_class, Class[] parameter_classes) throws ClassNotFoundException {
    Constructor[] all_constructors = test_class.getConstructors();
    Class[] current_parameters;
    for(int i = 0; i < all_constructors.length; i++) {
      current_parameters = all_constructors[i].getParameterTypes();
      if(current_parameters.length == parameter_classes.length) {
        for(int j = 0; j < parameter_classes.length; j++) {
          if(parameter_classes[j] == null) {
            parameter_classes[j] = current_parameters[j];
          }
        }
        break;
      }
    }
    for(int i = 0; i < parameter_classes.length; i++) {
      if(parameter_classes[i] == null) {
        parameter_classes[i] = Class.forName("java.lang.Object");
      }
    }
  }

	/**
	Method to replace System.out.println and System.err.println so that
	all output can be directed through a single method.
	*/
  protected static void println(String output, boolean print_both) {
    if(_dual_output && print_both) {
      _error_out.println(output);
    }
    _standard_out.println(output);
 }

	/**
	Method to replace System.err.println when an error has occured.
	*/
  protected static void errorPrintln(String output) {
    _error_out.println(output);
  }

	/**
	Method used to begin the testing.  Runs the setTestClassName method,
      outputs a header, runs the testDefinition method,
	and outputs the results (displayTestResults).
	*/
  public void runClassTest(int starting_id) throws Exception {
    this._starting_id = starting_id;
    _test_count = 0;
    _error_count = 0;
    this.setTestClassName();
    println(_indent1 + "Testing " + this._test_class_name, true);
    testDefinition();
    displayTestResults();
  }


  public static void runTestSuite(Vector test_suite) throws Exception {
    Test.runTestSuite(test_suite, System.out, System.err);
  }

	/**
	Takes a vector of Test classes and runs their runClassTest methods.
	Also does additional formatting and book keeping for all the classes.
	@param standard_out Stream used to output standard messages
	@param error_out Stream to output error messages and information on failed tests
	*/
  public static void runTestSuite(Vector test_suite, PrintStream standard_out, PrintStream error_out)
         throws Exception {
    String seperator = "\n------------------------------------------------------------------";
    standard_out.print(seperator + "\n\tTesting " + test_suite.size() + " class");
    if(standard_out != error_out && !(standard_out == System.out && error_out == System.err)) {
      error_out.print("Testing " + test_suite.size() + " class");
    }
    if(test_suite.size() > 1) {
      println("es", true);
    }
    int test_count = 0, error_count = 0, current_id = 1;
    Test current_test;
    current_test = (Test)(test_suite.get(0));
    for(int i = 0; i < test_suite.size(); i++) {
      println(seperator, true);
      current_test = (Test)(test_suite.get(i));
      current_test.runClassTest(current_id);
      test_count += current_test.getTestCount();
      error_count += current_test.getErrorCount();
      current_id = current_test.getNextTestID();
    }  
    float ratio = ((float)test_count - (float)error_count) / (float)test_count;
    int pre_success_rate = (int)(ratio * 1000);                              
    float sucess_rate = (float)pre_success_rate / (float)10.0;
    println(seperator + "\nNumber of tests made:\t" + test_count +
            "\nNumber of errors found:\t" + error_count +
            "\nSuccess rate:\t\t" + sucess_rate + "%", true);
  }

	/**
	Returns the number of tests that were performed (_test_count).
	*/
  protected int getTestCount() { return _test_count; }

	/**
	Returns the number of errors that were encountered (_error_count).
	*/
  protected int getErrorCount() { return _error_count; }

	/**
	Returns the _starting_id that should be used for the next test class.
	This is useful when running tests on several classes where a unique
	id for each test is needed.
	*/
  protected int getNextTestID() { return _starting_id + _test_count; }

	/**
	Converts any classes that have a primitive equivalent to that primitive
	Class type (eg. java.lang.Boolean=>boolean (Boolean.TYPE)).  This is sort
	of a hack and may present problems if the actual java.lang.* classes are needed
	*/
  private Class getPrimitive(Class class_object) {
    String class_name = class_object.getName();
    Class return_value = class_object;
    if(class_name.equals("java.lang.Boolean")) {
      return_value = Boolean.TYPE;
    }
    else if(class_name.equals("java.lang.Short")) {
      return_value = Short.TYPE;
    }
    else if(class_name.equals("java.lang.Integer")) {
      return_value = Integer.TYPE;
    }
    else if(class_name.equals("java.lang.Long")) {
      return_value = Long.TYPE;
    }
    else if(class_name.equals("java.lang.Float")) {
      return_value = Float.TYPE;
    }
    else if(class_name.equals("java.lang.Double")) {
      return_value = Double.TYPE;
    }
    else if(class_name.equals("java.lang.Byte")) {
      return_value = Byte.TYPE;
    }
    else if(class_name.equals("java.lang.Chararacter")) {
      return_value = Character.TYPE;
    }
    else if(class_name.equals("java.lang.Void")) {
      return_value = Void.TYPE;
    }
    return return_value;
  }

	/**
	Returns true if the given Class is a primitive, and false otherwise
	*/
  private Class getObjectClass(Class class_object) {
    String class_name = class_object.getName();
    Class return_value = class_object;
    if(class_name.equals("int")) {
      return_value = (new Integer(0)).getClass();
    }
    else if(class_name.equals("long")) {
      return_value = (new Long(0)).getClass();
    }
    else if(class_name.equals("float")) {
    	 return_value = (new Float(0)).getClass();
    }
    else if(class_name.equals("double")) {
      return_value = (new Double(0)).getClass();
    }
    else if(class_name.equals("boolean")) {
      return_value = (new Boolean(false)).getClass();
    }
    else if(class_name.equals("char")) {
       return_value = (new Character('c')).getClass();
    }
    else if(class_name.equals("byte")) {
       return_value = (new Byte((byte)0)).getClass();
    }
    else if(class_name.equals("short")) {
       return_value = (new Short((short)0)).getClass();
    }
    return return_value;
  }

	/**
	Similar to getMethod, except does a search through constructors for a match.
	This is needed beacause Sun's version does not recognize
	null values, and the NT port has some problems
	@see Test#getMethod(Object, String, Object[])
	*/
  private Constructor getConstructor(Class test_class, Object[] parameters)
          throws NoSuchMethodException, ClassNotFoundException {
    Constructor constructor = null;
    Class[] parameter_classes = new Class[parameters.length];
    Constructor[] constructors = test_class.getConstructors();
    boolean contains_null = false;

    for(int i = 0; i < parameters.length; i++) {
      if(parameters[i] != null) {
        parameter_classes[i] = this.getPrimitive(parameters[i].getClass());
      }
      else {
        contains_null = true;
        parameter_classes[i] = null;
      }
    }

    this.fixNullPositions(test_class, parameter_classes);
    Class[] current_constructor_parameters;
    for(int i = 0; i < constructors.length; i++) {
      current_constructor_parameters = constructors[i].getParameterTypes();
      if(parameter_classes.length != current_constructor_parameters.length) {
        continue;
      }
      boolean equal = true;
      for(int j = 0; j < current_constructor_parameters.length && equal; j++) {
        equal = isA(parameter_classes[j], current_constructor_parameters[j]);
      }
      if(equal) {
        return constructors[i];
      }
    }
    throw new NoSuchMethodException();
  }    

	/**
	Gets a Method object representing the method given by method_name with the return type given by
	expected_return_type in test_object.  This is needed beacause Sun's version does not recognize
	null values, and the NT port has some problems
	*/
  private Method getMethod(Object test_object, String method_name, Object[] parameters)
          throws NoSuchMethodException, ClassNotFoundException {
    Method return_value = null;
    Class[] parameter_classes = new Class[parameters.length];
    Method[] methods = test_object.getClass().getMethods();
    boolean contains_null = false;

    for(int i = 0; i < parameters.length; i++) {
      if(parameters[i] != null) {
        parameter_classes[i] = this.getPrimitive(parameters[i].getClass());
      }
      else {
        contains_null = true;
        parameter_classes[i] = null;
      }
    }

    this.fixNullPositions(test_object.getClass(), method_name, parameter_classes);
    Class[] current_method_parameters;
    for(int i = 0; i < methods.length; i++) {
      if(!methods[i].getName().equals(method_name)) {
        continue;
      }
      current_method_parameters = methods[i].getParameterTypes();
      if(parameter_classes.length != current_method_parameters.length) {
        continue;
      }
      boolean equal = true;
      for(int j = 0; j < current_method_parameters.length && equal; j++) {
        equal = isA(parameter_classes[j], current_method_parameters[j]);
      }
      if(equal) {
        return methods[i];
      }
    }
    throw new NoSuchMethodException();
  }

	/**
	Converts some problem classes to their normal usage
	(eg. sun.security.provider.DSAPublicKey=>java.security.PublicKey).
	A method to hack around some of the classes that present problems for NT when
	a method/constructor is found with Class.getMethod.  Solaris does not have this problem
	so as soon as NT gets it fixed, then this method should be removed.
	*/
  private void convertBadClasses(Class[] class_list) {
    if(class_list == null) {
      return;
    }
    String current_class_name;
    try {
      for(int i = 0; i < class_list.length; i++) {
        if(class_list[i] != null) {
          current_class_name = class_list[i].getName();
          if(current_class_name.equals("sun.security.provider.DSAPublicKey")) {
            class_list[i] = Class.forName("java.security.PublicKey");
          }
          else if(current_class_name.equals("sun.security.provider.DSA")) {
            class_list[i] = Class.forName("java.security.Signature");
          }
          else if(current_class_name.equals("sun.security.provider.DSAPrivateKey")) {
            class_list[i] = Class.forName("java.security.PrivateKey");
          }
        }
        else {
          errorPrintln("null value at: " + i + "  size:" + class_list.length);
        }
      }
    }
    catch(ClassNotFoundException e) {
      errorPrintln("Test:convertBadClasses:System does not contain correct packages for this class to work.");
    }
  }

	/**
	Returns true if the given class fits the is-a relationship for the
	target class/interface.
	*/
  private boolean isA(Class current_class, Class target_class) {
    if(current_class == null) {
      return true;
    }
    if(target_class == null) {
      return false;
    }
    if(current_class.getName().equals(target_class.getName())) {
      return true;
    }
    Class[] current_class_list = current_class.getInterfaces();
    if(current_class_list != null) {
      for(int i = 0; i < current_class_list.length; i++) {
        if(current_class_list[i] != null) {
          if(current_class_list[i].getName().equals(target_class.getName())) {
            return true;
          }
        }
      }
    }
    current_class = getObjectClass(current_class);
		/*
		Search through all of the subclasses and interfaces until
		Object is reached (Object is parent of all classes).  For interfaces
		the search is recursive, because interfaces have parents.
		*/
    while(current_class != null && !current_class.getName().equals("java.lang.Object")) {
      current_class = current_class.getSuperclass();
		/*
		Sometimes a null is returned by getSuperClass()
		*/
      if(current_class == null) {
        continue;
      }
		/*
		Return true if a match is made.
		*/
      if(current_class.getName().equals(target_class.getName())) {
        return true;
      }

		/*
		Search through all of the interfaces
		*/
      current_class_list = current_class.getInterfaces();
      for(int i = 0; i < current_class_list.length && target_class != null; i++) {
        if(current_class_list[i] == null) {
          continue;
        }
        if(current_class_list[i].getName().equals(target_class.getName())) {
          return true;
        }
		/*
		It is possible that a parent interface could match
		*/
        if(isA(current_class_list[i], target_class)) {
          return true;
        }
      }
    }
    return false;
  }

	/**
	Method that should be called whenever a new method will be tested.  Only
	serves for formatting the output, but makes the tests much more readable.
	*/
  protected void newMethod(String method_name) {
    String[] output_array = wrapText(method_name + " Method",
                                     _MAX_STRING_LENGTH - _indent2.length(),
                                     _indent2, _indent3 + _indent1);
    println("", true);
    for(int i = 0; i < output_array.length; i++) {
      println(output_array[i], true);
    }
  }

	/**
	Method that should be called whenever there is a method with several
	parts.  Allows the testing on the method to be better organized
	*/
  protected void note(String message) {
    String[] output_array = wrapText("Note: " + message,
                                     _MAX_STRING_LENGTH - _indent2.length(),
                                     _indent2, _indent3 + _indent1);
    println("", true);
    for(int i = 0; i < output_array.length; i++) {
      println(output_array[i], true);
    }
  }

	/**
	Tests for exceptions thrown when calling a constructor (return values are not possible).  This
	method is called by the other runConstructorTest methods.
	@param test_message Message that will be displayed for each test.
	@param expected_exception Exception that is expected to be thrown.  Should be either null if any
                                type of exception is expected.  Otherwise the full path of the exception.
	                          (eg. java.io.IOException)
	@param should_throw True if an exception is expected, false if not.
	@param parameters Parameters that will be passed to the constructor.
	@return Array of objects representing whether the test passed, an exception and the object created
		by the constructor
		return[0] - Boolean - passed. <br>
		return[1] - Exception - exception thrown. (null if none thrown) <br>
		return[2] - Object created by constructor (null if void return or exception thrown)
	*/
  private Object[] runConstructorTest(String input_message,
                                      String expected_exception, boolean should_throw, Object[] parameters)
            throws NoSuchMethodException, ClassNotFoundException {
    Exception exception_thrown = null;
    String this_class_name = this.getClass().getName();
    String test_class_name = this.getClass().getName();

		// Grab the package name up to ".test."
		// (eg. we're testing in edu.uidaho.hummer.test.dataobject.ResponeTest, so we grab edu.uidaho.hummer.)
    String test_class_package = test_class_name.substring(0, test_class_name.indexOf(".test."));
		// Now grab the individual class name (eg. Response)
    test_class_name = test_class_name.substring(test_class_name.lastIndexOf('.'), test_class_name.length() - 4);

		// Grab the rest of the package name and place it in front of the class name (eg. dataobject.)
    String package_substring = this_class_name.substring(test_class_package.length() + 5, this_class_name.lastIndexOf('.'));

		// put the rest of the pieces together
    test_class_name = test_class_package + package_substring + test_class_name;
    boolean passed = false, caught_exception = false;
    Object new_object = null;
    Class test_class = Class.forName(test_class_name);
    Constructor constructor;
    Method invoking_method;

	/*
	Try to get the constructor Class object representing the constructor given
	in the parameters
	*/
    try {
      constructor = this.getConstructor(test_class, parameters);
    }
    catch(NoSuchMethodException e) {  
      errorPrintln("Test:runConstructorTest: Could not find constructor");
      this.printPossibleConstructors(test_class);
      String exception_message = "Test:runConstructorTest(Object, String, String, boolean, Object[]):" +
                                 "\n   Called:\n\t" +
                                 test_class_name + "(";
      if(parameters != null && parameters.length > 0) {
        if(parameters[0] != null) {
          exception_message += parameters[0].getClass().getName();
          for(int i = 1; i < parameters.length; i++) {
            exception_message += ", " + parameters[i].getClass().getName();
          }
        }
        else {
          exception_message += "null";
        }
      }
      exception_message += ")";
      throw new NoSuchMethodException("\n\t" + exception_message + "\n" + e);
    }


    try {
      new_object = constructor.newInstance(parameters);
    } catch(Exception e) {
      if(e.getClass().getName().equals("java.lang.reflect.InvocationTargetException")) {
        e = (Exception)(((InvocationTargetException)e).getTargetException());
      }
      exception_thrown = e;
      if(!should_throw || expected_exception == null) {
        caught_exception = true;
      }
      else {
        if(e.getClass().getName().equals(expected_exception)) {
          caught_exception = true;
        }
      }
    }

    if((should_throw && caught_exception) ||
       (!should_throw && !caught_exception)) {
      passed = true;
    }
    else {
      passed = false;
    }
    String output_message = "";
    if(!should_throw) {
      if(expected_exception == null) {
        output_message = "No Exception thrown";
      }
      else {
        output_message = expected_exception + " was not thrown";
      }
    }
    else {
      if(expected_exception == null) {
        output_message = "Exception or any subclass of Exception is thrown";
      }
      else {
        output_message = expected_exception + " was thrown";
      }
    }
    this.runSimpleTest(input_message, output_message, passed);
    if(!passed && caught_exception) {
      println(_indent3 + _indent1 + exception_thrown + "\n", true);
    }
    Object[] return_value = new Object[3];
    return_value[0] = new Boolean(passed);
    return_value[1] = exception_thrown;
    return_value[2] = new_object;
    return return_value;
  }

	/**
	Tests for Constructor exceptions with no parameters.
	@see Test#runConstructorTest(String, String, boolean, Object[])
	*/
  protected Object[] runConstructorTest(String test_message,
                                      String expected_exception, boolean should_throw)
            throws NoSuchMethodException, ClassNotFoundException {
    return runConstructorTest(test_message, expected_exception, should_throw, new Object[0]);
  }

	/**
	Tests for Constructor exceptions with 1 parameter.
	@see Test#runConstructorTest(String, String, boolean, Object[])
	*/
  protected Object[] runConstructorTest(String test_message,
                                      String expected_exception, boolean should_throw,
                                      Object parameter1)
            throws NoSuchMethodException, ClassNotFoundException {
    Object[] parameters = new Object[1];
    parameters[0] = parameter1;
    return runConstructorTest(test_message, expected_exception, should_throw, parameters);
  }

	/**
	Tests for Constructor exceptions with 2 parameters.
	@see Test#runConstructorTest(String, String, boolean, Object[])
	*/
  protected Object[] runConstructorTest(String test_message,
                                      String expected_exception, boolean should_throw,
                                      Object parameter1, Object parameter2)
            throws NoSuchMethodException, ClassNotFoundException {
    Object[] parameters = new Object[2];
    parameters[0] = parameter1;
    parameters[1] = parameter2;
    return runConstructorTest(test_message, expected_exception, should_throw, parameters);
  }

	/**
	Tests for Constructor exceptions with 3 parameters.
	@see Test#runConstructorTest(String, String, boolean, Object[])
	*/
  protected Object[] runConstructorTest(String test_message,
                                      String expected_exception, boolean should_throw,
                                      Object parameter1, Object parameter2, Object parameter3)
            throws NoSuchMethodException, ClassNotFoundException {
    Object[] parameters = new Object[3];
    parameters[0] = parameter1;
    parameters[1] = parameter2;
    parameters[2] = parameter3;
    return runConstructorTest(test_message, expected_exception, should_throw, parameters);
  }

	/**
	Tests for Constructor exceptions with 4 parameters.
	@see Test#runConstructorTest(String, String, boolean, Object[])
	*/
  protected Object[] runConstructorTest(String test_message,
                                      String expected_exception, boolean should_throw,
                                      Object parameter1, Object parameter2, Object parameter3,
                                      Object parameter4)
            throws NoSuchMethodException, ClassNotFoundException {
    Object[] parameters = new Object[4];
    parameters[0] = parameter1;
    parameters[1] = parameter2;
    parameters[2] = parameter3;
    parameters[3] = parameter4;
    return runConstructorTest(test_message, expected_exception, should_throw, parameters);
  }

	/**
	Tests for Constructor exceptions with 5 parameters.
	@see Test#runConstructorTest(String, String, boolean, Object[])
	*/
  protected Object[] runConstructorTest(String test_message,
                                      String expected_exception, boolean should_throw,
                                      Object parameter1, Object parameter2, Object parameter3,
                                      Object parameter4, Object parameter5)
            throws NoSuchMethodException, ClassNotFoundException {
    Object[] parameters = new Object[5];
    parameters[0] = parameter1;
    parameters[1] = parameter2;
    parameters[2] = parameter3;
    parameters[3] = parameter4;
    parameters[4] = parameter5;
    return runConstructorTest(test_message, expected_exception, should_throw, parameters);
  }

	/**
	Tests for Constructor exceptions with 6 parameters.
	@see Test#runConstructorTest(String, String, boolean, Object[])
	*/
  protected Object[] runConstructorTest(String test_message,
                                      String expected_exception, boolean should_throw,
                                      Object parameter1, Object parameter2, Object parameter3,
                                      Object parameter4, Object parameter5, Object parameter6)
            throws NoSuchMethodException, ClassNotFoundException {
    Object[] parameters = new Object[6];
    parameters[0] = parameter1;
    parameters[1] = parameter2;
    parameters[2] = parameter3;
    parameters[3] = parameter4;
    parameters[4] = parameter5;
    parameters[5] = parameter6;
    return runConstructorTest(test_message, expected_exception, should_throw, parameters);
  }


	/**
	Method for calculating whether an exception has been passed.  This is called by the other
	runExceptionTest methods once the parameters have been determined.
	@return Whether the test passed and the exception that was thrown (null for no exception). <br>
		return[0] - Boolean - passed. <br>
		return[1] - Exception - exception thrown. <br>
		return[2] - Value returned when the method was invoked (null if void return or exception thrown)
	*/
  private Object[] runExceptionTest(String input_message, Object test_object, String method_name,
                                   String expected_exception, boolean should_throw,
                                   Object[] parameters) throws NoSuchMethodException, ClassNotFoundException {
    boolean passed = false, caught_exception = false;
    Exception exception_thrown = null;
    Object method_return_value = null;
    Method invoking_method;
    try {
      invoking_method = this.getMethod(test_object, method_name, parameters);
    }
    catch(NoSuchMethodException e) {
      errorPrintln("Test:runExceptionTest: Could not find method");
      this.printPossibleMethods(test_object.getClass(), method_name);
System.out.println("Test:runExceptionTest: Number of parameters: " + parameters.length);
      String exception_message = "Test:runExceptionTest(String, Object, String, String, boolean, Object[]):" +
                                 "\n   Called:\n\t" +
                                 method_name + "(";
      if(parameters != null && parameters.length > 0) {
        if(parameters[0] != null) {
          exception_message += parameters[0].getClass().getName();
          for(int i = 1; i < parameters.length; i++) {
            exception_message += ", " + parameters[i].getClass().getName();
          }
        }
        else {
          exception_message += "null";
        }
      }
      exception_message += ")";
      throw new NoSuchMethodException("\n\t" + exception_message + "\n" + e);
    }
    try {
      method_return_value = invoking_method.invoke(test_object, parameters);
    } catch(Exception e) {
      exception_thrown = e;
      if(exception_thrown.getClass().getName().equals("java.lang.reflect.InvocationTargetException")) {
        exception_thrown = (Exception)(((InvocationTargetException)e).getTargetException());
      }
      if(!should_throw || expected_exception == null) {
        caught_exception = true;
      }
      else {
        if(exception_thrown.getClass().getName().equals(expected_exception)) {
          caught_exception = true;
        }
      }
    }
    if((should_throw && caught_exception) ||
       (!should_throw && !caught_exception)) {
      passed = true;
    }
    else {
      passed = false;
    }
    Object[] return_value = new Object[3];
    return_value[0] = new Boolean(passed);
    return_value[1] = exception_thrown;
    return_value[2] = method_return_value;
    String output_message = "";
    if(!should_throw) {
      if(expected_exception == null) {
        output_message = "No Exception thrown";
      }
      else {
        output_message = expected_exception + " was not thrown";
      }
    }
    else {
      if(expected_exception == null) {
        output_message = "Exception or any subclass of Exception is thrown";
      }
      else {
        output_message = expected_exception + " was thrown";
      }
    }
    this.runSimpleTest(input_message, output_message, passed);
    if(!passed) {
      if(exception_thrown != null) {
        exception_thrown.printStackTrace(_standard_out);
        if(_dual_output) {
          exception_thrown.printStackTrace(_error_out);
        }
      }
      else {
        println(_indent3 + _indent1 + "No exception thrown.\n", false);
      } 
    }
    return return_value;
  }


	/**
	Basic test of a method for any exceptions being thrown (no parameters).  If there are not
	supposed to be any excpetions, then <i>null</i> should be used for expected_exception.
	@see Test#runExceptionTest(String,Object,String,String, boolean, Object[])
	*/
  protected Object[] runExceptionTest(String test_message, Object test_object,
                               String method_name, String expected_exception, boolean should_throw)
            throws NoSuchMethodException, InvocationTargetException, IllegalAccessException, ClassNotFoundException {
    Object[] parameters = new Object[0];
    return runExceptionTest(test_message, test_object, method_name, expected_exception, should_throw, parameters);
  }

	/**
	Test of a method for any exceptions being thrown with 1 parameter.
	@see Test#runExceptionTest(String,Object,String,String, boolean, Object[])
	*/
  protected Object[] runExceptionTest(String test_message, Object test_object,
                               String method_name, String expected_exception, boolean should_throw,
                               Object parameter1)
            throws NoSuchMethodException, InvocationTargetException, IllegalAccessException, ClassNotFoundException {
    Object[] parameters = new Object[1];
    parameters[0] = parameter1;
    return runExceptionTest(test_message, test_object, method_name, expected_exception, should_throw, parameters);
  }

	/**
	Test of a method for any exceptions being thrown with 2 parameters.
	@see Test#runExceptionTest(String,Object,String,String, boolean, Object[])
	*/
  protected Object[] runExceptionTest(String test_message, Object test_object,
                               String method_name, String expected_exception, boolean should_throw,
                               Object parameter1, Object parameter2)
            throws NoSuchMethodException, InvocationTargetException, IllegalAccessException, ClassNotFoundException {
    Object[] parameters = new Object[2];
    parameters[0] = parameter1;
    parameters[1] = parameter2;
    return runExceptionTest(test_message, test_object, method_name, expected_exception, should_throw, parameters);
  }

	/**
	Test of a method for any exceptions being thrown with 3 parameters.
	@see Test#runExceptionTest(String,Object,String,String, boolean, Object[])
	*/
  protected Object[] runExceptionTest(String test_message, Object test_object,
                               String method_name, String expected_exception, boolean should_throw,
                               Object parameter1, Object parameter2, Object parameter3)
            throws NoSuchMethodException, InvocationTargetException, IllegalAccessException, ClassNotFoundException {
    Object[] parameters = new Object[3];
    parameters[0] = parameter1;
    parameters[1] = parameter2;
    parameters[2] = parameter3;
    return runExceptionTest(test_message, test_object, method_name, expected_exception, should_throw, parameters);
  }

	/**
	Test of a method for any exceptions being thrown with 4 parameters.
	@see Test#runExceptionTest(String,Object,String,String, boolean, Object[])
	*/
  protected Object[] runExceptionTest(String test_message, Object test_object,
                               String method_name, String expected_exception, boolean should_throw,
                               Object parameter1, Object parameter2, Object parameter3,
                               Object parameter4)
            throws NoSuchMethodException, InvocationTargetException, IllegalAccessException, ClassNotFoundException {
    Object[] parameters = new Object[4];
    parameters[0] = parameter1;
    parameters[1] = parameter2;
    parameters[2] = parameter3;
    parameters[3] = parameter4;
    return runExceptionTest(test_message, test_object, method_name, expected_exception, should_throw, parameters);
  }

	/**
	Test of a method for any exceptions being thrown with 5 parameters.
	@see Test#runExceptionTest(String,Object,String,String, boolean, Object[])
	*/
  protected Object[] runExceptionTest(String test_message, Object test_object,
                               String method_name, String expected_exception, boolean should_throw,
                               Object parameter1, Object parameter2, Object parameter3,
                               Object parameter4, Object parameter5)
            throws NoSuchMethodException, InvocationTargetException, IllegalAccessException, ClassNotFoundException {
    Object[] parameters = new Object[5];
    parameters[0] = parameter1;
    parameters[1] = parameter2;
    parameters[2] = parameter3;
    parameters[3] = parameter4;
    parameters[4] = parameter5;
    return runExceptionTest(test_message, test_object, method_name, expected_exception, should_throw, parameters);
  }

	/**
	Test of a method for any exceptions being thrown with 6 parameters.
	@see Test#runExceptionTest(String,Object,String,String, boolean, Object[])
	*/
  protected Object[] runExceptionTest(String test_message, Object test_object,
                               String method_name, String expected_exception, boolean should_throw,
                               Object parameter1, Object parameter2, Object parameter3,
                               Object parameter4, Object parameter5, Object parameter6)
            throws NoSuchMethodException, InvocationTargetException, IllegalAccessException, ClassNotFoundException {
    Object[] parameters = new Object[6];
    parameters[0] = parameter1;
    parameters[1] = parameter2;
    parameters[2] = parameter3;
    parameters[3] = parameter4;
    parameters[4] = parameter5;
    parameters[5] = parameter6;
    return runExceptionTest(test_message, test_object, method_name, expected_exception, should_throw, parameters);
  }

	/**
	@deprecated Use runSimpleTest(String,String,boolean)
	*/
  protected boolean runSimpleTest(String input_message, boolean passed) {
    return runSimpleTest(input_message, "Deprecated method used", passed);
  }

	/**
	The most simple type of test that can be run.  It takes only a description of the input,
	a description of what is expected back, and whether it passed.
	Does not really do any sort of testing, but rather updates the state of the testing
	object and formats the output of the test message with results based on the boolean value.
	@param input_message A message describing the type of input is expected
	@param expected_output A message describing the type of output is expected
	@param passed True if the test should report passed, and false if it should report *FAILED*
	@return True if the test passed, false otherwise.  This is useful because the function
		can be run inside the parameters and the return given back through this method.
	*/
  protected boolean runSimpleTest(String input_message, String expected_output, boolean passed) {
		/*
		To eliminate the possibility of a race condition (up to two processes),
		if there is another process running in this method, then wait for it to return
		_test_running to false.
		*/
    String message_string = "";
    while(_test_running) {
      try {
        Thread.sleep(50);
      } catch(InterruptedException e) {}
    }
    _test_running = true;
    message_string += _indent3 + (_starting_id + _test_count) + " - Global Test ID\t" +
                      (_test_count + 1) + " - Class Test ID\n";
    String test_message = "Input: " + input_message;
    String[] message_array = wrapText(test_message, _MAX_STRING_LENGTH - 1, (_indent3 + _indent1),
                                      (_indent3 + _indent2));
    for(int i = 0; i < message_array.length; i++) {
      message_string += message_array[i] + "\n";
    }
    test_message = "Expected output: " + expected_output;
    message_array = wrapText(test_message, _MAX_STRING_LENGTH - 1, (_indent3 + _indent1),
                                      (_indent3 + _indent2));
    for(int i = 0; i < (message_array.length - 1); i++) {
      message_string += message_array[i] + "\n";
    }
		/*
		Set message_array equal to the last line of output.
		This is done because the passed/failed output must be put after, so it
		must be treated differently.
		*/
    test_message = message_array[message_array.length - 1];
    message_string += message_array[message_array.length - 1];
    int num_spaces = _MAX_STRING_LENGTH - test_message.length();
    if(num_spaces > 0) {
      for(int i = 0; i < num_spaces; i++) {
        message_string += " ";
      }
    } else {
      message_string += " ";
    }
    if(passed) {
      message_string += "passed.\n";
    } else {
      _error_count++;
      message_string += "*FAILED*\n";
    }
    _test_count++;
    _test_running = false;
    println(message_string, !passed);
    return passed;
  }

	/**
	Takes an input String and generates an array of strings representing the input
	String indented with the correct indentation (indent_first_line and indent_following).
	Each value in the array represents a line.
	@param text String to format
	@param max_length The maximum length that any line can be
	@param indent_first_line Amount to indent the first line
	@param indent_following Amount to indent the lines following the first line
	*/
  private String[] wrapText(String text, int max_length, String indent_first_line, String indent_following) {
		/*
		All lines are first put into a vector, and then placed into a new
		array of Strings before being returned
		*/
    Vector return_vector = new Vector();
    String[] return_value;
    String current_line = indent_first_line + text;
    if(current_line.length() > max_length) {
      int word_break = current_line.substring(0, max_length - 1).lastIndexOf(" ");
      if(word_break < 0 || word_break > max_length) {
        word_break = current_line.lastIndexOf(" ");
        if(word_break  < 0) {
          word_break = current_line.length() - 1;
        }
      }
      return_vector.add(current_line.substring(0, word_break));
      current_line = indent_following + current_line.substring(word_break + 1);
      while(current_line.length() > max_length) {
        String substring = current_line.substring(0, max_length - 1);
        word_break = current_line.substring(0, max_length - 1).lastIndexOf(" ");
        if(word_break < 0 || word_break >= max_length || word_break <= indent_following.length()) {
          word_break = current_line.lastIndexOf(" ");
          if(word_break < 0 || word_break <= indent_following.length()) {
            word_break = current_line.length() - 1;
          }
        }
        return_vector.add(current_line.substring(0, word_break));
        current_line = indent_following + current_line.substring(word_break + 1);
      }
    }
    return_vector.add(current_line);
    return_value = new String[return_vector.size()];
    for(int i = 0; i < return_vector.size(); i++) {
      return_value[i] = (String)(return_vector.get(i));
    }
    return return_value;
  }

	/**
	A class that sets the name of the class that will be tested (_test_class_name variable)
	*/
  protected abstract void setTestClassName();


	/**
	This method must be defined for a concrete instance, and defines what the test actually
	does.
	*/
  protected abstract void testDefinition() throws Exception;


	/**
	Prints out a message describing the possible constructors for a given
	class.
	*/
  private void printPossibleConstructors(Class test_class) {
    Constructor[] constructors = test_class.getConstructors();
    Class[] current_parameters;
    errorPrintln("Possible constructors for " + test_class.getName() + ":");
    if(constructors.length == 0) {
      errorPrintln("\t" + test_class.getName() + "()");
    }
    for(int i = 0; i < constructors.length; i++) {
      current_parameters = constructors[i].getParameterTypes();
      _error_out.print("\t" + test_class.getName() + "(");
      if(current_parameters.length > 0) {
        _error_out.print(current_parameters[0].getName());
        for(int j = 1; j < current_parameters.length; j++) {
          _error_out.print(", " + current_parameters[j].getName());
        }
      }
      errorPrintln(")");
    }
  }

  private void printPossibleMethods(Class test_class, String method_name) {
    Method[] methods = test_class.getMethods();
    Class[] current_parameters;
    errorPrintln("Possible parameters for " + test_class.getName() + "." + method_name + ":");
    if(methods.length == 0) {
      errorPrintln("\t" + test_class.getName() + "()");
    }
    for(int i = 0; i < methods.length; i++) {
      if(methods[i].getName().equals(method_name)) {
        current_parameters = methods[i].getParameterTypes();
        _error_out.print("\t" + methods[i].getName() + "(");
        if(current_parameters.length > 0) {
          _error_out.print(current_parameters[0].getName());
          for(int j = 1; j < current_parameters.length; j++) {
            _error_out.print(", " + current_parameters[j].getName());
          }
        }
        errorPrintln(")");
      }
    }
  }
}