/*
Java Hummer Project
File: HummerClientSocketTest.java
Date: 3-7-99
Author: Chad Ward
Description:  Tests HummerClientSocket class.

 Copyright (C) 1997-1998 The Hummer Project - hummer@cs.uidaho.edu
   
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
     
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package edu.uidaho.hummer.test.util.socket;

import java.security.*;
import java.util.*;
import java.io.*;
import java.net.*;
import edu.uidaho.hummer.dataobject.*;
import edu.uidaho.hummer.util.HummerKeyStore;
import edu.uidaho.hummer.util.DefaultKeyStore;
import edu.uidaho.hummer.util.StaticMethods;
import edu.uidaho.hummer.util.UnloadedKeyStoreException;
import edu.uidaho.hummer.util.UnknownAliasException;
import edu.uidaho.hummer.util.socket.HummerServerSocket;
import edu.uidaho.hummer.util.socket.HummerClientSocket;
import edu.uidaho.hummer.util.socket.HummerSocketException;
import edu.uidaho.hummer.test.Test;

/**
Description:  Tests HummerServerSocket class. <Br>
Most of the members of this class are static because there will be a thread
for the server and a thread for the client, but the information between the two
must be the same (socket information and key store information primarily). <br>
	The order of testing for validation method is:
	<ol type="I">
	  <li> Types of Clients
	  <ol type="1">
	    <li> Remote Account w/Untrusted (invalid) certificate
	    <li> Remote Account w/Trusted (valid) certificate
	    <li> Local Account w/HummerCertificate holding the current version of client's PublicKey
	    <li> Local Account w/Hummercertificate holding an outdated version of client's PublicKey
	  </ol>
	  <li> Types of Server Responses
	  <ol type="1">
	    <li> Response is null
	    <li> Type is just _EXCEPTION
	    <li> Type is _EXCEPTION masked with some other description
	    <li> _message does not equal _id sent to server
	    <li> _data is null
	    <li> _data size < 2
	    <li> _data size > 2 (remote account)
	    <li> _data size > 3 (local account)
	    <li> _data[0] is null
	    <li> _data[1] is null
	    <li> _data[2] is null (local account)
	    <li> Invalid protocol
	    <li> HummerCertificate (_data[1]) is expired
	    <li> HummerCertificate (_data[1]) is not valid (verified with server's PublicKey)
	    <li> HummerCertificate (_data[1]) is valid (new PublicKey held)
	    <li> HummerCertificate (_data[1]) is valid (same PublicKey)
	    <li> New HummerCertificate (_data[2]) is expired
	    <li> New HummerCertificate (_data[2]) not valid (verfied with PublicKey from _data[1])
	    <li> Valid information recieved
	  </ol>
	</ol>
	The order of testing for newAccountValidation is:
	<ol type="1">
	  <li> Response is null
	  <li> _message does not equal _id sent to server
	  <li> _data is null
	  <li> _data size < 5 (should be exactly 5)
	  <li> _data size > 5 (should be exactly 5)
	  <li> _data[0] is null (protocol)
	  <li> _data[1] is null (server PublicKey)
	  <li> _data[2] is null (client HummerCertificate)
	  <li> _data[3] is null (message_digest)
	  <li> _data[4] is null (random value and timestamp)
	  <li> Invalid protocol (_data[0])
	  <li> Invalid message digest (_data[3] - generated with client passsword
	                               (PersonalConfig), _data[1] and _data[4])
	  <li> Expired client HummerCertificate (_data[2])
	  <li> Invalid client HummerCertificate (_data[2])
	  <li> Valid data
	</ul>
	The order of testing for readResponseObject is:
	<ol type="1">
	  <li> Invalid signature.
	  <li> Valid signature. Wrong _id.
	  <li> Valid signature. _type is invalid 
	  <li> Valid signature. _type is valid (no commands)
	</ol>
	@see edu.uidaho.hummer.util.HummerServerSocket
	@author Chad Ward
*/
public class HummerClientSocketTest extends Test implements Runnable {
	/**
	A common InetAddress that will be used for communication during this test.
	It is static because multiple instances of this class will be created.
	*/
  protected static InetAddress _inet_address;
	/**
	A common port number that will be used for communication during this test.
	It is static because multiple instances of this class will be created.
	*/
  protected static int _port_number;

	/**
	A common set of signature keys will be required to do any sort of testsing,
	so this object must be static.
	*/
  protected static DefaultKeyStore _key_store;

	/**
	During <b>integration testing</b>, there will need to be 2 key stores, one for the
	server and one for the client.
	*/
  protected static DefaultKeyStore _server_key_store, _client_key_store;

	/**
	Hashtable of access rights for an alias.
	*/
  protected static Hashtable _local_user_access_table = new Hashtable(),
                             _new_local_user_access_table = new Hashtable(),
                             _remote_user_access_table = new Hashtable(),
                             _untrusted_server_access_table = new Hashtable(),
                             _uninitialized_server_access_table = new Hashtable(),
                             _trusted_server_access_table = new Hashtable();

	/**
	Signature KeyPair
	*/
  protected static KeyPair _local_user_key_pair, _new_local_user_key_pair, _remote_user_key_pair,
                           _untrusted_server_key_pair, _trusted_server_key_pair,
                           _key_store_key_pair1, _key_store_key_pair2, _local_server_key_pair;

	/**
	Signature object used for signing or verifying data.
	*/
  protected static Signature _trusted_server_verify, _trusted_server_sign,
                             _untrusted_server_verify, _untrusted_server_sign,
                             _local_server_sign, _local_server_verify;

	/**
	Certificate granted by a local server alias for local_user.
	*/
  protected static HummerCertificate _local_user_certificate;

	/**
	Certificate granted by a trusted server alias for remote_user.
	*/
  protected static HummerCertificate _trusted_remote_user_certificate;

	/**
	Certificate granted by an untrusted server alias for remote_user.
	*/
  protected static HummerCertificate _untrusted_remote_user_certificate;

	/**
	Certificate granted for local user by local server
	*/
  protected static HummerCertificate _local_server_certificate;

	/**
	Calendar for expiration dates on HummerCertificates
	*/
  protected static GregorianCalendar _future_expiration_date = new GregorianCalendar(),
                                     _past_expiration_date = new GregorianCalendar();

	/**
	Communication object used for integration testing between HummerClientSocket and HummerServerSocket
	*/
  protected static Response _client_message, _server_message;

	/**
	String representing the password
	*/
  protected final static PersonalConfig 
                   _new_local_user_config = new PersonalConfig("new_local_user", "new_local_user_pass".getBytes()),
                   _untrusted_server_config = new PersonalConfig("untrusted_server", null),
                   _uninitialized_server_config = new PersonalConfig("uninitialized_server", null),
                   _trusted_server_config = new PersonalConfig("trusted_server", null),
                   _local_user_config = new PersonalConfig("local_user", null),
                   _remote_user_config = new PersonalConfig("remote_user", null);

	/**
	String representing the password for this account
	*/
  protected final byte[] _local_user_pass = "local_user_pass".getBytes(),
                         _remote_user_pass = "remote_user_pass".getBytes(),
                         _trusted_server_pass = "trusted_server_pass".getBytes(),
                         _untrusted_server_pass = "untrusted_server_pass".getBytes(),
                         _uninitialized_server_pass = "uninitialized_server_pass".getBytes();

	/**
	Allows testing for HummerClientSocket to be run from the command line.
	If arguments are sent, the first is a filename for standard out, and the second is
	the filename for error out (one, two or no parameters will work).
	*/
  public static void main(String[] argv) throws Exception {
    PrintStream std_out = System.out, err_out = System.err;
    if(argv.length > 0) {
      std_out = new PrintStream(new FileOutputStream(argv[0]));
    }
    if(argv.length > 1) {
      err_out = new PrintStream(new FileOutputStream(argv[1]));
    }
    Test test_object = new HummerClientSocketTest(std_out, err_out);
    Vector test_suite = new Vector();
    test_suite.add(test_object);
    Test.runTestSuite(test_suite);
  }
	/**
	A seperate constructor was needed so that a client thread would be started,
	and the default is still needed for an uninitialized Test, so this is added.
	*/
  public HummerClientSocketTest() { super(); }

	/**
	Constructor to specify where to place output messages
	*/
  public HummerClientSocketTest(PrintStream std_out, PrintStream err_out) {
    super(std_out, err_out);
  }

	/**
	A constructor is needed to start up a client thread.
	*/
  public HummerClientSocketTest(boolean is_server, PrintStream std_out, PrintStream err_out) {
    super(std_out, err_out);
  }


  protected void setTestClassName() {
    this._test_class_name = "HummerClientSocket";
  }

	/**
	Defines the tests run.  Four methods are tested: Constructor (1), validation, newAccountValidation,
	and readResponseObject. <br>
	*/
  protected void testDefinition() throws Exception {
    Thread client_thread = new Thread(new HummerClientSocketTest(false, _standard_out, _error_out));
    ServerSocket server_socket = new ServerSocket(0);
    ObjectOutputStream out_stream;
    ObjectInputStream in_stream;
    Socket client_socket;
    HummerSignedObject signed_output;
    Response output = new Response(), input;
    Object input_object;
    long output_message_id, input_message_id = StaticMethods.getUniqueTimestamp();
    PublicKey current_public_key;
    String correct_protocol = "default", current_alias, current_signer_alias;
    HummerCertificate current_certificate;
    Signature current_verify;
    _port_number = server_socket.getLocalPort();
    _inet_address = InetAddress.getLocalHost();
    PersonalConfig personal_config = new PersonalConfig(_inet_address.getHostAddress(), null);

		/* *********************************
		Constructor testing
		************************************/
    setStaticVariables();
    client_thread.start();


		/*
		Test for server sending back null Response
		*/
    client_socket = server_socket.accept();
    in_stream = new ObjectInputStream(client_socket.getInputStream());
    in_stream.readObject();
    out_stream = new ObjectOutputStream(client_socket.getOutputStream());
    out_stream.writeObject(null);
    input = (Response)(in_stream.readObject());
    if(runSimpleTest("Server:Sent null Response object", "Recieved HummerSocketException back",
                input._message.getClass().getName().equals("edu.uidaho.hummer.util.socket.HummerSocketException"))) {
      runSimpleTest("Server:Sent null Response object", "Exception type is _INVALID_DATA_FORMAT",
                    ((HummerSocketException)input._message)._type == HummerSocketException._INVALID_DATA_FORMAT);
    }
    out_stream.close();
    in_stream.close();

		/*
		Test for server sending back invalid null Response._data
		*/
    output = new Response();
    output._id = input_message_id;
    output._message = new Long(input._id);
    output._data = null;
    client_socket = server_socket.accept();
    in_stream = new ObjectInputStream(client_socket.getInputStream());
    in_stream.readObject();
    out_stream = new ObjectOutputStream(client_socket.getOutputStream());
    signed_output = new HummerSignedObject(output, _local_server_sign);
    out_stream.writeObject(signed_output);
    input = (Response)(in_stream.readObject());
    if(runSimpleTest("Server:Sent null value for Response._data object", "Recieved HummerSocketException back",
                input._message.getClass().getName().equals("edu.uidaho.hummer.util.socket.HummerSocketException"))) {
      runSimpleTest("Server:Sent null value for Response._data object", "Exception type is _INVALID_DATA_FORMAT",
                    ((HummerSocketException)input._message)._type == HummerSocketException._INVALID_DATA_FORMAT);
    }
    out_stream.close();
    in_stream.close();


		/*
		Test for server sending back Response with _data Vector size < 2
		*/
    client_socket = server_socket.accept();
    in_stream = new ObjectInputStream(client_socket.getInputStream());
    input = (Response)(in_stream.readObject());
    output = new Response();
    output._id = input_message_id;
    output._message = new Long(input._id);
    output._data.add(correct_protocol);
    out_stream = new ObjectOutputStream(client_socket.getOutputStream());
    signed_output = new HummerSignedObject(output, _local_server_sign);
    out_stream.writeObject(signed_output);
    input = (Response)(in_stream.readObject());
    if(runSimpleTest("Server:Sent Response object _data size < 2", "Recieved HummerSocketException back",
                input._message.getClass().getName().equals("edu.uidaho.hummer.util.socket.HummerSocketException"))) {
      runSimpleTest("Server:Sent Response object _data size < 2", "Exception type is _INVALID_DATA_FORMAT",
                    ((HummerSocketException)input._message)._type == HummerSocketException._INVALID_DATA_FORMAT);
    }
    out_stream.close();
    in_stream.close();


		/*
		Test for server sending back Response with _data Vector size > 3
		*/
    PublicKey server_public_key = _key_store.getPersonalKeyPair().getPublic();
    output = new Response();
    client_socket = server_socket.accept();
    in_stream = new ObjectInputStream(client_socket.getInputStream());
    input = (Response)(in_stream.readObject());
    output._id = input_message_id;
    output._message = new Long(input._id);
    output._data.add(correct_protocol);
    output._data.add(new HummerCertificate(_local_server_sign,
                                           server_public_key,
                                           null, _future_expiration_date));
    output._data.add(new HummerCertificate(_local_server_sign,
                                           _local_user_key_pair.getPublic(),
                                           null, _future_expiration_date));
    output._data.add("hello there");
    out_stream = new ObjectOutputStream(client_socket.getOutputStream());
    signed_output = new HummerSignedObject(output, _local_server_sign);
    out_stream.writeObject(signed_output);
    input = (Response)(in_stream.readObject());
    if(runSimpleTest("Server:Sent Response object _data size > 3", "Recieved HummerSocketException back",
                input._message.getClass().getName().equals("edu.uidaho.hummer.util.socket.HummerSocketException"))) {
      runSimpleTest("Server:Sent Response object _data size > 3", "Exception type is _INVALID_DATA_FORMAT",
                    ((HummerSocketException)input._message)._type == HummerSocketException._INVALID_DATA_FORMAT);
    }
    out_stream.close();
    in_stream.close();

		/*
		Test for server sending back Response signed with wrong key
		*/
    output = new Response();
    client_socket = server_socket.accept();
    in_stream = new ObjectInputStream(client_socket.getInputStream());
    input = (Response)(in_stream.readObject());
    output._id = input_message_id;
    output._message = new Long(input._id);
    output._data.add(correct_protocol);
    output._data.add(new HummerCertificate(_local_server_sign,
                                           server_public_key,
                                           null, _future_expiration_date));
    out_stream = new ObjectOutputStream(client_socket.getOutputStream());
    signed_output = new HummerSignedObject(output, _trusted_server_sign);
    out_stream.writeObject(signed_output);
    input = (Response)(in_stream.readObject());
    if(runSimpleTest("Server:Sent Response object signed with wrong key",
                                "Recieved HummerSocketException back",
                input._message.getClass().getName().equals("edu.uidaho.hummer.util.socket.HummerSocketException"))) {
      runSimpleTest("Server:Sent Response object signed with wrong key",
                               "Exception type is _INVALID_SIGNATURE",
                    ((HummerSocketException)input._message)._type == HummerSocketException._INVALID_SIGNATURE);
    }
    out_stream.close();
    in_stream.close();

		/*
		Test for server sending back invalid HummerCertificate
		*/
    output = new Response();
    client_socket = server_socket.accept();
    in_stream = new ObjectInputStream(client_socket.getInputStream());
    input = (Response)(in_stream.readObject());
    output._id = input_message_id;
    output._message = new Long(input._id);
    output._data.add(correct_protocol);
    output._data.add(new HummerCertificate(_trusted_server_sign,
                                           server_public_key,
                                           null, _future_expiration_date));
    out_stream = new ObjectOutputStream(client_socket.getOutputStream());
    signed_output = new HummerSignedObject(output, _local_server_sign);
    out_stream.writeObject(signed_output);
    input = (Response)(in_stream.readObject());
    if(runSimpleTest("Server:Sent Response object with invalid HummerCertificate", "Recieved HummerSocketException back",
                input._message.getClass().getName().equals("edu.uidaho.hummer.util.socket.HummerSocketException"))) {
      runSimpleTest("Server:Sent Response object with invalid HummerCertificate", "Exception type is _INVALID_LOGIN",
                    ((HummerSocketException)input._message)._type == HummerSocketException._INVALID_LOGIN);
    }
    out_stream.close();
    in_stream.close();


		/*
		Test for server sending back expired HummerCertificate
		*/
    output = new Response();
    client_socket = server_socket.accept();
    in_stream = new ObjectInputStream(client_socket.getInputStream());
    input = (Response)(in_stream.readObject());
    output._id = input_message_id;
    output._message = new Long(input._id);
    output._data.add(correct_protocol);
    output._data.add(new HummerCertificate(_local_server_sign,
                                           server_public_key,
                                           null, new GregorianCalendar()));
    out_stream = new ObjectOutputStream(client_socket.getOutputStream());
    signed_output = new HummerSignedObject(output, _local_server_sign);
    out_stream.writeObject(signed_output);
    input = (Response)(in_stream.readObject());
    if(runSimpleTest("Server:Sent Response object with expired HummerCertificate", "Recieved HummerSocketException back",
                input._message.getClass().getName().equals("edu.uidaho.hummer.util.socket.HummerSocketException"))) {
      runSimpleTest("Server:Sent Response object with expired HummerCertificate", "Exception type is _EXPIRED_HUMMER_CERTIFICATE",
                    ((HummerSocketException)input._message)._type == HummerSocketException._EXPIRED_HUMMER_CERTIFICATE);
    }
    out_stream.close();
    in_stream.close();


		/*
		Test for server sending back Response wrong key id in message
		*/
    output = new Response();
    client_socket = server_socket.accept();
    in_stream = new ObjectInputStream(client_socket.getInputStream());
    input = (Response)(in_stream.readObject());
    output._id = input_message_id;
    output._message = new Long(0);
    output._data.add(correct_protocol);
    output._data.add(_key_store.getPersonalHummerCertificate());
    out_stream = new ObjectOutputStream(client_socket.getOutputStream());
    signed_output = new HummerSignedObject(output, _local_server_sign);
    out_stream.writeObject(signed_output);
    input = (Response)(in_stream.readObject());
    if(runSimpleTest("Server:Sent Response object with wrong id in the message",
                     "Recieved HummerSocketException back",
                input._message.getClass().getName().equals("edu.uidaho.hummer.util.socket.HummerSocketException"))) {
      runSimpleTest("Server:Sent Response object with wrong id in the message",
                    "Exception type is _INVALID_MESSAGE_ID",
                    ((HummerSocketException)input._message)._type == HummerSocketException._INVALID_MESSAGE_ID);
    }



		/*
		Test for remote client with untrusted HummerCertificate
		*/
    Socket old_socket = client_socket;
    client_socket = server_socket.accept();
    out_stream.close();
    in_stream.close();
    old_socket.close();
    in_stream = new ObjectInputStream(client_socket.getInputStream());
    input = (Response)(in_stream.readObject());

    current_alias = _remote_user_config.getAlias();
    current_signer_alias = _untrusted_server_config.getAlias();
    current_certificate = _key_store.getHummerCertificate(current_signer_alias);
    current_public_key = (PublicKey)_key_store.getKey(AliasInfo._PUBLIC_KEY_TYPE,
                                                      current_signer_alias);
    current_verify = StaticMethods.getVerifyingSignatureObject(current_public_key);
    runSimpleTest("Server:Remote client (" + current_alias + ") w/untrusted HummerCertificate",
                  "Protocol equals \"" + correct_protocol + "\"",
                  ((String)(input._data.get(0))).equals(correct_protocol));
    runSimpleTest("Server:Remote client (" + current_alias + ") w/untrusted HummerCertificate",
                  "Alias equals \"" + current_alias + "\"",
                  ((String)(input._data.get(1))).equals(current_alias));
    runSimpleTest("Server:Remote client (" + current_alias + ") w/untrusted HummerCertificate",
                  "Signer alias equals \"" + current_signer_alias + "\"",
                  ((String)(input._data.get(2))).equals(current_signer_alias));
    runSimpleTest("Server:Remote client (" + current_alias + ") w/untrusted HummerCertificate",
                  "HummerCertificate sent equals HummerCertificate for " + current_signer_alias,
                  current_certificate.equals((HummerCertificate)(input._data.get(3)),
                                             current_verify));
    out_stream = new ObjectOutputStream(client_socket.getOutputStream());
    throwException(out_stream, "HummerClientSocketTest:Server: Giving you an exception",
                   input, (input._id + 1), HummerSocketException._NON_TRUSTED_HUMMER_CERTIFICATE);


		/*
		Test for remote client with trusted HummerCertificate
		*/
    old_socket = client_socket;
    client_socket = server_socket.accept();
    out_stream.close();
    in_stream.close();
    old_socket.close();
    in_stream = new ObjectInputStream(client_socket.getInputStream());
    input = (Response)(in_stream.readObject());
    current_signer_alias = _trusted_server_config.getAlias();
    current_certificate = _key_store.getHummerCertificate(current_signer_alias);
    current_public_key = (PublicKey)_key_store.getKey(AliasInfo._PUBLIC_KEY_TYPE, current_signer_alias);
    current_verify = StaticMethods.getVerifyingSignatureObject(current_public_key);

    runSimpleTest("Server:Remote client (" + current_alias + ") w/trusted HummerCertificate",
                  "Protocol equals \"" + correct_protocol + "\"",
                  ((String)(input._data.get(0))).equals(correct_protocol));
    runSimpleTest("Server:Remote client (" + current_alias + ") w/trusted HummerCertificate",
                  "Alias equals \"" + current_alias + "\"",
                  ((String)(input._data.get(1))).equals(current_alias));
    runSimpleTest("Server:Remote client (" + current_alias + ") w/trusted HummerCertificate",
                  "Signer alias equals \"" + current_signer_alias + "\"",
                  ((String)(input._data.get(2))).equals(current_signer_alias));
    runSimpleTest("Server:Remote client (" + current_alias + ") w/trusted HummerCertificate",
                  "HummerCertificate recieved is valid for " + current_signer_alias + "'s PublicKey",
                  current_certificate.isValid(current_verify));
    output._id = input_message_id;
    output._type = Response._OK;
    output._message = new Long(input._id);
    output._data = new Vector();
    output._data.add(correct_protocol);
    _trusted_server_sign = StaticMethods.getSigningSignatureObject(_trusted_server_key_pair.getPrivate());
    output._data.add(new HummerCertificate(_trusted_server_sign,
                                           _local_server_key_pair.getPublic(),
                                           null, _future_expiration_date));
    signed_output = new HummerSignedObject(output, _local_server_sign);
    out_stream = new ObjectOutputStream(client_socket.getOutputStream());
    out_stream.writeObject(signed_output);

		/*
		Test for local client w/HummerCertificate containing updated PublicKey
		*/
    old_socket = client_socket;
    client_socket = server_socket.accept();
    out_stream.close();
    in_stream.close();
    old_socket.close();
    in_stream = new ObjectInputStream(client_socket.getInputStream());
    input = (Response)(in_stream.readObject());

    current_alias = _local_user_config.getAlias();
    current_signer_alias = _inet_address.getHostAddress();
    current_certificate = _key_store.getHummerCertificate(current_signer_alias);
    current_public_key = (PublicKey)_key_store.getKey(AliasInfo._PUBLIC_KEY_TYPE, current_signer_alias);
    current_verify = StaticMethods.getVerifyingSignatureObject(current_public_key);

    runSimpleTest("Server:Local client (" + current_alias + ") w/HummerCertificate containing updated PublicKey",
                  "Protocol equals \"" + correct_protocol + "\"",
                  ((String)(input._data.get(0))).equals(correct_protocol));
    runSimpleTest("Server:Local client (" + current_alias + ") w/HummerCertificate containing updated PublicKey",
                  "Alias equals \"" + current_alias + "\"",
                  ((String)(input._data.get(1))).equals(current_alias));
    runSimpleTest("Server:Local client (" + current_alias + ") w/HummerCertificate containing updated PublicKey",
                  "Signer alias equals \"" + current_signer_alias + "\"",
                  ((String)(input._data.get(2))).equals(current_signer_alias));
    runSimpleTest("Server:Local client (" + current_alias + ") w/HummerCertificate containing updated PublicKey",
                  "HummerCertificate sent is valid for " + current_signer_alias + "'s PublicKey",
                  current_certificate.isValid(current_verify));
    output._id = input_message_id;
    output._type = Response._OK;
    output._message = new Long(input._id);
    output._data = new Vector();
    output._data.add(correct_protocol);
    output._data.add(new HummerCertificate(_local_server_sign,
                                           _local_server_key_pair.getPublic(),
                                           null, _future_expiration_date));
    signed_output = new HummerSignedObject(output, _local_server_sign);
    out_stream = new ObjectOutputStream(client_socket.getOutputStream());
    out_stream.writeObject(signed_output);


		/*
		Test for local client w/HummerCertificate containing outdated PublicKey
		*/

    old_socket = client_socket;
    client_socket = server_socket.accept();
    out_stream.close();
    in_stream.close();
    old_socket.close();
    in_stream = new ObjectInputStream(client_socket.getInputStream());
    input = (Response)(in_stream.readObject());

    runSimpleTest("Server:Local client (" + current_alias + ") w/HummerCertificate containing outdated PublicKey",
                  "Protocol equals \"" + correct_protocol + "\"",
                  ((String)(input._data.get(0))).equals(correct_protocol));
    runSimpleTest("Server:Local client (" + current_alias + ") w/HummerCertificate containing outdated PublicKey",
                  "Alias equals \"" + current_alias + "\"",
                  ((String)(input._data.get(1))).equals(current_alias));
    runSimpleTest("Server:Local client (" + current_alias + ") w/HummerCertificate containing outdated PublicKey",
                  "Signer alias equals \"" + current_signer_alias + "\"",
                  ((String)(input._data.get(2))).equals(current_signer_alias));
    runSimpleTest("Server:Local client (" + current_alias + ") w/HummerCertificate containing outdated PublicKey",
                  "HummerCertificate sent equals HummerCertificate for " + current_signer_alias,
                  current_certificate.equals((HummerCertificate)(input._data.get(3)), current_verify));
    output._id = input_message_id;
    output._type = Response._OK;
    output._message = new Long(input._id);
    output._data = new Vector();
    output._data.add(correct_protocol);
    output._data.add(_key_store.getHummerCertificate(_inet_address.getHostAddress()));
    signed_output = new HummerSignedObject(output, _local_server_sign);
    out_stream = new ObjectOutputStream(client_socket.getOutputStream());
    out_stream.writeObject(signed_output);
    input_object = in_stream.readObject();
    if(runSimpleTest("Server:Local client (" + current_alias + ") w/HummerCertificate containing outdated PublicKey",
                     "Recieved a second object of type equal to HummerSignedObject",
                     input_object.getClass().getName().equals("edu.uidaho.hummer.dataobject.HummerSignedObject"))) {
      HummerSignedObject signed_input = (HummerSignedObject)input_object;
      input = (Response)(signed_input.getObject());
      Signature signer_verify = StaticMethods.getVerifyingSignatureObject(current_certificate.getPublicKey());
      runSimpleTest("Server:Local client (" + current_alias + ") w/HummerCertificate containing outdated PublicKey",
                    "Signed object recieved is valid",
                    signed_input.verify(signer_verify));
      if(runSimpleTest("Server:Local client (" + current_alias +
                       ") w/HummerCertificate containing outdated PublicKey",
                       "Second Response object is not null",
                       input != null)) {
        runSimpleTest("Server:Local client (" + current_alias + ") w/HummerCertificate containing outdated PublicKey",
                      "Second Response object's _data Vector is not null",
                      input._data != null);
        runSimpleTest("Server:Local client (" + current_alias + ") w/HummerCertificate containing outdated PublicKey",
                      "_id for second object recieved is equal to sent _id + 1",
                      input._id == (input_message_id + 1));
        if(input._data != null) {
          runSimpleTest("Server:Local client (" + current_alias + ") w/HummerCertificate containing outdated PublicKey",
                        "Second Response object's _data Vector is of size = 1",
                        input._data.size() == 1);
          runSimpleTest("Server:Local client (" + current_alias + ") w/HummerCertificate containing outdated PublicKey",
                        "PublicKey in second Response is not null",
                        input._data.get(0) != null);
          if(input._data.get(0) != null) {
            runSimpleTest("Server:Local client (" + current_alias + ") " +
                          "w/HummerCertificate containing outdated PublicKey",
                          "PublicKey in second Response equals current PublicKey in HummerKeyStore",
                          ((PublicKey)(input._data.get(0))).equals(_key_store.getPersonalKeyPair().getPublic()));
          }
        }
      }
    }
    else if(input_object.getClass().getName().equals("edu.uidaho.hummer.dataobject.Response")) {
      Response input_response = (Response)input_object;
      if(input_response._message != null) {
        println("Response type recieved.  Message type: " +
               ((Response)input_object)._message.getClass().getName(), true);
      }
      else {
        println("Response type recieved.  Message is null.", true);
      }        
    }
    else {
      println("Type: " + input_object.getClass().getName(), true);
    }


		/* ***************************************************************************************
		 ===================================================================
		 * Begin integration testing between HummerClientSocket and HummerServerSocket
		 * First, there is a new HummerKeyStore, which has only an administrator account,
		 * 	which is not initialized.  It will be initialized, and some basic communication
		 *	will be done (Response objects sent)
		 ===================================================================
		 ****************************************************************************************/
		/*
		Set the old key store to null, just in case it gets used again it doesn't mess up the
		testing.  It shouldn't be used after this point at all.
		*/
    Thread.sleep(300);
    _key_store = null;

    _server_key_store = (DefaultKeyStore)HummerKeyStore.getInstance();
    _server_key_store.load("server_admin_pass".getBytes());
    old_socket = client_socket;
    client_socket = server_socket.accept();
    out_stream.close();
    in_stream.close();
    old_socket.close();
    HummerServerSocket hummer_server_socket = new HummerServerSocket(null, client_socket, _server_key_store);
   
    runExceptionTest("Server: New HummerKeyStore (only admin account) for validation method.",
                     hummer_server_socket, "validation", null, false, null);


    runExceptionTest("Server:Reading Response object of type, _OK", hummer_server_socket,
                     "readResponseObject", null, false);
    _server_message = new Response();
    _server_message._type = Response._OK;
    _server_message._message = "Server message";
    _server_message._data.add("Server Data[0]");
    _server_message._data.add("Server Data[1]");

    runExceptionTest("Server:Writing Response object of type, _OK", hummer_server_socket,
                     "writeResponseObject", null, false, _server_message);

    _server_message._type = Response._EXCEPTION;
    runExceptionTest("Server:Writing Response object of type, _EXCEPTION", hummer_server_socket,
                     "writeResponseObject", null, false, _server_message);

    client_socket = server_socket.accept();
    hummer_server_socket = new HummerServerSocket(null, client_socket, _server_key_store);
    runExceptionTest("Server: admin account login as normal login (validation method).",
                     hummer_server_socket, "validation", null, false, null);
    input = (Response)(runExceptionTest("Server:Reading Response object of type, _OK", hummer_server_socket,
                     "readResponseObject", null, false)[2]);
    Thread.sleep(50);
    if(runSimpleTest("Server:Reading Response object of type, _OK", "Object recieved is not null.",
                     input != null)) {
      boolean equal = true;
      if(input._message == null) {
        equal = equal && (_client_message._message == null);
      }
      else {
        equal = equal && (_client_message._message.equals(input._message));
      }
      runSimpleTest("Server:Reading Response object of type, _OK",
                    "Response._message recieved from client equals message sent by client.", equal);
      equal = true;
      if(input._data == null) {
        equal = equal && (_client_message._data == null);
      }
      else {
        equal = equal && (_client_message._data.equals(input._data));
      }
      runSimpleTest("Server:Reading Response object of type, _OK",
                    "Response._data recieved from client equals data sent by client.", equal);
      runSimpleTest("Server:Reading Response object of type, _OK",
                    "Response._type recieved from client equals type sent by client.",
                    input._type == _client_message._type);
    }

    Thread.sleep(200);
    _server_message._type = Response._OK;
    runExceptionTest("Server:Writing Response object of type, _OK", hummer_server_socket,
                     "writeResponseObject", null, false, _server_message);

    client_socket = server_socket.accept();
    hummer_server_socket.close();
    hummer_server_socket = new HummerServerSocket(null, client_socket, _server_key_store);
    runExceptionTest("Server: Validating with client (Calling validation method).", hummer_server_socket,
                     "validation", null, false, null);
    runSimpleTest("Server: Opening socket with client.",                      
                  "PublicKey for client is equal to that in key store of the server.",
                  Arrays.equals(_client_key_store.getPersonalKeyPair().getPublic().getEncoded(),
                                _server_key_store.getKey(AliasInfo._PUBLIC_KEY_TYPE, "admin").getEncoded()));
    runSimpleTest("Server: Opening socket with client.",                      
                  "PublicKey for server is equal to that in key store of the client.",
                  Arrays.equals(_server_key_store.getPersonalKeyPair().getPublic().getEncoded(),
              _client_key_store.getKey(AliasInfo._PUBLIC_KEY_TYPE, client_socket.getInetAddress().getHostAddress()).getEncoded()));
 
    hummer_server_socket.writeResponseObject(_server_message);
    input = (Response)(runExceptionTest("Server: Reading command Response from client", hummer_server_socket,
                                        "readResponseObject", null, false)[2]);
    runSimpleTest("Server: Reading command Response from client",
                  "_message and _data values returned are null", input._message == null && input._data == null);
    if(runSimpleTest("Server: Reading command Response from client",
                     "Alias is in the HummerKeyStore",
                     _server_key_store.contains("user1"))) {
      runSimpleTest("Server: Reading command Response from client",
                    "Alias added is a user type alias", 
                    _server_key_store.getAliasType("user1") == AliasInfo._USER_ALIAS_TYPE);
      runSimpleTest("Server: Reading command Response from client	",
                    "Alias added is for a new (uninitialized account)", 
                    _server_key_store.isNewAccount("user1"));
    }

		/*
		Wait for the client to finish before finsishing up...after this method, statistics are tallied.
		*/
   client_thread.join();
//   _client_key_store.store("client_key_store.hum");
//   _server_key_store.store("server_key_store.hum");
 }











/* **************************************************************************
 * ===============         ==================================================
 * =                                                 ========================
 * 	Below is the run() method used by the client
 * =                                                 ========================
 * ===============         ==================================================
 ***************************************************************************/
  public void run() {
    try {
      KeyPair server_key_pair = _key_store.getPersonalKeyPair();
      String correct_protocol = "default";
      HummerCertificate trusted_certificate = _trusted_remote_user_certificate,
                        untrusted_certificate = _untrusted_remote_user_certificate;
      Response input = null, output = null;
      HummerClientSocket hummer_socket;
      ObjectInputStream in_stream;
      ObjectOutputStream out_stream;
      Socket socket = null;
      if(_inet_address != null) {
        socket = new Socket(_inet_address, _port_number);
      }
      else {
        errorPrintln("DefaulServerSocketTest:run (client): " +
                           "InetAddress.getLocalHost() is returning a null value!");
        throw new NullPointerException();
      }

		/* *************************
		Constructor testing
		***************************/
      newMethod("Constructor");
      hummer_socket = (HummerClientSocket)(runConstructorTest("Acceptable parameters", null, false,
                                                               _remote_user_config, socket, _key_store)[2]);

		/* ***************************************
		==========================================
		* validation testing
		==========================================
		*****************************************/

      newMethod("validation");


		/*
		Test for invalid data recieved from server.
		*/
      Thread.sleep(300);
      note("Testing invalid responses on local account");
      runExceptionTest("Client: Recieved null Response object", hummer_socket, "validation",
                       "edu.uidaho.hummer.util.socket.HummerSocketException", true, null);


      Thread.sleep(300);
      socket = new Socket(_inet_address, _port_number);
      hummer_socket.close();
      hummer_socket = new HummerClientSocket(_local_user_config, socket, _key_store);
      runExceptionTest("Client: Recieved null Response._data", hummer_socket, "validation",
                       "edu.uidaho.hummer.util.socket.HummerSocketException", true, null);


      Thread.sleep(300);
      socket = new Socket(_inet_address, _port_number);
      hummer_socket.close();
      hummer_socket = new HummerClientSocket(_local_user_config, socket, _key_store);
      runExceptionTest("Client: Recieved Response with _data Vector size < 2", hummer_socket, "validation",
                       "edu.uidaho.hummer.util.socket.HummerSocketException", true, null);


      Thread.sleep(300);
      socket = new Socket(_inet_address, _port_number);
      hummer_socket.close();
      hummer_socket = new HummerClientSocket(_local_user_config, socket, _key_store);
      runExceptionTest("Client: Recieved Response with _data Vector size > 3", hummer_socket, "validation",
                       "edu.uidaho.hummer.util.socket.HummerSocketException", true, null);


      Thread.sleep(300);
      socket = new Socket(_inet_address, _port_number);
      hummer_socket.close();
      hummer_socket = new HummerClientSocket(_local_user_config, socket, _key_store);
      HummerSocketException e = (HummerSocketException)(runExceptionTest("Client: Recieved Response signed with wrong key", hummer_socket, "validation",
                       "edu.uidaho.hummer.util.socket.HummerSocketException", true, null)[1]);


      Thread.sleep(300);
      socket = new Socket(_inet_address, _port_number);
      hummer_socket.close();
      hummer_socket = new HummerClientSocket(_local_user_config, socket, _key_store);
      runExceptionTest("Client: Recieved Response with invalid HummerCertificate", hummer_socket, "validation",
                       "edu.uidaho.hummer.util.socket.HummerSocketException", true, null);


      Thread.sleep(300);
      socket = new Socket(_inet_address, _port_number);
      hummer_socket.close();
      hummer_socket = new HummerClientSocket(_local_user_config, socket, _key_store);
      runExceptionTest("Client: Recieved Response with expired HummerCertificate", hummer_socket, "validation",
                       "edu.uidaho.hummer.util.socket.HummerSocketException", true, null);


      Thread.sleep(300);
      socket = new Socket(_inet_address, _port_number);
      hummer_socket.close();
      hummer_socket = new HummerClientSocket(_local_user_config, socket, _key_store);
      runExceptionTest("Client: Recieved Response with wrong id in message", hummer_socket, "validation",
                       "edu.uidaho.hummer.util.socket.HummerSocketException", true, null);


		/*
		Remote client with untrusted HummerCertificate
		*/
      Thread.sleep(300);
      note("Remote client with untrusted HummerCertificate");
      socket = new Socket(_inet_address, _port_number);
      hummer_socket.close();
      hummer_socket = new HummerClientSocket(_remote_user_config, socket, _key_store);
      runExceptionTest("Client:Remote client alias (" + _remote_user_config.getAlias() +
                       ") with untrusted HummerCertificate",
                       hummer_socket, "validation",
                       "edu.uidaho.hummer.util.socket.HummerSocketException",
                       true, _untrusted_server_config.getAlias());

		/*
		Remote client with trusted HummerCertificate
		*/
      Thread.sleep(300);
      note("Remote client with trusted HummerCertificate");
      socket = new Socket(_inet_address, _port_number);
      hummer_socket.close();
      hummer_socket = new HummerClientSocket(_remote_user_config, socket, _key_store);
      runExceptionTest("Client:Remote client alias (" + _remote_user_config.getAlias() +
                       ") with trusted HummerCertificate",
                       hummer_socket, "validation",
                       null, false, _trusted_server_config.getAlias());

		/*
		Local account with current keys
		*/
      Thread.sleep(300);
      note("Local client with current keys");
      socket = new Socket(_inet_address, _port_number);
      hummer_socket.close();
      hummer_socket = new HummerClientSocket(_local_user_config, socket, _key_store);
      runExceptionTest("Client:Local client alias (" + _local_user_config.getAlias() +
                       ") with current KeyPair", hummer_socket, "validation",
                       null, false, _inet_address.getHostAddress());

		/*
		Local account after client's key store has updated KeyPair (HummerCertificate holds old key)
		*/
      Thread.sleep(300);
      note("Local client with outdated keys (key store update KeyPairs");
      _key_store.updateKeyPair(true);
      Thread.sleep(200);
      socket = new Socket(_inet_address, _port_number);
      hummer_socket.close();
      hummer_socket = new HummerClientSocket(_local_user_config, socket, _key_store);
      runExceptionTest("Client:Local client alias (" + _local_user_config.getAlias() +
                       ") logging in with HummerCertificate holding " +
                       "server's old PublicKey", hummer_socket, "validation",
                       null, false, _inet_address.getHostAddress());


		/* ***************************************************************************************
		 ========================================================================================
		 * Begin integration testing between HummerClientSocket and HummerServerSocket
		 * First, there is a new HummerKeyStore, which has only an administrator account,
		 * 	which is not initialized.  It will be initialized, and some basic communication
		 *	will be done (Response objects sent)
		 ========================================================================================
		 ****************************************************************************************/
      String host_address = InetAddress.getLocalHost().getHostAddress();
      _client_key_store = (DefaultKeyStore)HummerKeyStore.getInstance();
      _client_key_store.load("server_admin_pass".getBytes());
      PersonalConfig admin_config = new PersonalConfig("admin", "server_admin_pass".getBytes());
      socket = new Socket(_inet_address, _port_number);
      HummerClientSocket hummer_client_socket = new HummerClientSocket(admin_config, socket, _client_key_store);
      Thread.sleep(300);

      println("\n*********************************************************\n", false);
      note("Integration Testing - HummerServerSocket-HummerClientSocket");
      long time_delay = System.currentTimeMillis() - System.currentTimeMillis(),
           start_time = System.currentTimeMillis(), end_time;
      runExceptionTest("Client: New HummerKeyStore (only admin account) for validation method.",
                       hummer_client_socket, "validation", null, false, host_address);
      end_time = System.currentTimeMillis();
      runSimpleTest("Client: HummerKeyStore after initial admin initialization",
                    "admin account is no longer \"new\"", !_server_key_store.isNewAccount("admin"));
      note("DefualtClientSocketTest: Time for new user validation: " +
                         (end_time - start_time - time_delay) + " ms.");

      _client_message = new Response();
      _client_message._type = Response._OK;
      _client_message._message = "The message";
      _client_message._data.add("Data[0]");
      _client_message._data.add("Data[1]");

      runExceptionTest("Client:Writing Response object of type, _OK", hummer_client_socket,
                       "writeResponseObject", null, false, _client_message);

      input = (Response)(runExceptionTest("Client:Reading Response object of type, _OK", hummer_client_socket,
                       "readResponseObject", null, false)[2]);
      if(runSimpleTest("Client:Reading Response object of type, _OK", "Object recieved is not null.",
                        input != null)) {
        boolean equal = true;
        if(input._message == null) {
          equal = equal && (_server_message._message == null);
        }
        else {
          equal = equal && (_server_message._message.equals(input._message));
        }
        if(input._data == null) {
          equal = equal && (_server_message._data == null);
        }
        else {
          equal = equal && (_server_message._data.equals(input._data));
        }
        runSimpleTest("Client:Reading Response object of type, _OK",
                      "Object recieved equals object sent by server",
                      equal);
      }

      HummerSocketException exception =
           (HummerSocketException)(runExceptionTest("Client:Reading Response object of type, _EXCEPTION",
                                          hummer_client_socket,
                                          "readResponseObject",
                                          "edu.uidaho.hummer.util.socket.HummerSocketException", true)[1]);
      hummer_client_socket.close();
		/**
		Normal login
		*/
      note("Normal login");
      socket = new Socket(_inet_address, _port_number);
      hummer_client_socket = new HummerClientSocket(admin_config, socket, _client_key_store);
      Thread.sleep(500);
      note("Normal validation");
      start_time = System.currentTimeMillis();
      runExceptionTest("Client: admin account login as normal login (validation method).",
                       hummer_client_socket, "validation", null, false, null);
      end_time = System.currentTimeMillis();
      note("DefualtClientSocketTest: Time for normal validation: " +
                         (end_time - start_time - time_delay) + " ms.");
      Thread.sleep(200);
      runExceptionTest("Client:Writing Response object of type, _OK", hummer_client_socket,
                       "writeResponseObject", null, false, _client_message);
      input = (Response)(runExceptionTest("Client:Reading Response object of type, _OK", hummer_client_socket,
                                          "readResponseObject", null, false)[2]);
      if(runSimpleTest("Client:Reading Response object of type, _OK", "Object recieved is not null.",
                        input != null)) {
        boolean equal = true;
        if(input._message == null) {
          equal = equal && (_server_message._message == null);
        }
        else {
          equal = equal && (_server_message._message.equals(input._message));
        }
        if(input._data == null) {
          equal = equal && (_server_message._data == null);
        }
        else {
          equal = equal && (_server_message._data.equals(input._data));
        }
        runSimpleTest("Client:Reading Response object of type, _OK",
                      "Object recieved equals object sent by server",
                      equal);
      }

      socket = new Socket(_inet_address, _port_number);

      hummer_client_socket = new HummerClientSocket(admin_config, socket, _client_key_store);
      hummer_client_socket.validation(null);
      input = (Response)(runExceptionTest("Client:Reading Response object of type, _OK", hummer_client_socket,
                                          "readResponseObject", null, false)[2]);
      if(runSimpleTest("Client:Reading Response object of type, _OK", "Object recieved is not null.",
                        input != null)) {
        boolean equal = true;
        if(input._message == null) {
          equal = equal && (_server_message._message == null);
        }
        else {
          equal = equal && (_server_message._message.equals(input._message));
        }
        if(input._data == null) {
          equal = equal && (_server_message._data == null);
        }
        else {
          equal = equal && (_server_message._data.equals(input._data));
        }
        runSimpleTest("Client:Reading Response object of type, _OK",
                      "Object recieved equals object sent by server",
                      equal);
      }

      Thread.sleep(200);
      note("Sending commands.");
      runExceptionTest("Client: Sending command Response object to add user (user1).", hummer_client_socket,
                       "addUserAlias", null, false, "user1", "test".getBytes());
      runExceptionTest("Client: Sending final null value to end transmission gracefully", hummer_client_socket,
                       "writeResponseObject", null, false, null);
      hummer_client_socket.close();
    }
    catch(Exception e) {
      System.out.println("HummerServerSocketTest:run(Client):");
      e.printStackTrace();
    }
  }

	/**
	Initialize some static variables that are shared between the client and the server.
	*/
  private void setStaticVariables() throws Exception {
    _local_server_key_pair = StaticMethods.generateKeyPair();
    _local_user_key_pair = StaticMethods.generateKeyPair();
    _new_local_user_key_pair = StaticMethods.generateKeyPair();
    _remote_user_key_pair = StaticMethods.generateKeyPair();
    _untrusted_server_key_pair = StaticMethods.generateKeyPair();
    _trusted_server_key_pair = StaticMethods.generateKeyPair();

    _local_server_sign = StaticMethods.getSigningSignatureObject(_local_server_key_pair.getPrivate());
    _local_server_verify = StaticMethods.getVerifyingSignatureObject(_local_server_key_pair.getPublic());
    _trusted_server_sign = StaticMethods.getSigningSignatureObject(_trusted_server_key_pair.getPrivate());
    _trusted_server_verify = StaticMethods.getVerifyingSignatureObject(_trusted_server_key_pair.getPublic());
    _untrusted_server_sign = StaticMethods.getSigningSignatureObject(_untrusted_server_key_pair.getPrivate());
    _untrusted_server_verify = StaticMethods.getVerifyingSignatureObject(_untrusted_server_key_pair.getPublic());



    _future_expiration_date.add(Calendar.DAY_OF_YEAR, 5);
		/*
		Create unique access tables for all aliases
		*/
    _remote_user_access_table.put("table1.read", Boolean.FALSE);
    _remote_user_access_table.put("table2.read", Boolean.FALSE);
    _remote_user_access_table.put("table1.write", Boolean.FALSE);

    _local_user_access_table.put("table1.read", Boolean.FALSE);
    _local_user_access_table.put("table2.read", Boolean.TRUE);
    _local_user_access_table.put("table1.write", Boolean.FALSE);

    _new_local_user_access_table.put("table1.read", Boolean.TRUE);
    _new_local_user_access_table.put("table2.read", Boolean.FALSE);
    _new_local_user_access_table.put("table1.write", Boolean.FALSE);

    _untrusted_server_access_table.put("table1.read", Boolean.TRUE);
    _untrusted_server_access_table.put("table2.read", Boolean.TRUE);
    _untrusted_server_access_table.put("table1.write", Boolean.FALSE);

    _uninitialized_server_access_table.put("table1.read", Boolean.FALSE);
    _uninitialized_server_access_table.put("table2.read", Boolean.FALSE);
    _uninitialized_server_access_table.put("table1.write", Boolean.TRUE);

    _trusted_server_access_table.put("table1.read", Boolean.TRUE);
    _trusted_server_access_table.put("table2.read", Boolean.TRUE);
    _trusted_server_access_table.put("table1.write", Boolean.TRUE);

    GregorianCalendar expiration_date = new GregorianCalendar();
    expiration_date.add(Calendar.DAY_OF_YEAR, 5);

    _trusted_remote_user_certificate = new HummerCertificate(_trusted_server_sign,
                                                      _remote_user_key_pair.getPublic(),
                                                      _remote_user_access_table, expiration_date);

    _untrusted_remote_user_certificate = new HummerCertificate(_untrusted_server_sign,
                                                      _remote_user_key_pair.getPublic(),
                                                      _remote_user_access_table, expiration_date);

    _key_store = (DefaultKeyStore)HummerKeyStore.getInstance();



    _key_store.load("admin_pass".getBytes());
    _key_store.addUserAlias(_local_user_config.getAlias(),
                            _local_user_pass,
                            _local_user_access_table);
    _key_store.addUserAlias(_new_local_user_config.getAlias(),
                            _new_local_user_config.getPassword(),
                            _new_local_user_access_table);
    _key_store.addServerAlias(_inet_address.getHostAddress(),
                              (_inet_address.getHostAddress() + "_pass").getBytes(),
                              true, new Hashtable());
    _key_store.addServerAlias(_untrusted_server_config.getAlias(),
                              _untrusted_server_pass,
                              false,
                              _untrusted_server_access_table);
    _key_store.addServerAlias(_uninitialized_server_config.getAlias(),
                              _uninitialized_server_pass,
                              true,
                              _uninitialized_server_access_table);
    _key_store.addServerAlias(_trusted_server_config.getAlias(),
                              _trusted_server_pass,
                              true,
                              _trusted_server_access_table);
     byte[] random_timestamp = StaticMethods.getRandomTimestampBytes(),
            untrusted_digest = StaticMethods.getDigest(_untrusted_server_pass,
                                                       random_timestamp,
                                                       _untrusted_server_key_pair.getPublic()),
            trusted_digest = StaticMethods.getDigest(_trusted_server_pass,
                                                     random_timestamp,
                                                     _trusted_server_key_pair.getPublic()),
            local_user_digest = StaticMethods.getDigest(_local_user_pass,
                                                       random_timestamp,
                                                       _local_user_key_pair.getPublic()),
            local_server_digest = StaticMethods.getDigest((_inet_address.getHostAddress() + "_pass").getBytes(),
                                                          random_timestamp,
                                                          _local_server_key_pair.getPublic());
    HummerCertificate server_certificate_trusted, server_certificate_untrusted;
    server_certificate_trusted = new HummerCertificate(_trusted_server_sign,
                                               _key_store.getPersonalKeyPair().getPublic(),
                                               null,
                                               _key_store.getPersonalKeyPairExpiration());
    server_certificate_untrusted = new HummerCertificate(_untrusted_server_sign,
                                               _key_store.getPersonalKeyPair().getPublic(),
                                               null,
                                               _key_store.getPersonalKeyPairExpiration());
    _local_server_certificate = new HummerCertificate(_local_server_sign,
                                                      _key_store.getPersonalKeyPair().getPublic(),
                                                      null, _key_store.getPersonalKeyPairExpiration());

    _key_store.initializeAccount(_inet_address.getHostAddress(), local_server_digest,
                                 random_timestamp, _local_server_key_pair.getPublic());
    _key_store.setHummerCertificate(_inet_address.getHostAddress(), _local_server_certificate);
    _key_store.initializeAccount(_untrusted_server_config.getAlias(), untrusted_digest,
                                 random_timestamp,
                                 _untrusted_server_key_pair.getPublic());
    _key_store.setHummerCertificate(_untrusted_server_config.getAlias(), server_certificate_untrusted);
    _key_store.initializeAccount(_trusted_server_config.getAlias(), trusted_digest,
                                 random_timestamp,
                                 _trusted_server_key_pair.getPublic());
    _key_store.setHummerCertificate(_trusted_server_config.getAlias(), server_certificate_trusted);
    _key_store.initializeAccount(_local_user_config.getAlias(),
                                 local_user_digest,
                                 random_timestamp,
                                 _local_user_key_pair.getPublic());
    _local_user_certificate = _key_store.generateHummerCertificate(_local_user_config.getAlias());
  }

	/**
	Used to make it easier to send exceptions to the client. 
	This was cut and pasted from HummerSocket.throwException.
	*/
  protected void throwException(ObjectOutputStream out_stream, String exception_message, Response response,
                                long output_message_id, int addition_type_information) {
    HummerSocketException exception = new HummerSocketException(exception_message, response,
                                                                addition_type_information);
    Response output = new Response();
    output._type = Response._EXCEPTION;
    output._id = output_message_id;
    output._message = exception;
    try {
      out_stream.writeObject(output);
    }
    catch(Exception e) {}
    
  }
}