/*
Java Hummer Project
File: ToolGroup.java
Date: 4-19-99
Author: Chad Ward
Description:  Client handler for handling the requests of
              starting and stopping tools on the system.

 Copyright (C) 1997-1998 The Hummer Project - hummer@cs.uidaho.edu
   
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
     
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package edu.uidaho.hummer.tool;

import java.io.IOException;
import java.net.Socket;
import java.util.Hashtable;
import java.util.Vector;
import java.util.Enumeration;
import java.util.Properties;
import java.util.Arrays;
import java.util.jar.JarFile;
import java.util.jar.JarEntry;
import java.security.GeneralSecurityException;

import edu.uidaho.hummer.util.StaticMethods;
import edu.uidaho.hummer.util.HummerKeyStore;

/**
Holder for all the tools that are running on the system.
Will gather all the system tools upon creation of a ToolGroup
object.  Tools can then be instantiated on the fly using
the reflect classes with the class information gathered.
@author Chad Ward
*/
public final class ToolGroup {

	/**
	Name of the operating system that this ToolServer is
	running on (value returned by System.getProperties().getProperty("os.name")).
	*/
  private String _os_name;

	/**
	List of the names of tools available for this system.  This list is
	created upon construction of a ToolGroup object.
	*/
  private Hashtable _tool_table = new Hashtable();
  
	/**
	Directory that the system dependant tools are held.
	*/
  private String _tool_path;

	/**
	ThreadGroup used to handle all the Thread processes.
	*/
  private ThreadGroup _thread_group = new ThreadGroup("Hummer Tools");

	/**
	Gathers information from the system to find the Hummer.jar file and
	the operating system name.  From this the names of all the system dependant tool
	names for Hummer are gathered.
	*/
  public ToolGroup(HummerKeyStore key_store) {
    _os_name = System.getProperties().getProperty("os.name");
    Properties sys_properties = System.getProperties();
    String class_path = sys_properties.getProperty("java.class.path"),
    	      jar_path = null, file_separator = sys_properties.getProperty("file.separator");
    
    _tool_path = "edu/uidaho/hummer/tool/".replace('/', file_separator.charAt(0));
    jar_path = StaticMethods.getJarDirectory() + "Hummer.jar";
    _tool_path += _os_name + file_separator;
    JarFile jar_file = null;
    try {
      jar_file = new JarFile(jar_path);
    }
    catch(IOException e) {
      System.err.println("ToolServer:getToolList: Could not open Hummmer jar file: " + e);
      return;
    }
    String tool_name = null, tmp_tool_path = _tool_path.replace('/', '.');
    Properties constants = null;
    try {
      constants = StaticMethods.getHummerProperties();
    }
    catch(IOException e) {
      System.err.println("ToolGroup:init: Could not open Hummer.properties for construction of tools: " + e);
      return;
    }
    Object[] parameters = new Object[3];
    parameters[0] = constants;
    parameters[1] = _thread_group;
    parameters[2] = key_store;
    Class[] name_classes = new Class[3];
    name_classes[0] = constants.getClass();
    name_classes[1] = _thread_group.getClass();
    name_classes[2] = key_store.getClass();
    for(Enumeration enum = jar_file.entries(); enum.hasMoreElements();) {
      JarEntry current_entry = (JarEntry)(enum.nextElement());
      String entry_name = current_entry.getName();
      if(entry_name.startsWith(_tool_path) &&
      	!current_entry.isDirectory()) {
        tool_name = entry_name.substring(entry_name.lastIndexOf(file_separator.charAt(0)) + 1,
                                                 entry_name.lastIndexOf("."));
        HummerTool tool = null;
        java.lang.reflect.Constructor tool_constructor;
        try {
          tool_constructor = (Class.forName(tmp_tool_path + tool_name)).getConstructor(name_classes);
          tool = (HummerTool)tool_constructor.newInstance(parameters);
        }
        catch(ClassNotFoundException e) {
          System.err.println("ToolGroup:init: Could not create tool: " + tmp_tool_path + tool_name +
                             ": " + e);
          continue;
        }
        catch(NoSuchMethodException e) {
          System.err.println("ToolGroup:init: Could not create tool: " + tmp_tool_path + tool_name +
                             ": " + e);
          continue;
        }
        catch(java.lang.reflect.InvocationTargetException e) {
         System.err.println("ToolGroup:init: Could not create tool: " + tmp_tool_path + tool_name +
                             ": " + e);
          continue;
        }
        catch(InstantiationException e) {
          System.err.println("ToolGroup:init: Could not create tool: " + tmp_tool_path + tool_name +
                             ": " + e);
          continue;
        }
        catch(IllegalAccessException e) {
          System.err.println("ToolGroup:init: Could not create tool: " + tmp_tool_path + tool_name +
                             ": " + e);
          continue;
        }
        tool.setRunning(false);
        _tool_table.put(tool.getName(), tool);
      }
    }
    _tool_path = _tool_path.replace('/', '.');
  }

	/**
	Returns an alphabatized listing of the available tools for this
	system.
	*/
  public final String[] getToolList() {
    String[] return_value = new String[_tool_table.size()];
    int i = 0;
    for(Enumeration enum = _tool_table.keys(); enum.hasMoreElements();) {
      return_value[i] = enum.nextElement().toString();
      i++;
    }
    Arrays.sort(return_value);
    return return_value;
  }


	/**
	Gets the status of all the tools available for this system.
	The values are returned in a Vector of String arrays.  The first
	value in the Vector is an array of Strings representing a header.
	*/
  public final Vector getToolStatus() {
    String[] header = new String[3];
    header[0] = "Tool Name";
    header[1] = "Running";
    header[2] = "Status";
    String[] tool_list = getToolList();
    Object[] current_tool_info;
    Vector tool_info = new Vector();
    tool_info.add(header);
    HummerTool current_tool;
    for(int i = 0; i < tool_list.length; i++) {
      current_tool_info = new Object[3];
      current_tool_info[0] = tool_list[i];
      current_tool = (HummerTool)_tool_table.get(tool_list[i]);
      if(current_tool == null) {
        current_tool_info[1] = Boolean.FALSE;
      }
      else {
        current_tool_info[1] = new Boolean(current_tool.isRunning());
      }
      current_tool_info[2] = getToolStatus(tool_list[i]);
      if(current_tool_info[2] == null) {
        current_tool_info[2] = "Normal";
      }
      tool_info.add(current_tool_info);
    }
    return tool_info;
  }

	/**
	Returns status of a tool for the system.
	@return True if the tool has been created and is running.
		False if the tool has either not been instantiated or
		is not running.
	*/
  public final String getToolStatus(String tool_name) {
    if(!_tool_table.containsKey(tool_name)) {
      return "Unknown Tool";
    }
    HummerTool tool = (HummerTool)_tool_table.get(tool_name);
    return tool.getStatus();
  }

	/**
	Returns false if the tool does not exist,
	the tool reference in the tool table is null,
	or if the HummerTool.isRunning() method returns false, otherwise
	true.
	*/
  public final boolean toolIsRunning(String tool_name) {
    HummerTool tool = (HummerTool)_tool_table.get(tool_name);
    if(tool == null) {
      return false;
    }
    return tool.isRunning();
  }

	/**
	Sets all of the tools in the tool table to the given boolean.
	*/
  public final void setToolsRunning(boolean running) {
    String[] tool_list = getToolList();
    for(int i = 0; i < tool_list.length; i++) {
      setToolRunning(running, tool_list[i]);
    }
  }

	/**
	Sets the running status of a tool.  If the tool has not
	been instantiated, then it is created and put in the _tool_table.
	@return True if the tool was correctly started, false otherwise.
	*/
  public final boolean setToolRunning(boolean running, String tool_name) {
    if(!_tool_table.containsKey(tool_name)) {
      return false;
    }
    HummerTool tool = (HummerTool)_tool_table.get(tool_name);
    tool.setRunning(running);
    return true;
  }

}