/*
Java Hummer Project
File: DiffieHellman.java
Date: 1-21-99
Author: Chad Ward
Description:  Interface for generating key using Diffie-Hellman
              public key exchange protocol.

 Copyright (C) 1997-1998 The Hummer Project - hummer@cs.uidaho.edu
   
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
     
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package edu.uidaho.hummer.util;

import java.security.*;
import java.security.spec.*;
import javax.crypto.*;
import javax.crypto.spec.*;
import javax.crypto.interfaces.*;

/**
Uses the Java Cryptography Extension classes to implement
the Diffie-Hellman public key exchange protocol.  After
the server has generated an initial key pair, key pair
generation is simplified by the server sending out a copy
of its public key

@author Chad Ward
*/
public class DiffieHellman {
  /** 
  Diffie-Hellman public and private key pair
  */
  private KeyPair _key_pair;

  /**
  Default type of encryption key.  See generateSharedSecretKey method
  */
  public static final String _DEFAULT_KEY_TYPE = "DESede";

  /**
  Default type of cipher.  See generateSharedSecretKey method
  */
  public static final String _DEFAULT_CIPHER_TYPE = "DESede/ECB/PKCS5Padding";

  /**
  Creates a DiffieHellman object with specified algorithm for encryption/decryption
  and the public and private keys
  */  
  public DiffieHellman() {}

  /**
  <B>This will take a while to generate, so this
  function should be used VERY sparingly!</B>
  <Br>
  NOTE - For Diffie-Hellman to work, both parties must be using
  the same modulus and base.  This can be derived from the public
  key of either party.  Also, generating key pairs can require
  a lot of processor time.

  @return A DiffieHellman public key pair using
  JCE package algorithms.
  */
  public DHPublicKey generateKeyPair() {
    try {
      AlgorithmParameterGenerator paramgen = AlgorithmParameterGenerator.getInstance("DH");
      paramgen.init(512);
      AlgorithmParameters params = paramgen.generateParameters();
      DHParameterSpec dhparamspec = (DHParameterSpec)params.getParameterSpec(DHParameterSpec.class);
      KeyPairGenerator keypairgen = KeyPairGenerator.getInstance("DH");
      keypairgen.initialize(dhparamspec);
      _key_pair = keypairgen.generateKeyPair();
    } catch(NoSuchAlgorithmException e) {
      System.err.println("This system does not recognize DiffieHellman as a valid algorithm.\n" +
                         "A shared secret key for data encryption/decryption cannot be generated");
    } catch(InvalidAlgorithmParameterException e) {
      System.err.println("The system has generated an invalid algorithm parameter.\n" +
                         "A shared secret key for data encryption/decryption cannot be generated");
    } catch(InvalidParameterSpecException e) {
      System.err.println("The system has generated an invalid parameter.\n" +
                         "A shared secret key for data encryption/decryption cannot be generated");
    }
    return (DHPublicKey)(_key_pair.getPublic());
  }

  /**
  NOTE - For Diffie-Hellman to work, both parties must be using
  the same modulus and base.  This can be derived from the public
  key of either party.  Also, generating key pairs can require
  a lot of processor time.

  @param publickey Public key generated by using a common modulus and base
  @return The DiffieHellman public key pair using
  a public key that was generated using a common modulus
  and base.
  */
  public DHPublicKey generateKeyPair(PublicKey publickey) {
    try {
      KeyPairGenerator keypairgen = KeyPairGenerator.getInstance("DH");
      keypairgen.initialize(((DHPublicKey)publickey).getParams());
      _key_pair = keypairgen.generateKeyPair();
    } catch(NoSuchAlgorithmException e) {
      System.err.println("This system does not recognize DiffieHellman as a valid algorithm.\n" +
                         "A shared secret key for data encryption/decryption cannot be generated");
    } catch(InvalidAlgorithmParameterException e) {
      System.err.println("The system has generated an invalid algorithm parameter.\n" +
                         "A shared secret key for data encryption/decryption cannot be generated");
    }
    return (DHPublicKey)(_key_pair.getPublic());
  }

  /**
  Generates a shared secret key using the public key of the other party and an algorithm
  name used for encryption and decryption

  @param remotepublickey Public key of the other party involved in the Diffie-Hellman
  key agreement
  @param encodingalgorithm Algorithm that will be used with the secret key generated for
  encrypting/decrypting data
  @return Secret key identical to the other party in the agreement
  */
  public SecretKey generateSharedSecretKey(PublicKey remotepublickey, String encodingalgorithm) 
                     throws NoSuchAlgorithmException, InvalidKeyException {
    if(_key_pair == null) {
      throw new InvalidKeyException("KeyPair has not been generated using generateKeyPair method");
    }
    KeyAgreement keyagreement = KeyAgreement.getInstance("DH");
    keyagreement.init(_key_pair.getPrivate());
    keyagreement.doPhase(remotepublickey, true);
    SecretKey sharedsecretkey = keyagreement.generateSecret(encodingalgorithm);
    return sharedsecretkey;
  }

  /**
  Gets the public key from the key pair
  @return DHPublicKey from _key_pair object
  */
  public DHPublicKey getPublic() { return (DHPublicKey) _key_pair.getPublic(); }
}
