/*
Java Hummer Project
File: StaticMethods.java
Date: 2-11-99
Author: Chad Ward
Description:  Class of public static methods.  Place to hold
		general algorithms that are not specific to any
		single class.

 Copyright (C) 1997-1998 The Hummer Project - hummer@cs.uidaho.edu
   
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
     
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package edu.uidaho.hummer.util;

import java.util.Vector;
import java.util.Date;
import java.util.Hashtable;
import java.util.Properties;
import java.io.ByteArrayOutputStream;
import java.io.DataOutputStream;
import java.io.ObjectOutputStream;
import java.io.IOException;
import java.io.BufferedInputStream;
import java.io.FileInputStream;
import java.io.InputStream;
import java.io.FileNotFoundException;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.sql.DriverManager;
import java.sql.Connection;
import java.sql.Statement;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.security.PublicKey;
import java.security.KeyPair;
import java.security.Signature;
import java.security.KeyPairGenerator;
import java.security.SecureRandom;
import java.security.MessageDigest;
import java.security.NoSuchProviderException;
import java.security.NoSuchAlgorithmException;

import edu.uidaho.hummer.dataobject.Response;

/**
Class of public static methods.  Place to hold general algorithms that are not specific to any single class.
@author Chad Ward
*/

public final class StaticMethods {
	/**
	All the characters recognized as whitespace are space, tab, newline, and linefeed (" \t\n\f")
	@see StaticMethods#split(String, String)
	@see StaticMethods#split(String, String, boolean)
	*/
  public static final String WHITESPACE = " \t\n\f";

	/**
	Signature object used by getSignatureObject.  Is initialized the first time
	that getSignatureObject is called, and then references to it are returned
	following that.
	*/
  private static Signature signature_object = null;

	/**
	Property object representing the Hummer.properties file.
	*/
  private static Hashtable _hummer_property_table = null;

	/**
	Directory where the Hummer.jar file can be located.  It is assumed
	that the Hummer.properties file can also be found here.
	*/
  private static String _jar_directory = null;

	/**
	Converts an array of bytes to an hex string
	*/
  public static String byteToHexString(byte[] array) {
    String return_value = "";
    String curr_value;
    if(array.length > 0) {
      curr_value = byteToHexString(array[0]);
      if(curr_value.length() == 1) {
        curr_value = " " + curr_value;
      }
      return_value = curr_value;
      for(int i = 1; i < array.length; i++) {
        curr_value = byteToHexString(array[i]);
        if(curr_value.length() == 1) {
          curr_value = " " + curr_value;
        }
        return_value += " " + curr_value;
      }
    }
    return return_value;
  }

	/**
	Converts a byte to it's hex string representation.
	*/
  public static String byteToHexString(byte value) {
    String return_value = "";
    if((value & 0xF0) != 0) {
      byte top_half = (byte)((value & 0xF0) >> 4);
      return_value = byteToHexString(top_half) + byteToHexString((byte)(value & (byte)0xF));
    }
    else{
      if(value < 9) {
        return_value += ((int)(value));
      }
      else {
        switch(value) {
          case 10:
            return_value += "A";
            break;
          case 11:
            return_value += "B";
            break;
          case 12:
            return_value += "C";
            break;
          case 13:
            return_value += "D";
            break;
          case 14:
            return_value += "E";
            break;
          case 15:
            return_value += "F";
        }
      }
    }
    return return_value;
  }

	/**
	Returns true if the two arrays of strings are equal, false otherwise.
	*/
  public static boolean equals(String[] array1, String[] array2) {
    boolean equal = true;
    if(array1 == null || array2 == null) {
      if(array1 == null && array2 == null) {
        return true;
      } else {
        return false;
      }
    }
    if(array1.length != array2.length) {
      equal = false;
    }
    for(int i = 0; equal && i < array1.length; i++) {
      if(!array1[i].equals(array2[i])) {
        equal = false;
      }
    }
    return equal;
  }

	/**
	Calls split(String, String, boolean) with repeat equal to true
	@see StaticMethods#split(String, String, boolean)
	*/
   public static String[] split(String line, String seperators) {
      return StaticMethods.split(line, seperators, true);
   }

	/**
	Splits the String line into an array of Strings delimited by any characters in the
	string seperators.  If repeat is true, then a single "splitting" will be done for each
	occurance of any number of delimiting characters, otherwise a new value in the return array
	will be created for each delimiting character (will contain null value)
	*/
   public static String[] split(String line, String seperators, boolean repeat) {
      Vector parts = new Vector();
      String[] return_value;
      String current_value;
      int current_index = 0;
      if(line == null) {
         return new String[0];
      }
      while(current_index < line.length()) {
         while(current_index < line.length() && seperators.indexOf(line.charAt(current_index)) > -1) {
						// while the current character in the line is present in the
						//	string of seperators, then increment current_index
            if(!repeat) {
						// if this is not to be repeated, then only do it once
						//	Example.  line="hello###there#Mister",  seperators="#"
						//		repeat = true  return_value = {"hello", "there", "Mister"}
						//		repeat = false return_value = {"hello", "", "", "there", "Mister"}
               current_index++;
               break;
            }
            current_index++;
         }
         current_value = "";
         while(current_index < line.length() && seperators.indexOf(line.charAt(current_index)) < 0) {
						// while the current character is not a delimiting character, and not
						//	the end of the string append it to the end of current_value
            current_value = current_value + line.charAt(current_index);
            current_index++;
         }
						// add the new string to the Vector
         parts.addElement(current_value);
      }
						// initialize return_value
      return_value = new String[parts.size()];
      for(int i = 0; i < parts.size(); i++) {
         return_value[i] = (String)parts.elementAt(i);
      }
      return return_value;
   }


	/**
	Converts an object to its representation in bytes
	@param object The Object to be converted
	*/
  public static byte[] getBytes(Object object) throws IOException {
   /* if(object == null) {
      return new byte[0];
    }*/
    ByteArrayOutputStream byte_array_output = new ByteArrayOutputStream();
    ObjectOutputStream data_output = new ObjectOutputStream(byte_array_output);
    data_output.writeObject(object);
    return byte_array_output.toByteArray();
  }


	/**
	Returns all the columns of the specified table in the specified database
	in a Vector of Hashtables.
	@param url The URL of the database.
	@param user The username to use for connecting to the database
	@param password The password to connect to the database.
	@param table_name The table to query.
	@param where The "where" statement in a select query to place restrictions
		on which tuples to select.
	@returns A Vector of Hashtables.  Each object in the Vector represents a tuple
		in the table. Each column in the table can be retrieved using the column
		name from the table.
	*/

  public static Vector getDBTable(String table_name, String where)
         throws IOException, FileNotFoundException, UnknownHostException, SQLException {
    Hashtable hummer_properties = getHummerPropertyTable(false);
    String url = hummer_properties.get("jdbc.url").toString(),
           user = hummer_properties.get("jdbc.user").toString(),
           password = hummer_properties.get("jdbc.password").toString();
    Vector return_value = new Vector();
    Connection connection = DriverManager.getConnection(url, user, password);
    Statement statement = connection.createStatement();
    String query = "select * from " + table_name;

    if(where != null && !where.equals("")) {
      query += " where " + where;
    }
    query += ";";
    ResultSet results = statement.executeQuery(query);
    ResultSetMetaData meta_data = results.getMetaData();
    String[] column_names = new String[meta_data.getColumnCount()];
    Hashtable current_tuple;

    boolean first_row = true;
    for(int i = 0; i < column_names.length; i++) {
      column_names[i] = meta_data.getColumnName(i + 1);
    }
    String current_value;
    while(results.next()) {
      current_tuple = new Hashtable();
      for(int i = 0; i < column_names.length; i++) {
        current_value = results.getString(column_names[i]);
        if(current_value == null) {
          current_value = "NULL";
        }
        current_tuple.put(column_names[i], current_value);
      }
      return_value.add(current_tuple);
    }
    return return_value;
  }


	/**
	Returns the directory that the Hummer.jar file can be found in.
	The value is gathered from System.properties.getProperty("java.class.path")
	If it is not found in the class path, then empty string is returned.("")
	*/
  public static String getJarDirectory() {
    if(_jar_directory == null) {
      Properties constants = System.getProperties();
      String file_separator = constants.getProperty("file.separator"),
             class_path = constants.getProperty("java.class.path"),
             path_separator = constants.getProperty("path.separator");
      String[] class_paths = StaticMethods.split(class_path, path_separator, false);
      for(int i = 0; i < class_paths.length; i++) {
        if(class_paths[i].indexOf("Hummer.jar") >= 0) {
          _jar_directory = class_paths[i];
          break;
        }
      }
      if(_jar_directory != null) {
        _jar_directory = _jar_directory.substring(0, _jar_directory.lastIndexOf(file_separator) + 1);
      }
      else {
        _jar_directory = "";
      }
    }
    return _jar_directory;
  }

  public static Properties getHummerProperties() throws IOException, FileNotFoundException {
    Properties constants = System.getProperties();
    String file_separator = constants.getProperty("file.separator");
    String path = getJarDirectory() + "Hummer.properties";
    FileInputStream in_stream = new FileInputStream(getJarDirectory() + "Hummer.properties");
    constants.load(new BufferedInputStream(in_stream));
    in_stream.close();
    return constants;
  }
	/**
	Gets the Properties object representing the Hummer.properties files.
	Gets the values from the values Hummer.properties and converts them to a
	more useful format (attempts to retrieve values that are specific to the local address)
	@param force_update Boolean determining whether the _constants object should be re-read
		from the Hummer.properties file (true=force re-read)
	*/
  public static Hashtable getHummerPropertyTable(boolean force_update)
         throws FileNotFoundException, IOException, UnknownHostException {
    String url, driver, user, password;
    if(_hummer_property_table == null || force_update) {
      _hummer_property_table = new Hashtable();
      Properties constants = getHummerProperties();
      String address = InetAddress.getLocalHost().getHostAddress(),
             host_name = InetAddress.getLocalHost().getHostName(),
             host_name_prefix = "";
      int dot_index = host_name.indexOf('.');
      if(dot_index < 0) {
        host_name_prefix = host_name;
      }
      else {
        host_name_prefix = host_name.substring(0, dot_index);
      }
      if (constants.containsKey("jdbc.url." + address)) {
        _hummer_property_table.put("jdbc.url", constants.getProperty("jdbc.url." + address) + host_name_prefix);
      }
      else {
        _hummer_property_table.put("jdbc.url", constants.getProperty("jdbc.url"));
      }
  
      if (constants.containsKey("jdbc.driver." + address)) {
        _hummer_property_table.put("jdbc.driver", constants.getProperty("jdbc.driver." + address));
      }
      else {
        _hummer_property_table.put("jdbc.driver", constants.getProperty("jdbc.driver"));
      }
  
      if (constants.containsKey("jdbc.user." + address)) {
        _hummer_property_table.put("jdbc.user", constants.getProperty("jdbc.user." + address));
      }
      else {
        _hummer_property_table.put("jdbc.user", constants.getProperty("jdbc.user"));
      }
      if (constants.containsKey("jdbc.password." + address)) {
        _hummer_property_table.put("jdbc.password", constants.getProperty("jdbc.password." + address));
      }
      else {
        _hummer_property_table.put("jdbc.password", constants.getProperty("jdbc.password"));
      }
      _hummer_property_table.put("port.HUMMER_PORT", new Integer(constants.getProperty("port.HUMMER_PORT")));
      _hummer_property_table.put("port.MESSAGE_PORT", new Integer(constants.getProperty("port.MESSAGE_PORT")));
      _hummer_property_table.put("port.CONFIG_PORT", new Integer(constants.getProperty("port.CONFIG_PORT")));
      _hummer_property_table.put("port.ALERT_INTERFACE_PORT",
                                 new Integer(constants.getProperty("port.ALERT_INTERFACE_PORT")));
      _hummer_property_table.put("port.PEER_PORT", new Integer(constants.getProperty("port.PEER_PORT")));
      _hummer_property_table.put("const.BUFFER_SIZE", new Integer(constants.getProperty("const.BUFFER_SIZE")));
      _hummer_property_table.put("const.MAX_MESSAGE_AGE", new Integer(constants.getProperty("const.MAX_MESSAGE_AGE")));
      _hummer_property_table.put("const.LENGTH_KEY_PAIR_VALIDITY",
                       new Integer(constants.getProperty("const.LENGTH_KEY_PAIR_VALIDITY")));
      _hummer_property_table.put("const.NUMBER_BACKUP_KEYS", new Integer(constants.getProperty("const.NUMBER_BACKUP_KEYS")));
      _hummer_property_table.put("const.PROTOCOL", constants.getProperty("const.PROTOCOL"));
      _hummer_property_table.put("const.KEY_STORE_FILE_EXTENSION",
                                 constants.getProperty("const.KEY_STORE_FILE_EXTENSION"));
    }
    return _hummer_property_table;
  }



	/**
	Converts a double and long value to a array of bytes
	@param random A double value.  For the message digest of this password, this number should be random
	@param timestamp A long value.  For the message digest of this password, this number represents the current time
		(new Date()).getTime() would be a good method for generating this number
	*/
  public static byte[] getRandomTimestampBytes() throws IOException {
    ByteArrayOutputStream byte_array_output = new ByteArrayOutputStream();
    DataOutputStream data_output = new DataOutputStream(byte_array_output);
    data_output.writeDouble(Math.random());
    data_output.writeLong(StaticMethods.getUniqueTimestamp());
    return byte_array_output.toByteArray();
  }

	/**
	Generates two timestamps spaced slightly, and the space depends upon the speed of
	the computer at the time of generation.  Will generate a timestamp that is not
	as predictable.
	*/
  public static long getUniqueTimestamp() {
    long timestamp = System.currentTimeMillis();
		/*
		Do some task that is going to take an amount of time that will
		always going to take a reasonable amount of time greater than
		1 ms.  This should be great enough that it will be create a random
		difference with the previous timestamp
		*/
    for(int i = 0; i < 10000; i++) {
      int k = i;
      for(int j = 0; j < i; j++) {
        k *= k;
      }
    }
    timestamp += System.currentTimeMillis();
    return timestamp;
  }

	/**
	Returns the default access table for a peer server account in a HummerKeyStore.
	There is a single key in the Hashtable, "is_peer_moderator", which is set to true.
	*/
  public static Hashtable getDefaultPeerAccessTable() {
    Hashtable return_value = new Hashtable();
    return_value.put("is_peer_moderator", Boolean.TRUE);
    return return_value;
  }

	/**
	Returns the default access table for a server account in a HummerKeyStore.
	There are read and write priveldges given for each of the 4 main tables.
	The naming convention is $lt;table_name&gt;.read and $lt;table_name&gt;.write <br>
	The default is that all reads are set to true, and writes are set to false. <br>
	The table names are:
	<ul>
	  <li> M_S
	  <li> peer
	  <li> level
	  <li> filter
	</ul>
	*/
  public static Hashtable getDefaultServerAccessTable() {
    Hashtable return_value = new Hashtable();
    return_value.put("M_S.read", new Boolean(true));
    return_value.put("peer.read", new Boolean(true));
    return_value.put("level.read", new Boolean(true));
    return_value.put("filter.read", new Boolean(true));

    return_value.put("M_S.write", new Boolean(false));
    return_value.put("peer.write", new Boolean(false));
    return_value.put("level.write", new Boolean(false));
    return_value.put("filter.write", new Boolean(false));
    return return_value;
  }

	/**
	Returns the default access table for a user account in a HummerKeyStore.
	The first main priveldge is the ability to start and stop Hummer servers,
	which is currently set to true (<b>Should be changed!</b>).
	There are also read and write priveldges given for each of the 4 main tables.
	The naming convention is $lt;table_name&gt;.read and $lt;table_name&gt;.write <br>
	The default is that all reads are set to true, and writes are set to false. <br>
	The table names are:
	<ul>
	  <li> M_S
	  <li> peer
	  <li> level
	  <li> filter
	</ul>
	*/
  public static Hashtable getDefaultUserAccessTable() {
    Hashtable return_value = new Hashtable();
    return_value.put("Control_Servers", new Boolean(true));
    return_value.put("Control_Tools", new Boolean(true));

    return_value.put("M_S.read", new Boolean(true));
    return_value.put("peer.read", new Boolean(true));
    return_value.put("level.read", new Boolean(true));
    return_value.put("filter.read", new Boolean(true));

    return_value.put("M_S.write", new Boolean(false));
    return_value.put("peer.write", new Boolean(false));
    return_value.put("level.write", new Boolean(false));
    return_value.put("filter.write", new Boolean(false));
    return return_value;
  }

	/**
	Returns the access table for a user account with full read/write privledges.
	The first main priveldge is the ability to start and stop Hummer servers,
	which is currently set to true.
	There are read and write priveldges given for each of the 4 main tables.
	The naming convention is $lt;table_name&gt;.read and $lt;table_name&gt;.write <br>
	The default is that all reads are set to true, and writes are set to false. <br>
	The table names are:
	<ul>
	  <li> M_S
	  <li> peer
	  <li> level
	  <li> filter
	</ul>
	*/
  public static Hashtable getUserFullAccessTable() {
    Hashtable return_value = new Hashtable();
    return_value.put("Control_Servers", new Boolean(true));

    return_value.put("M_S.read", new Boolean(true));
    return_value.put("peer.read", new Boolean(true));
    return_value.put("level.read", new Boolean(true));
    return_value.put("filter.read", new Boolean(true));

    return_value.put("M_S.write", new Boolean(true));
    return_value.put("peer.write", new Boolean(true));
    return_value.put("level.write", new Boolean(true));
    return_value.put("filter.write", new Boolean(true));
    return return_value;
  }


	/**
	Gets the bytes returned for MessageDigest.digest() with _password and the given array of bytes
	@param random_timestamp Array of bytes representing a random <i>double</i> and a <i>long</i> representing
		a timestamp
	@returns Bytes returned by a digest created with the password and the random_timestamp array
	*/
  public static byte[] getDigest(byte[] password, byte[] random_timestamp, PublicKey public_key)
         throws NoSuchAlgorithmException {
    if(password == null) {
      password = new byte[0];
    }
    if(random_timestamp == null) {
      random_timestamp = new byte[0];
    }
    if(public_key == null) {
      throw new NullPointerException("StaticMethods:getDigest: PublicKey cannot be null");
    }
    MessageDigest message_digest = null;
    message_digest = MessageDigest.getInstance("SHA");
    message_digest.update(password);
    message_digest.update(random_timestamp);
    message_digest.update(public_key.getEncoded());
    return message_digest.digest();
  }

	/**
	Gets the bytes returned for MessageDigest.digest() with password
	@returns Bytes returned by a digest created with the password and the random_timestamp array
	*/
  public static byte[] getDigest(byte[] password)
         throws NoSuchAlgorithmException {
    if(password == null) {
      password = new byte[0];
    }
    MessageDigest message_digest = null;
    message_digest = MessageDigest.getInstance("SHA");
    message_digest.update(password);
    return message_digest.digest();
  }

	/**
	Generates a PublicKey and PrivateKey key pair for signatures.  Initializes the KeyPairGenerator
	with a SecureRandom, with a seed pulled from the current time in milleseconds.  The key size is 1024 bits.
	*/
  public static KeyPair generateKeyPair() throws NoSuchAlgorithmException, NoSuchProviderException {
    KeyPairGenerator key_generator = KeyPairGenerator.getInstance("DSA");
    SecureRandom secure_random = SecureRandom.getInstance("SHA1PRNG", "SUN");
    secure_random.setSeed((new Date()).getTime());
    key_generator.initialize(1024, secure_random);
    return key_generator.generateKeyPair();
  }

	/**
	Returns a Signature object.  The type of Signature is "SHA1withDSA".
	
  public static Signature getSignatureObject() throws NoSuchAlgorithmException {
    if(signature_object == null) {
      signature_object = Signature.getInstance("SHA1withDSA");
    }
    return signature_object;
  }
*/
	/**
	Creates a new signature object and initializes it for signing
	data.  The algorithm used is "SHA1withDSA".
	*/
  public static java.security.Signature getSigningSignatureObject(java.security.PrivateKey private_key)
         throws NoSuchAlgorithmException, java.security.InvalidKeyException {
    java.security.Signature return_value = java.security.Signature.getInstance("SHA1withDSA");
    return_value.initSign(private_key);
    return return_value;
  }


	/**
	Creates a new signature object and initializes it for verifying signatures.
	The algorithm used is "SHA1withDSA".
	*/
  public static java.security.Signature getVerifyingSignatureObject(java.security.PublicKey public_key)
         throws NoSuchAlgorithmException, java.security.InvalidKeyException {
    java.security.Signature return_value = java.security.Signature.getInstance("SHA1withDSA");
    return_value.initVerify(public_key);
    return return_value;
  }

	/**
	Returns String representation of a Response object.
	@see edu.uidaho.hummer.dataobject.Response
	*/
  public static String responseToString(Response object, String indent) {
    if(object == null) {
      return "";
    }
    String return_value = indent + "type: " + object._type + "\n" + indent +
                          "id: " + object._id + "\n" + indent +
                          "message: " + object._message + "\n" + indent +
                          "data -" + "\n";
    if(object._data != null) {
      for(int i = 0; i < object._data.size(); i++) {
        return_value += indent + indent + "data[" + i + "]";
        if(object._data.get(i) != null) {
          return_value += "(" + object._data.get(i).getClass().getName() + ")";
        }
        return_value += ": " + object._data.get(i) + "\n";
      }
    }
    return return_value;
  }
}