/*
Java Hummer Project
File: HummerProtocol.java
Date: 1-30-99
Author: Chad Ward
Description:  General interface for Hummer validation of data
              recieved across a socket.  Parent class of server and
              client protocols.

 Copyright (C) 1997-1998 The Hummer Project - hummer@cs.uidaho.edu
   
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
     
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package edu.uidaho.hummer.util.socket;

import java.util.Hashtable;
import java.net.UnknownHostException;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import edu.uidaho.hummer.dataobject.*;
import edu.uidaho.hummer.util.HummerKeyStore;
import edu.uidaho.hummer.util.UnloadedKeyStoreException;
import edu.uidaho.hummer.util.UnknownAliasException;

public abstract class HummerProtocol {

	/**
	ID that will be placed in Response._id for all outgoing messages
	*/
  protected long _output_message_id;

	/**
	ID that will be placed in Response._id for all incoming messages
	*/
  protected long _input_message_id;

	/**
	Name given to the current protocol.  eg. "default"
	*/
  protected String _protocol_name;

	/**
	True if the concrete version of HummerSocket is a server, false
	if the concrete version is a client.
	*/
  protected boolean _is_server = false;

	/**
	Alias of the current partner that communication is being made with.
	*/
  protected String _current_partner_alias;


	/**
	HummerKeyStore object used for storing keys, dates keys created, and NewUserValidator objects for each user
	*/
  protected HummerKeyStore _key_store;

	/**
	A class for holding the username and password for an account.
	*/
  protected PersonalConfig _personal_config;

	/**
	Access rights of the current client alias (this is only used by server protocols).
	*/
  protected Hashtable _current_client_access_table = null;

	/**
	Input object recieved for validation.
	*/
  protected Response _validation_response;

	/**
	Name of the protocol
	*/
  protected String _name;

	/**
	Protected constructor, because this is an abstract class so it should
	only be instantiated by the child classes.
	*/
  protected HummerProtocol(PersonalConfig personal_config, HummerKeyStore key_store) {
    this._personal_config = personal_config;
    this._key_store = key_store;
    if(_key_store == null) {
      throw new NullPointerException("HummerProtocol:constructor: HummerKeyStore is null.");
    }
  }

	/**
	Creates a concrete instance of a Protocol class.
	@param is_server_protocol True if this protocol should be run from the server
	                          side, false otherwise
	@param protocol_name The name of the protocol, which should be taken from
	                     the Hummer.properties file.  Currently the types are:
	                     'default'.
	*/
  public static HummerProtocol getProtocol(boolean is_server_protocol, String protocol_name,
                                           PersonalConfig personal_config, HummerKeyStore key_store)
         throws NoSuchAlgorithmException, HummerSocketException {
    if(protocol_name.equals("default")) {
      try {
        if(is_server_protocol) {
          return new DefaultServerProtocol(personal_config, key_store);
        }
        else {
          return new DefaultClientProtocol(personal_config, key_store);
        }
      }
      catch(UnknownHostException e) {
        throw new HummerSocketException("HummerProtocol:getProtocol: " + e, null, 0);
      }
      catch(UnloadedKeyStoreException e) {
        throw new HummerSocketException("HummerProtocol:getProtocol: " + e, null, 0);
      }
      catch(KeyStoreException e) {
        throw new HummerSocketException("HummerProtocol:getProtocol: " + e, null, 0);
      }
    }
    else {
      throw new NoSuchAlgorithmException("HummerProtocol:getProtocol: " +
                                         "\"" + protocol_name + "\" is not a known protocol.");
    }
  }

	/**
	Returns the current table (Hashtable) representing the access rights of the
	client that has connected to this HummerSocket
	*/
  public Hashtable getCurrentClientAccessTable() {
    return _current_client_access_table;
  }
	/**
	Returns the value that should be placed in the Response._id when
	sending a response to the current communication partner.
	*/
  public long getOutputMessageID() { return _output_message_id; }

	/**
	Returns the alias of the current partner that communication is occuring with.
	*/
  public String getCurrentPartnerAlias() { return _current_partner_alias; }

	/**
	Returns the alias that is being used to communicate with the current partner.
	*/
  public String getPersonalAlias() { return _personal_config.getAlias(); }

	/**
	Returns the protocol name.
	*/
  public String getString() { return _name; }


	/**
	Implementation specific method for initializing information for the protocol.
	For example the default protocol for a client will set the current partner alias
	and signer alias in this method.
	*/
  public abstract void initializeInformation(Object protocol_information, Object other_information) 
         throws HummerSocketException, UnknownAliasException, UnloadedKeyStoreException, KeyStoreException;

	/**
	Checks the basic format of a message recieved from the current partner.  This
	is specific to the concrete Protocol instance.
	*/
  public abstract void checkBasicInputDataFormat(Object input_object) 
         throws HummerSocketException, UnknownAliasException, UnloadedKeyStoreException, KeyStoreException;

	/**
	Returns true if newAccountValidation should be done, false if the account
	that the client is logging into has already been initialized.
	*/
  public abstract boolean isNewAccountValidation() 
         throws HummerSocketException, UnknownAliasException, UnloadedKeyStoreException, KeyStoreException;

	/**
	Checks the data format for initialized account login.
	*/
  public abstract void initializedAccountCheckInputDataFormat() 
         throws HummerSocketException, UnknownAliasException, UnloadedKeyStoreException, KeyStoreException;

	/**
	Verifies the information that the current partner has sent to validate itself.
	*/
  public abstract void initializedAccountVerifyInformation()
         throws HummerSocketException, UnknownAliasException, UnloadedKeyStoreException, KeyStoreException;

	/**
	Prepares a response to send to the partner to validate this protocol's identity.
	For a client this will be the initial communication, but for a server this will
	be called after verifying the client's information.
	*/
  public abstract Object initializedAccountPrepareResponse()
         throws HummerSocketException, UnknownAliasException, UnloadedKeyStoreException, KeyStoreException;

	/**
	Performs any final operations, and if a response needs to be sent to the partner,
	then a non-null value is returned.
	*/
  public abstract Object initializedAccountFinalize() 
         throws HummerSocketException, UnknownAliasException, UnloadedKeyStoreException, KeyStoreException;


	/**
	Checks the basic format of a message recieved from the current partner.  This
	is specific to the concrete Protocol instance.
	*/
  public abstract void newAccountCheckInputDataFormat() 
         throws HummerSocketException, UnknownAliasException, UnloadedKeyStoreException, KeyStoreException;

	/**
	Verifies the information that the current partner has sent to validate itself.
	*/
  public abstract void newAccountVerifyInformation() 
         throws HummerSocketException, UnknownAliasException, UnloadedKeyStoreException, KeyStoreException;

	/**
	Prepares a response to send to the partner to validate this protocol's identity.
	For a client this will be the initial communication, but for a server this will
	be called after verifying the client's information.
	*/
  public abstract Object newAccountPrepareResponse() 
         throws HummerSocketException, UnknownAliasException, UnloadedKeyStoreException, KeyStoreException;

	/**
	Performs any final operations, and if a response needs to be sent to the partner,
	then a non-null value is returned.
	*/
  public abstract Object newAccountFinalize() 
         throws HummerSocketException, UnknownAliasException, UnloadedKeyStoreException, KeyStoreException;

	/**
	Checks the information that was sent from the current partner and determines
	whether a new protocol should be used or not.
	*/
  public abstract HummerProtocol updateProtocol() 
         throws HummerSocketException, UnknownAliasException, UnloadedKeyStoreException, KeyStoreException;

	/**
	Checks the information received from the current partner to insure its
	validity.
	*/
  public abstract Response verifyInput(Object input_object)
         throws HummerSocketException;

	/**
	Prepares a Response object to be sent to the current partner.
	*/
  public abstract Object prepareOutput(Response output) 
         throws HummerSocketException;
}