/*
Java Hummer Project
File: HummerServerSocket.java
Date: 4-5-99
Author: Chad Ward
Description:  Socket interface for a server-side connection of communication.
              This "default" version does not use any of the JCE classes, therefore
              it does do any two-way encryption/decryption.

 Copyright (C) 1997-1998 The Hummer Project - hummer@cs.uidaho.edu

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package edu.uidaho.hummer.util.socket;


import java.io.*;
import java.net.*;
import java.util.*;

import java.security.cert.CertificateException;
import java.security.*;
import edu.uidaho.hummer.dataobject.*;
import edu.uidaho.hummer.util.*;


/**
Socket interface for a server-side connection of communication.
The type of protocol is determined at runtime by looking at the
Hummer.propties file for the 'protocol' value. <br>
<h4>Class Usage and Flow:</h4>
<ul>
  <li>Constructor - Initializes variables in socket to given parameters, and attempts to get
      the PrivateKey for signing objects sent to the server.
  <li>validation - Method used to communicate to validate this HummerClientSocket with the client.
  <li>readResponseObject - Reads and validates a Response object sent by the client.  If the _command
      value in the Response object recieved is not 0, then the meaning of the Response object is parsed,
      and readResponseObject is called recursively.
  <li>getCurrentClientAccessTable - Returns the access table of the current client.
</ul>
<b> If the client makes a command request to the server by setting the _command value
in the Response object, then it is possible that a Response object with null _message and null _data
value is returned.  This should be interpreted as an end of transmission. </b>
<h4>Usage Example</h4>

Open a socket to the client.  The initialization of the ServerSocket object is not shown here.
	<blockquote><b>
	Socket socket = server_socket.accept();
	</b></blockquote>

Create a HummerKeyStore object and load it.  Here the HummerKeyStore is assumed to have already
been written to a file called "key_store.hum"
	<blockquote><b>
	HummerKeyStore key_store = new HummerKeyStore(); <br>
	key_store.load(true, "europa.csds.uidaho.edu");
	</b></blockquote>

Create a HummerServerSocket object with the values create above.  The first
parameter for the constructor is ignored.
	<blockquote><b>
	HummerServerSocket hummer_socket = new HummerServerSocket(null, socket, key_store);
	</b></blockquote>

Validate communication with the client.  Null is used for a parameter because it
only makes sense in the context of a client socket.
	<blockquote><b>
	hummer_socket.validation(null);
	</b></blockquote>


Now the server has been validated to the client, so normal communication can occur.
	<blockquote><b>
      Response input = hummer_socket.readResponseObject();
	</b></blockquote>

@author Chad Ward
*/
public final class HummerServerSocket extends HummerSocket {

	/**
	Access rights of the current alias (after validation, is set to the value in the HummerCertificate
	given by the client.)
	*/
  private Hashtable _current_client_access_table = null;



  public HummerServerSocket(PersonalConfig personal_config, Socket socket, HummerKeyStore key_store)
         throws UnloadedKeyStoreException, IOException, HummerSocketException,
                NoSuchAlgorithmException, KeyStoreException {
    super(true, personal_config, socket, key_store);
    this._key_store = (DefaultKeyStore)key_store;
    if(socket == null) {
      throw new HummerSocketException("HummerServerSocket:constructor: Socket is null.", null, 0);
    }
  }

	/**
	@return True if the current partner alias is "admin", or if the current personal
	alias equals the partner alias.
	*/
  public boolean hasAdminAccess() {
    return (_current_protocol.getCurrentPartnerAlias().equals("admin") ||
            _current_protocol.getPersonalAlias().equals(_current_protocol.getCurrentPartnerAlias()));
  }


  public void validation(Object protocol_value)
         throws HummerSocketException, KeyStoreException, UnrecoverableKeyException,
                UnloadedKeyStoreException, UnknownAliasException, CertificateException,
                IOException, OptionalDataException {
    Object output = null;
    Object input = null;
    if(_has_validated) {
      throw new HummerSocketException("HummerServerSocket: Have already called validation().", null, 0);
    }
    try {
      _current_protocol.initializeInformation(protocol_value, null);
      _object_input_stream = new ObjectInputStream(_socket.getInputStream());
      try {
        input = (Response)(_object_input_stream.readObject());
      }
      catch(ClassNotFoundException e) {
        throwException("HummerServerSocket:validation: " +
                       "Invalid data recieved over socket.  " + e, null,
                       HummerSocketException._INVALID_DATA_FORMAT);
      }
  		/*
  		Check for input types, such as null pointers, and a size that is neither for a new account
		or normal login.
		*/
      _current_protocol.checkBasicInputDataFormat(input);

      _current_protocol = _current_protocol.updateProtocol();

      if(_current_protocol.isNewAccountValidation()) {
		/*
		Check that the data recieved is valid for new account login.	
		*/
        _current_protocol.newAccountCheckInputDataFormat();

		/*
		Verify that the login of the client is valid.
		*/
        _current_protocol.newAccountVerifyInformation();

		/*
		Prepare a Reponse object to validate to the client.
		*/
        output = _current_protocol.newAccountPrepareResponse();
		/*
		Send the Response to the client to verify the server.
		*/
        _object_output_stream = new ObjectOutputStream(_socket.getOutputStream());
        _object_output_stream.writeObject(output);

		/*
		Perform any final things that may be needed for the protocol.
		If these require data to be sent back, then a non-null
		value will be returned, and sent back to the client.
		*/
        if((output = _current_protocol.newAccountFinalize()) != null) {
          _object_output_stream.writeObject(output);
        }
      }
      else {
		/*
		Check that the data recieved is valid for normal login.	
		*/
        _current_protocol.initializedAccountCheckInputDataFormat();

		/*
		Verify that the login of the client is valid.
		*/
        _current_protocol.initializedAccountVerifyInformation();

		/*
		Prepare a Reponse object to validate to the client.
		*/
        output = _current_protocol.initializedAccountPrepareResponse();
		/*
		Send the Response to the client to verify the server.
		*/
        _object_output_stream = new ObjectOutputStream(_socket.getOutputStream());
        _object_output_stream.writeObject(output);
		/*
		Perform any final things that may be needed for the protocol.
		If these require data to be sent back, then a non-null
		value will be returned, and sent back to the client.
		*/
        if((output = _current_protocol.initializedAccountFinalize()) != null) {
          _object_output_stream.writeObject(output);
        }
      }
    }
    catch(HummerSocketException e) {
      throwException(e);
    }
    catch(KeyStoreException e) {
      throwException(e.toString(), null,
                     HummerSocketException._INVALID_LOGIN);
    }
    _has_validated = true;
  }

  public Hashtable getCurrentClientAccessTable() { return _current_protocol.getCurrentClientAccessTable(); }

	/**
	Reads a Response object from the socket and checks the signature before
	returning it.  If the input is a command type Response object, then the command is parsed
	and the next value is read from the socket.
	@throws HummerSocketException When validation() has not been called or the signature is invalid
	*/
  public Response readResponseObject()
         throws HummerSocketException, IOException, OptionalDataException,
                KeyStoreException, UnloadedKeyStoreException, CertificateException, UnknownAliasException {
    if(!_has_validated) {
      throw new HummerSocketException("HummerServerSocket:readResponseObject: " +
                                      "Have not called validation() yet.", null, 0);
    }
    if(_object_input_stream == null) {
      _object_input_stream = new ObjectInputStream(_socket.getInputStream());
    }
    Object input_object = null;
    try {
      input_object = _object_input_stream.readObject();
    }
    catch(java.io.EOFException e) {
       throw new HummerSocketException("HummerServerSocket:readResponseObject: " +
                                       "Client has disconnected prematurely from the server.", null, 0);
    }
    catch(ClassNotFoundException e) {
      throw new HummerSocketException("HummerServerSocket:readResponseObject: " +
                                      "Invalid stream input (could not generate a " +
                                      "valid object from stream)." + e,
                                      null, 0);
    }
    Response input = null, return_value = null;
    try {
      input = _current_protocol.verifyInput(input_object);
    }
    catch(HummerSocketException e) {
      throwException(e);
    }
    catch(NullPointerException e) {
    	 throwException("HummerServerSocket:readResponseObject: Null value recieved from client",
    	                (Response)input_object, HummerSocketException._INVALID_DATA_FORMAT);
    }
    if(input._command != 0) {
      switch(input._command) {

	/* *****************************************
	add aliases
	*******************************************/

        case Response._ADD_ALIAS:
          /*
          Data should be of size 4 and should be:
            data[0] = alias
            data[1] = alias type (from HummerKeyStore)
            data[2] = server is trusted (new server accounts only)
            data[3] = password
          */
          String alias = "";
          byte[] password = null;
          boolean certificates_honored = false;
          int alias_type = -1;
          if(!hasAdminAccess()) {
            throwException("HummerServerSocket:readResponseObject: Access to user " +
                           "accounts can only be by the admin account.",
                           input, HummerSocketException._INVALID_COMMAND);
          }
          if(input._data == null) {
            throwException("HummerServerSocket:readResponseObject: " +
                           "Null data vector recieved.", input,
                           HummerSocketException._INVALID_DATA_FORMAT);
          }
          if(input._data.size() != 4) {
            throwException("HummerServerSocket:readResponseObject: " +
                           "Invalid format of _data Vector in Response object.  " +
                           "Should contain exactly 4 elements for adding aliases.",
                           input, HummerSocketException._INVALID_DATA_FORMAT);
          }
          if(input._data.get(0) == null) {
            throwException("HummerServerSocket:readResponseObject: " +
                           "Null alias recieved for adding alias.",
                           input, HummerSocketException._INVALID_DATA_FORMAT);
          }
          try {
            alias = (String)(input._data.get(0));
          }
          catch(ClassCastException e) {
            throwException("DefualtServerSocket:readResponseObject: " +
                           "Invalid data type recieved for alias for adding alias.  " +
                           "Should be java.lang.String.  Type recieved: " +
                           input._data.get(0).getClass().getName() + ".",
                           input, HummerSocketException._INVALID_DATA_FORMAT);
          }
          if(input._data.get(1) == null) {
            throwException("HummerServerSocket:readResponseObject: " +
                           "Null value for is_server recieved for adding alias.",
                           input, HummerSocketException._INVALID_DATA_FORMAT);
          }
          try {
            alias_type = ((Integer)(input._data.get(1))).intValue();
          }
          catch(ClassCastException e) {
            throwException("DefualtServerSocket:readResponseObject: " +
                           "Invalid data type recieved for alias_type for adding alias.  " +
                           "Should be java.lang.Integer.  Type recieved: " +
                           input._data.get(1).getClass().getName() + ".",
                           input, HummerSocketException._INVALID_DATA_FORMAT);
          }
          if(alias_type == AliasInfo._SERVER_ALIAS_TYPE && input._data.get(2) == null) {
            throwException("HummerServerSocket:readResponseObject: " +
                           "Null value for is_trusted recieved for adding alias.",
                           input, HummerSocketException._INVALID_DATA_FORMAT);
          }
          if(alias_type == AliasInfo._SERVER_ALIAS_TYPE) {
            try {
              certificates_honored = ((Boolean)(input._data.get(2))).booleanValue();
            }
            catch(ClassCastException e) {
              throwException("DefualtServerSocket:readResponseObject: " +
                             "Invalid data type recieved for certificates_honored for adding alias.  " +
                             "Should be java.lang.Boolean.  Type recieved: " +
                             input._data.get(2).getClass().getName() + ".",
                             input, HummerSocketException._INVALID_DATA_FORMAT);
            }
          }
          if(input._data.get(3) == null) {
            throwException("HummerServerSocket:readResponseObject: " +
                           "Null password recieved for adding alias.",
                           input, HummerSocketException._INVALID_DATA_FORMAT);
          }
          try {
            password = (byte[])(input._data.get(3));
          }
          catch(ClassCastException e) {
            throwException("DefualtServerSocket:readResponseObject: " +
                           "Invalid data type recieved for password for add alias.  " + 
                           "Should be java.lang.String.  Type recieved: " +
                           input._data.get(3).getClass().getName() + ".",
                           input, HummerSocketException._INVALID_DATA_FORMAT);
          }
          if(!_key_store.contains(alias)) {
            if(alias_type == AliasInfo._SERVER_ALIAS_TYPE) {

              _key_store.addServerAlias(alias, password,
                                        certificates_honored,
                                        StaticMethods.getDefaultServerAccessTable());
              _key_store.store(true, null);
            }
            else if(alias_type == AliasInfo._USER_ALIAS_TYPE) {
              _key_store.addUserAlias(alias, password,
                                      StaticMethods.getDefaultUserAccessTable());
            }
            else if(alias_type == AliasInfo._PEER_ALIAS_TYPE) {
              _key_store.addPeerAlias(alias, password);
              _key_store.store(true, null);
            }
            else {
              throwException("DefaultServerSocket:readResponseObject: " +
                             "Invalid alias type recieved for adding new alias.",
                             input, HummerSocketException._INVALID_DATA_FORMAT);
            }
          }
          break;

			/*
			Add a manager for the local server.
			Does this by opening a socket to the target machine and then
			validating.  During validation a new account for the
			new manager is created, withthe PublicKey of the
			manager and a HummerCertificate granted by the manager
			is held in the account.  The trust for the manager
			is set to true so that others can login to this system
			with a HummerCertificate granted by the manager.
			Following that, a new HummerCertificate is sent
			to the new manager so that the manager can login to this machine.
			data[0] = PersonalConfig to use for logging into the new manager
			data[1] = Address of the new manager
			*/
        case Response._ADD_MANAGER:
          PersonalConfig manager_config = null;
          String manager_address = null;
          if(!hasAdminAccess()) {
            throwException("HummerServerSocket:readResponseObject: Adding manager " +
                           "accounts can only be by the admin account.",
                           input, HummerSocketException._INVALID_COMMAND);
          }
          if(input._data == null) {
            throwException("HummerServerSocket:readResponseObject: " +
                           "Null data vector recieved.", input,
                           HummerSocketException._INVALID_DATA_FORMAT);
          }
          if(input._data.size() != 2) {
            throwException("HummerServerSocket:readResponseObject: " +
                           "Invalid format of _data Vector in Response " +
                           "object for adding manager alias.  " +
                           "Should contain exactly 2 elements.",
                           input, HummerSocketException._INVALID_DATA_FORMAT);
          }
          if(input._data.get(0) == null) {
            throwException("HummerServerSocket:readResponseObject: " +
                           "Null alias recieved for adding manager alias.",
                           input, HummerSocketException._INVALID_DATA_FORMAT);
          }
          try {
            manager_config = (PersonalConfig)(input._data.get(0));
          }
          catch(ClassCastException e) {
            throwException("DefualtServerSocket:readResponseObject: " +
                           "Invalid data type recieved for adding manager alias.  " +
                           "Should be edu.uidaho.hummer.dataobject.Response.  Type recieved: " +
                           input._data.get(0).getClass().getName() + ".",
                           input, HummerSocketException._INVALID_DATA_FORMAT);
          }
          if(input._data.get(1) == null) {
            throwException("HummerServerSocket:readResponseObject: " +
                           "Null password recieved for adding manager alias.",
                           input, HummerSocketException._INVALID_DATA_FORMAT);
          }
          try {
            manager_address = (String)(input._data.get(1));
          }
          catch(ClassCastException e) {
            throwException("DefualtServerSocket:readResponseObject: " +
                           "Invalid data type recieved for password for adding manager alias.  " + 
                           "Should be java.lang.String.  Type recieved: " +
                           input._data.get(1).getClass().getName() + ".",
                           input, HummerSocketException._INVALID_DATA_FORMAT);
          }
          Hashtable hummer_values = StaticMethods.getHummerPropertyTable(false);
          int config_port = ((Integer)(hummer_values.get("port.CONFIG_PORT"))).intValue();
          InetAddress manager_inet_address = InetAddress.getByName(manager_address);
          manager_address = manager_inet_address.getHostAddress();
          Socket manager_socket = new Socket(manager_inet_address, config_port);
          HummerClientSocket manager_hummer_socket = null;
          if(_key_store.contains(manager_address) && !_key_store.isNewAccount(manager_address)) {
            manager_config.initializeAccount();
          }
          try {
            manager_hummer_socket = new HummerClientSocket(manager_config,
                                                           manager_socket, _key_store);
            manager_hummer_socket.validation(null);
            manager_hummer_socket.addManager();
            manager_hummer_socket.endOfTransmission(null);
          }
          catch(NoSuchAlgorithmException e) {
            throwException("HummerServerSocket:readResponseObject: " + e, input, 0);
          }
          catch(UnrecoverableKeyException e) {
            throwException("HummerServerSocket:readResponseObject: " + e, input, 0);
          }
          catch(HummerSocketException e) {
            System.err.println("HummerServerSocket:readResponseObject: Could not add manager (" +
                               manager_address + "): " + e);
          }
          break;
  

	/*
	Add a peer moderator for the local server.
	Does this by opening a socket to the target machine and then
	validating.  During validation a new account for the
	new moderator is created, with the PublicKey of the
	moderator and a HummerCertificate granted by the moderator
	is held in the account.  The trust for the moderator
	is set to true so that others can login to this system
	with a HummerCertificate granted by the moderator.
	Following that, a new HummerCertificate is sent
	to the new moderator so that the moderator can login to this machine.
	*/
        case Response._ADD_PEER_MODERATOR:
          PersonalConfig moderator_config = null;
          String moderator_address = null;
          if(!hasAdminAccess()) {
            throwException("HummerServerSocket:readResponseObject: Adding moderator " +
                           "accounts can only be by the admin account.",
                           input, HummerSocketException._INVALID_COMMAND);
          }
          if(input._data == null) {
            throwException("HummerServerSocket:readResponseObject: " +
                           "Null data vector recieved (add peer moderator).", input,
                           HummerSocketException._INVALID_DATA_FORMAT);
          }
          if(input._data.size() != 2) {
            throwException("HummerServerSocket:readResponseObject: " +
                           "Invalid format of _data Vector in Response object " +
                           "for add peer moderator alias.  " +
                           "Should contain exactly 2 elements.",
                           input, HummerSocketException._INVALID_DATA_FORMAT);
          }
          if(input._data.get(0) == null) {
            throwException("HummerServerSocket:readResponseObject: " +
                           "Null alias recieved for peer moderator alias.",
                           input, HummerSocketException._INVALID_DATA_FORMAT);
          }
          try {
            moderator_config = (PersonalConfig)(input._data.get(0));
          }
          catch(ClassCastException e) {
            throwException("DefualtServerSocket:readResponseObject: " +
                           "Invalid data type recieved for peer moderator alias.  " +
                           "Should be edu.uidaho.hummer.dataobject.Response.  Type recieved: " +
                           input._data.get(0).getClass().getName() + ".",
                           input, HummerSocketException._INVALID_DATA_FORMAT);
          }
          if(input._data.get(1) == null) {
            throwException("HummerServerSocket:readResponseObject: " +
                           "Null password recieved for peer moderator alias.",
                           input, HummerSocketException._INVALID_DATA_FORMAT);
          }
          try {
            moderator_address = (String)(input._data.get(1));
          }
          catch(ClassCastException e) {
            throwException("DefualtServerSocket:readResponseObject: " +
                           "Invalid data type recieved for password for peer alias moderator alias.  " + 
                           "Should be java.lang.String.  Type recieved: " +
                           input._data.get(1).getClass().getName() + ".",
                           input, HummerSocketException._INVALID_DATA_FORMAT);
          }
          hummer_values = StaticMethods.getHummerPropertyTable(false);
          config_port = ((Integer)(hummer_values.get("port.CONFIG_PORT"))).intValue();
          InetAddress moderator_inet_address = InetAddress.getByName(moderator_address);
          moderator_address = moderator_inet_address.getHostAddress();
          Socket moderator_socket = new Socket(moderator_inet_address, config_port);
          HummerClientSocket moderator_hummer_socket = null;
          if(_key_store.contains(moderator_address) && !_key_store.isNewAccount(moderator_address)) {
            moderator_config.initializeAccount();
          }
          try {
            moderator_hummer_socket = new HummerClientSocket(moderator_config,
                                                             moderator_socket, _key_store);
            moderator_hummer_socket.validation(null);
            moderator_hummer_socket.addPeerModerator();
            moderator_hummer_socket.endOfTransmission(null);
          }
          catch(NoSuchAlgorithmException e) {
            throwException("HummerServerSocket:readResponseObject: " + e, input, 0);
          }
          catch(UnrecoverableKeyException e) {
            throwException("HummerServerSocket:readResponseObject: " + e, input, 0);
          }
          break;
  

  	/* *****************************************
	reset alias
	*******************************************/
        case Response._RESET_ALIAS:
          if(!hasAdminAccess()) {
            throwException("HummerServerSocket:readResponseObject: Deletion to user " +
                           "accounts can only be by the admin account.",
                           input, HummerSocketException._INVALID_COMMAND);
          }
          if(input._data == null) {
            throwException("HummerServerSocket:readResponseObject: " +
                           "Null data vector recieved.", input,
                           HummerSocketException._INVALID_DATA_FORMAT);
          }
          if(input._data.size() != 2) {
            throwException("HummerServerSocket:readResponseObject: " +
                           "Invalid format of _data Vector in Response object for reseting alias.  " +
                           "Should contain exactly 2 elements.",
                           input, HummerSocketException._INVALID_DATA_FORMAT);
          }
          if(input._data.get(0) == null) {
            throwException("HummerServerSocket:readResponseObject: " +
                           "Null alias recieved for reseting alias.", input,
                           HummerSocketException._INVALID_DATA_FORMAT);
          }
          alias = null;
          try {
            alias = (String)(input._data.get(0));
          }
          catch(ClassCastException e) {
            throwException("DefualtServerSocket:readResponseObject: " +
                           "Invalid data type recieved for reseting alias.  Should be java.lang.String.",
                           input, HummerSocketException._INVALID_DATA_FORMAT);
          }
          if(input._data.get(1) == null) {
            throwException("HummerServerSocket:readResponseObject: " +
                           "Null password recieved for reseting alias.",
                           input, HummerSocketException._INVALID_DATA_FORMAT);
          }
          password = null;
          try {
            password = (byte[])(input._data.get(1));
          }
          catch(ClassCastException e) {
            throwException("DefualtServerSocket:readResponseObject: " +
                           "Invalid data type recieved for password for reseting alias.  " + 
                           "Should be java.lang.String.  Type recieved: " +
                           input._data.get(1).getClass().getName() + ".",
                           input, HummerSocketException._INVALID_DATA_FORMAT);
          }
          try {
            _key_store.resetAlias(alias, password);
          }
          catch(UnknownAliasException e) {
            throwException("HummerServerSocket:readResponseObject: " +
                           "Attempted to delete unknown alias.",
                           input, HummerSocketException._UNKNOWN_ALIAS);
          }
          break;


	/* *****************************************
	delete aliases
	*******************************************/
        case Response._DELETE_ALIAS:
          if(!hasAdminAccess()) {
            throwException("HummerServerSocket:readResponseObject: Deletion to user " +
                           "accounts can only be by the admin account.",
                           input, HummerSocketException._INVALID_COMMAND);
          }
          if(input._data == null) {
            throwException("HummerServerSocket:readResponseObject: " +
                           "Null data vector recieved.", input,
                           HummerSocketException._INVALID_DATA_FORMAT);
          }
          if(input._data.size() != 1) {
            throwException("HummerServerSocket:readResponseObject: " +
                           "Invalid format of _data Vector in Response object for deleting alias.  " +
                           "Should contain exactly 1 element.",
                           input, HummerSocketException._INVALID_DATA_FORMAT);
          }
          if(input._data.get(0) == null) {
            throwException("HummerServerSocket:readResponseObject: " +
                           "Null alias recieved for deleting alias.",
                           input, HummerSocketException._INVALID_DATA_FORMAT);
          }
          alias = "";
          try {
            alias = (String)(input._data.get(0));
          }
          catch(ClassCastException e) {
            throwException("DefualtServerSocket:readResponseObject: " +
                           "Invalid data type recieved for deleting alias.  Should be java.lang.String.",
                           input, HummerSocketException._INVALID_DATA_FORMAT);
          }
          if(alias.equals("admin")) {
            throwException("HummerServerSocket:readResponseObject: " +
                           "Attempted to delete \"admin\" alias.", input, HummerSocketException._INVALID_COMMAND);
          }
          try {
            _key_store.removeAlias(alias);
          }
          catch(UnknownAliasException e) {
            throwException("HummerServerSocket:readResponseObject: " +
                           "Attempted to delete unknown alias.",
                           input, HummerSocketException._UNKNOWN_ALIAS);
          }
          break;

	/* ******************************************
	reload key store
	Causes the server to re-read its key store from
	file.
	********************************************/
        case Response._RELOAD_KEY_STORE:
          if(!hasAdminAccess()) {
            throwException("HummerServerSocket:readResponseObject: Reloading key store " +
                           "can only be done by the admin account.",
                           input, HummerSocketException._INVALID_COMMAND);
          }
          if(input != null && input._data.size() != 0) {
            throwException("HummerServerSocket:readResponseObject: " +
                           "Invalid format of _data Vector in Response object.  " +
                           "Should not contain any elements.",
                           input, HummerSocketException._INVALID_DATA_FORMAT);
          }
          try {
            _key_store.load(true, null);
          }
          catch(ClassNotFoundException e) {
            throwException("HummerServerSocket:readResponseObject: " +
                           "Error while loading key store: " + e, null, 0);
          }
                           
          break;

	/* *****************************************
	get aliases
	Places alias info in the _data vector of the Response object
	Each of these objects is an array of objects, where the
	first value in the array is the alias.
	The first value in the vector is a set of headers for the
	meanings of the values in the array.  These values are
	gathered from the HummerKeyStore.getAliasStatistics methods
	*******************************************/

        case Response._GET_ALIASES:
          if(!hasAdminAccess()) {
            throwException("HummerServerSocket:readResponseObject: Access to user " +
                           "accounts can only be by the admin account.",
                           input, HummerSocketException._INVALID_COMMAND);
          }
          if(input != null && input._data.size() != 0) {
            throwException("HummerServerSocket:readResponseObject: " +
                           "Invalid format of _data Vector in Response object.  " +
                           "Should not contain any elements.",
                           input, HummerSocketException._INVALID_DATA_FORMAT);
          }
          Response output = new Response();
          output._type = Response._OK;
          output._data.add(_key_store.getAliasStatisticsHeader());
          for(java.util.Enumeration enum = _key_store.getAliases(); enum.hasMoreElements();) {
            output._data.add(_key_store.getAliasStatistics(enum.nextElement().toString()));
          }
          writeResponseObject(output);
          break;


	/* *****************************************
	get access values
	*******************************************/
        case Response._GET_ACCESS_VALUES:
          Hashtable access_values = null;
          if(!hasAdminAccess()) {
            throwException("HummerServerSocket:readResponseObject: Getting access values " +
                           "can only be by the admin account.",
                           input, HummerSocketException._INVALID_COMMAND);
          }
          if(input._data == null) {
            throwException("HummerServerSocket:readResponseObject: " +
                           "Null data vector recieved.", input, HummerSocketException._INVALID_DATA_FORMAT);
          }
          if(input._data.size() != 1) {
            throwException("HummerServerSocket:readResponseObject: " +
                           "Invalid format of _data Vector in Response object for getting acess values.  " +
                           "Should contain exactly 1 element.",
                           input, HummerSocketException._INVALID_DATA_FORMAT);
          }
          if(input._data.get(0) == null) {
            throwException("HummerServerSocket:readResponseObject: " +
                           "Null alias recieved for getting acccess values.", input,
                           HummerSocketException._INVALID_DATA_FORMAT);
          }
          alias = "";
          try {
            alias = (String)(input._data.get(0));
          }
          catch(ClassCastException e) {
            throwException("DefualtServerSocket:readResponseObject: " +
                           "Invalid data type recieved for alias for getting access values.  " +
                           "Should be java.lang.String.",
                           input, HummerSocketException._INVALID_DATA_FORMAT);
          }
          try {
            access_values = _key_store.getAccessTable(alias);
            output = new Response();
            output._message = access_values;
            writeResponseObject(output);
          }
          catch(UnknownAliasException e) {
            throwException("HummerServerSocket:readResponseObject: " +
                           "Attempted to get information about unknown alias.",
                           input, HummerSocketException._UNKNOWN_ALIAS);
          }
          break;


	/* *****************************************
	set access values
	*******************************************/
        case Response._SET_ACCESS_VALUES:
          access_values = null;
          if(!hasAdminAccess()) {
            throwException("HummerServerSocket:readResponseObject: Setting access values " +
                           "can only be by the admin account.",
                           input, HummerSocketException._INVALID_COMMAND);
          }
          if(input._data == null) {
            throwException("HummerServerSocket:readResponseObject: " +
                           "Null data vector recieved.", input, HummerSocketException._INVALID_DATA_FORMAT);
          }
          if(input._data.size() != 2) {
            throwException("HummerServerSocket:readResponseObject: " +
                           "Invalid format of _data Vector in Response object for getting acess values.  " +
                           "Should contain exactly 2 elements.",
                           input, HummerSocketException._INVALID_DATA_FORMAT);
          }
          if(input._data.get(0) == null) {
            throwException("HummerServerSocket:readResponseObject: " +
                           "Null alias recieved for setting access values.", input,
                           HummerSocketException._INVALID_DATA_FORMAT);
          }
          if(input._data.get(1) == null) {
            throwException("HummerServerSocket:readResponseObject: " +
                           "Null access values (Hashtable) recieved.", input,
                           HummerSocketException._INVALID_DATA_FORMAT);
          }
          alias = "";
          try {
            alias = (String)(input._data.get(0));
          }
          catch(ClassCastException e) {
            throwException("DefualtServerSocket:readResponseObject: " +
                           "Invalid data type recieved for alias for setting access values.  " +
                           "Should be java.lang.String.",
                           input, HummerSocketException._INVALID_DATA_FORMAT);
          }
          Hashtable access_table = null;
          try {
            access_table = (Hashtable)(input._data.get(1));
          }
          catch(ClassCastException e) {
            throwException("DefualtServerSocket:readResponseObject: " +
                           "Invalid data type recieved for access values.  " +
                           "Should be java.util.Hashtable.",
                           input, HummerSocketException._INVALID_DATA_FORMAT);
          }
          try {
            _key_store.setAccessTable(alias, access_table);
          }
          catch(UnknownAliasException e) {
            throwException("HummerServerSocket:readResponseObject: " +
                           "Attempted to access values for unknown alias(" + alias + ").",
                           input, HummerSocketException._UNKNOWN_ALIAS);
          }
          break;




	/* *****************************************
	update public key
	*******************************************/

        case Response._UPDATE_PUBLIC_KEY:
          if(input._data == null) {
            throwException("HummerServerSocket:readResponseObject: " +
                           "Null data vector recieved.", input, HummerSocketException._INVALID_DATA_FORMAT);
          }
          if(input._data.size() != 1) {
            throwException("HummerServerSocket:readResponseObject: " +
                           "Invalid format of _data Vector in Response object.  " +
                           "Should be only 1 element.  Recieved size: " + input._data.size() + ".",
                           input, HummerSocketException._INVALID_DATA_FORMAT);
          }
          PublicKey public_key = null;
          try {
            public_key = (PublicKey)(input._data.get(0));
          }
          catch(ClassCastException e) {
            throwException("HummerServerSocket:readResponseObject: " +
                           "Invalid information sent (ClassCastException).", input,
                           HummerSocketException._INVALID_DATA_FORMAT);
          }
          HummerCertificate certificate;
          try {
            _key_store.setKey(AliasInfo._PUBLIC_KEY_TYPE, _current_protocol.getCurrentPartnerAlias(),
                              public_key);
            certificate = _key_store.generateHummerCertificate(_current_protocol.getCurrentPartnerAlias());
            Response certificate_response = new Response();
            certificate_response._message = certificate;
            writeResponseObject(certificate_response);
          }
          catch(UnknownAliasException e) {
            throwException("HummerServerSocket:readResponseObject: Cannot set PublicKey " +
                           "for an alias that does not exist on this system.", input,
                           HummerSocketException._INVALID_COMMAND);
          }
          break;

	/* *****************************************
	set hummer certificate
	*******************************************/

        case Response._SET_HUMMER_CERTIFICATE:
          if(input._data == null) {
            throwException("HummerServerSocket:readResponseObject: " +
                           "Null data vector recieved.", input, HummerSocketException._INVALID_DATA_FORMAT);
          }
          if(input._data.size() != 1) {
            throwException("HummerServerSocket:readResponseObject: " +
                           "Invalid format of _data Vector in Response object.  " +
                           "Should be exactly 1 element for setting HummerCertificate.  Recieved size: " +
                           input._data.size() + ".",
                           input, HummerSocketException._INVALID_DATA_FORMAT);
          }
          HummerCertificate hummer_certificate = null;
          try {
            hummer_certificate = (HummerCertificate)(input._data.get(0));
          }
          catch(ClassCastException e) {
            throwException("HummerServerSocket:readResponseObject: " +
                           "Invalid information sent (ClassCastException).  " +
                           "Should be HummerCertificate, recieved type: " +
                           input._data.get(0).getClass().getName(), input,
                           HummerSocketException._INVALID_DATA_FORMAT);
          }
          _key_store.setHummerCertificate(_current_protocol.getCurrentPartnerAlias(), hummer_certificate);
          break;

        default:
          throwException("HummerServerSocket:readResponseObject: Invalid command. " +
                         "Only one command can be called for each Response object.",
                         input, HummerSocketException._INVALID_COMMAND);
      }
      return_value = readResponseObject();
    }
    else {
      return_value = input;
    }
    return return_value;
  }

}
